import { jsx, jsxs, Fragment as Fragment$1 } from 'react/jsx-runtime';
import React, { memo, useRef, useState, useEffect, useCallback, useMemo, Fragment, createContext, useImperativeHandle, useContext, forwardRef, Children, cloneElement, createRef, useLayoutEffect, isValidElement } from 'react';
import cometta, { normalize, createStyleSheet } from 'cometta';

function BaseNative({
  /* theme, */
  children
}) {
  return children;
}
var BaseNative$1 = memo(BaseNative);

function createMeta(selector, props, overwrite = true) {
  selector = selector.trim().toLowerCase().startsWith("meta") ? selector : `meta${selector}`;
  let meta = document.querySelector(selector);
  if (!meta || overwrite) {
    meta = meta || document.createElement("meta");
    Object.entries(props).forEach(([attr, val]) => meta.setAttribute(attr, val));
    document.head.prepend(meta);
  }
  return meta;
}

function BaseWeb({
  /* theme, */
  children
}) {
  createMeta("[charset]", { charset: "UTF-8" }, false);
  createMeta('[name="viewport"]', { name: "viewport", content: "width=device-width, initial-scale=1.0" }, false);
  return children;
}
var BaseWeb$1 = memo(BaseWeb);

const canUseDOM = Boolean(typeof window !== "undefined" && window?.document?.createElement);
const canUseNative = typeof navigator !== "undefined" && navigator?.product === "ReactNative";
const native = canUseNative;
const web = !native && canUseDOM;
const server = !native && !web;
const Platform = {
  web,
  native,
  server
};

function useDeferredValue(value, initialValue, delayMs = 500) {
  const timeoutRef = useRef();
  const [state, setState] = useState(initialValue);
  useEffect(() => {
    timeoutRef.current = setTimeout(() => {
      setState(value);
    }, delayMs);
    return () => {
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
    };
  }, [value, delayMs]);
  return state;
}

const rbkGlobal = {
  theme: {},
  mapping: {},
  styles: {}
};

function crypt(str) {
  return crc32(str).toString(16);
}
function crc32(str) {
  if (!rbkGlobal.crcTable) {
    rbkGlobal.crcTable = makeCRCTable();
  }
  let crc = 0 ^ -1;
  for (let i = 0; i < str.length; i++) {
    crc = crc >>> 8 ^ rbkGlobal.crcTable[(crc ^ str.charCodeAt(i)) & 255];
  }
  return (crc ^ -1) >>> 0;
}
function makeCRCTable() {
  const crcTable = [];
  let c;
  for (let n = 0; n < 256; n++) {
    c = n;
    for (let k = 0; k < 8; k++) {
      c = c & 1 ? 3988292384 ^ c >>> 1 : c >>> 1;
    }
    crcTable[n] = c;
  }
  return crcTable;
}

function uuid() {
  let dt = (/* @__PURE__ */ new Date()).getTime();
  return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function(c) {
    const r = (dt + Math.random() * 16) % 16 | 0;
    dt = Math.floor(dt / 16);
    return (c == "x" ? r : r & 3 | 8).toString(16);
  });
}

function useHtmlId(id) {
  return useRef(id ?? `rbk-id-${crypt(uuid())}`).current;
}

const store = {};
const listeners = {};
function useGlobalState(key, initialState) {
  const listenerId = useHtmlId();
  const [state, _setState] = useState(store[key] ?? initialState);
  const setState = useCallback(
    (value) => {
      const next = value instanceof Function ? value(store[key]) : value;
      Object.values(listeners[key]).forEach((listener) => listener(next));
      store[key] = next;
    },
    [key]
  );
  useMemo(() => {
    store[key] = store[key] ?? initialState;
    listeners[key] = listeners[key] ?? {};
  }, [key]);
  useMemo(() => {
    listeners[key][listenerId] = _setState;
  }, [key, listenerId]);
  useEffect(() => {
    return () => {
      delete listeners[key][listenerId];
    };
  }, [key, listenerId]);
  return [state, setState];
}

function usePortals() {
  return useGlobalState("rbk-portals", {});
}

function Portal() {
  const [portals] = usePortals();
  const portalsDeferred = useDeferredValue(portals, {}, 0);
  return /* @__PURE__ */ jsx(Fragment, { children: Object.entries(portalsDeferred).map(([key, children]) => /* @__PURE__ */ jsx(Fragment, { children }, key)) });
}

const RbkContext = createContext(null);

function extract(props, ...mixin) {
  let extracted = {};
  if (!Array.isArray(props)) {
    props = [props];
  }
  if (!Array.isArray(mixin)) {
    mixin = [];
  }
  for (const item of mixin) {
    if (!item)
      continue;
    if (Array.isArray(item)) {
      extracted = { ...extracted, ...extract(props, ...item) };
      continue;
    }
    for (const prop of props) {
      if (!(prop in item)) {
        continue;
      }
      extracted[prop] = item[prop];
      delete item[prop];
    }
  }
  return extracted;
}

function omit(props, ...mixin) {
  let result = {};
  if (!Array.isArray(props)) {
    props = [props];
  }
  for (const item of mixin) {
    if (!item)
      continue;
    if (Array.isArray(item)) {
      result = { ...result, ...omit(props, ...item) };
      continue;
    }
    for (const prop in item) {
      if (!props.includes(prop)) {
        result[prop] = item[prop];
      }
    }
  }
  return result;
}

const boxSizeProps = [
  "width",
  "height",
  "maxWidth",
  "maxHeight",
  // Aliases
  "w",
  "h",
  "maxw",
  "maxh",
  "minw",
  "minh",
  "ww",
  "hh"
];
const flexContainerAlignProps = [
  "placeContent",
  "placeItems",
  "alignContent",
  "alignItems",
  "justifyContent",
  "justifyItems",
  // Aliases
  "center"
];
const flexChildAlignProps = [
  "placeSelf",
  "alignSelf",
  "justifySelf",
  // Aliases
  "align",
  "justify"
];
const flexAlignProps = [...flexContainerAlignProps, ...flexChildAlignProps];
const flexContainerProps = [
  "flexDirection",
  "flexFlow",
  "flexWrap",
  ...flexContainerAlignProps,
  // Aliases
  "direction",
  "row",
  "column",
  "reverse",
  "wrap",
  "noWrap"
];
const flexChildProps = [
  "flex",
  "flexBasis",
  "flexGrow",
  "flexShrink",
  "order",
  ...flexChildAlignProps,
  // Aliases
  "grow",
  "shrink",
  "basis"
];
const flexProps = [...flexAlignProps, ...flexContainerProps, ...flexChildProps];
const customSpacings = [
  "i",
  "t",
  "b",
  "l",
  "r",
  "m",
  "mt",
  "mb",
  "ml",
  "mr",
  "mh",
  "mx",
  "mv",
  "my",
  "p",
  "pt",
  "pb",
  "pl",
  "pr",
  "ph",
  "px",
  "pv",
  "py",
  "g",
  "gx",
  "gy"
];
const customStyleProps = [
  "bg",
  "border",
  "borderBottom",
  "borderLeft",
  "borderRight",
  "borderTop",
  "corners",
  // borderRadius * spacing
  "direction",
  "h",
  // height
  "lh",
  // line height
  "ls",
  // line spacing
  "maxh",
  "maxw",
  "minh",
  "minw",
  "position",
  "shadow",
  // box-shadow
  "w",
  // width
  ...customSpacings
];
const gapSpacings = ["gap", "rowGap", "columnGap"];
const spacings = [
  "position",
  "top",
  "bottom",
  "left",
  "right",
  "margin",
  "marginTop",
  "marginBottom",
  "marginLeft",
  "marginRight",
  "marginHorizontal",
  "marginVertical",
  "padding",
  "paddingTop",
  "paddingBottom",
  "paddingLeft",
  "paddingRight",
  "paddingHorizontal",
  "paddingVertical",
  // Gaps
  ...gapSpacings,
  // Aliases
  ...customSpacings
];
const transformProps = [
  "matrix",
  "perspective",
  "rotate",
  "rotateX",
  "rotateY",
  "rotateZ",
  "scale",
  "scaleX",
  "scaleY",
  "translateX",
  "translateY",
  "skewX",
  "skewY"
];
const notPxProps = [
  "animationIterationCount",
  "aspectRatio",
  "borderImageOutset",
  "borderImageSlice",
  "borderImageWidth",
  "boxFlex",
  "boxFlexGroup",
  "boxOrdinalGroup",
  "columnCount",
  "columns",
  "fillOpacity",
  "flex",
  "flexGrow",
  "flexNegative",
  "flexOrder",
  "flexPositive",
  "flexShrink",
  "floodOpacity",
  "fontWeight",
  "gridArea",
  "gridColumn",
  "gridColumnEnd",
  "gridColumnSpan",
  "gridColumnStart",
  "gridRow",
  "gridRowEnd",
  "gridRowSpan",
  "gridRowStart",
  "lineClamp",
  "lineHeight",
  "opacity",
  "order",
  "orphans",
  "rotate",
  "scale",
  "stopOpacity",
  "strokeDasharray",
  "strokeDashoffset",
  "strokeMiterlimit",
  "strokeOpacity",
  "strokeWidth",
  "tabSize",
  "transform",
  "widows",
  "zIndex",
  "zoom"
];
const styleProps = [
  ...boxSizeProps,
  ...flexProps,
  ...spacings,
  ...customStyleProps,
  "aspectRatio",
  "backfaceVisibility",
  "backgroundColor",
  "borderBottomColor",
  "borderBottomEndRadius",
  "borderBottomLeftRadius",
  "borderBottomRightRadius",
  "borderBottomStartRadius",
  "borderBottomWidth",
  "borderColor",
  "borderEndColor",
  "borderEndWidth",
  "borderLeftColor",
  "borderLeftWidth",
  "borderRadius",
  "borderRightColor",
  "borderRightWidth",
  "borderStartColor",
  "borderStartWidth",
  "borderStyle",
  "borderTopColor",
  "borderTopEndRadius",
  "borderTopLeftRadius",
  "borderTopRightRadius",
  "borderTopStartRadius",
  "borderTopWidth",
  "borderWidth",
  "boxShadow",
  "color",
  "direction",
  "display",
  "fontFamily",
  "fontSize",
  "fontStyle",
  "fontVariant",
  "fontWeight",
  "letterSpacing",
  "lineHeight",
  "opacity",
  "overflow",
  "pointerEvents",
  "textAlign",
  "textTransform",
  "transform",
  "zIndex"
];
const aliases = {
  w: "width",
  h: "height",
  maxw: "maxWidth",
  maxh: "maxHeight",
  minw: "minWidth",
  minh: "minHeight",
  lh: "lineHeight",
  ls: "letterSpacing",
  bg: "backgroundColor",
  // Flex Container
  direction: "flexDirection",
  // Flex Item
  grow: "flexGrow",
  shrink: "flexShrink",
  basis: "flexBasis",
  align: "alignSelf",
  justify: "justifySelf"
};

function css(...mixin) {
  return cometta.css(...mixin);
}

function jss(...mixin) {
  return cometta.jss(...mixin);
}

function isObject(item) {
  return typeof item === "object" && Boolean(item) && !Array.isArray(item);
}
function transform(value) {
  const parsed = {};
  if (typeof value === "string") {
    value.split(/\s/g).forEach((item) => {
      const match = item.trim().match(/(.*)\((.*)\)(\[([^\]]*)\])?/);
      if (match?.[2]) {
        parsed[match?.[1]] = match?.[2];
      }
    });
  }
  if (Array.isArray(value)) {
    value.forEach((item) => {
      if (isObject(item)) {
        Object.entries(item).forEach(([attr, val]) => {
          parsed[attr] = val;
        });
      }
    });
  }
  if (isObject(value)) {
    Object.entries(value).forEach(([attr, val]) => {
      parsed[attr] = val;
    });
  }
  Object.entries(parsed).forEach(([attr, val]) => {
    parsed[attr] = isNaN(val) ? val : Number(val);
  });
  return jss(parsed);
}

function clone(mixin) {
  const result = Array.isArray(mixin) ? [] : {};
  for (const key in mixin) {
    let value = mixin[key];
    if (value && typeof value === "object") {
      value = clone(value);
    }
    result[key] = value;
  }
  return result;
}

function defined(value) {
  if (value === null) {
    return false;
  }
  if (typeof value === "undefined") {
    return false;
  }
  if (typeof value === "number" && isNaN(value)) {
    return false;
  }
  return true;
}

function pick(option, defaultOption, options = {}) {
  return options?.[option] ?? options?.[defaultOption];
}

function useDefaultRef(ref, initialValue) {
  const defaultRef = useRef(initialValue);
  useImperativeHandle(ref, () => defaultRef.current, [defaultRef]);
  return defaultRef;
}

function jssWithTransform(style) {
  const resolved = jss(style);
  return omit("transform", { ...resolved, ...transform(resolved?.transform) });
}
function useAnimation(style, ref) {
  const { web, native } = rbkGlobal.mapping;
  const baseStyle = useMemo(() => style ? jssWithTransform(style) : {}, [style]);
  const elRef = useDefaultRef(ref);
  const animationName = useHtmlId();
  const styleRef = useRef(clone(baseStyle));
  const transformRef = useRef({});
  const runIdRef = useRef();
  const timeoutRef = useRef();
  const setStyle = useCallback(
    (attr, value, unit = null) => {
      const isTransforming = transformProps.includes(attr);
      let prop = attr;
      if (web) {
        if (defined(value)) {
          if (!unit && !notPxProps.includes(attr)) {
            unit = "px";
          }
          if (unit) {
            value = `${value}${unit}`;
          }
        }
      }
      if (native) {
        if (defined(value)) {
          if (unit === "px") {
            unit = null;
          }
          if (unit) {
            value = `${value}${unit}`;
          }
        }
      }
      if (isTransforming) {
        prop = "transform";
        styleRef.current[attr] = value;
        if (web) {
          transformRef.current[attr] = `${attr}(${value})`;
          value = Object.values(transformRef.current).join(" ");
        }
        if (native) {
          transformRef.current[attr] = { [attr]: value };
          value = Object.values(transformRef.current);
        }
      }
      requestAnimationFrame(() => {
        if (!elRef.current)
          return;
        if (web) {
          elRef.current.style[prop] = value;
        }
        if (native) {
          elRef.current.setNativeProps({ [prop]: value });
        }
      });
      if (!isTransforming) {
        styleRef.current[attr] = value;
      }
    },
    [elRef, native, web]
  );
  const resolveValue = useCallback((value) => {
    const match = `${value}`.match(/^(-?[\d.]+)([a-z%]*)/i);
    return [parseFloat(match?.[1] ?? "0"), match?.[2]?.trim()?.toLowerCase() || null];
  }, []);
  const stop = useCallback(() => {
    if (web && elRef.current) {
      elRef.current.style.animationPlayState = "paused";
    }
    runIdRef.current = void 0;
    if (timeoutRef.current) {
      clearTimeout(timeoutRef.current);
    }
    timeoutRef.current = null;
  }, [elRef, runIdRef, timeoutRef, web]);
  const reset = useCallback(() => {
    stop();
    if (web && elRef.current) {
      elRef.current.style.animation = "";
    }
    Object.keys(styleRef.current).forEach((attr) => {
      setStyle(attr, baseStyle?.[attr] ?? void 0);
    });
  }, [baseStyle, elRef, setStyle, stop, web]);
  const start = useCallback(
    async (options) => {
      return new Promise((resolve) => {
        stop();
        let {
          to,
          from = styleRef.current,
          boomerang = false,
          delay = 0,
          duration = 250,
          iterations = 1,
          throttle = 0,
          timing = "linear",
          web_useRawStyle = false
        } = options;
        from = jssWithTransform(from);
        to = jssWithTransform(to);
        const lastState = clone(to);
        const meta = Object.fromEntries(
          Object.keys(to).map((attr) => {
            const [fromValue, fromUnit] = resolveValue(from?.[attr] || 0);
            const [toValue, toUnit] = resolveValue(to[attr]);
            return [
              attr,
              {
                from: fromValue,
                to: toValue,
                unit: toUnit || fromUnit
              }
            ];
          })
        );
        if (web && !web_useRawStyle) {
          const transformFromArr = Object.entries(extract([...transformProps], from)).map(([attr, value]) => ({
            [attr]: value
          }));
          const transformToArr = Object.entries(extract([...transformProps], to)).map(([attr, value]) => ({
            [attr]: value
          }));
          cometta.createStyleSheet(
            `
             @keyframes ${animationName} {
               from { ${css(from, transformFromArr.length > 0 && { transform: transformFromArr })} }
               to { ${css(to, transformToArr.length > 0 && { transform: transformToArr })} }
             }
          `,
            { uniqueId: animationName }
          );
          const direction = boomerang ? "alternate" : "normal";
          iterations = iterations === -1 ? "infinite" : iterations;
          iterations = typeof iterations === "number" && boomerang ? iterations * 2 : iterations;
          Object.keys(to).forEach((attr) => {
            setStyle(attr, meta[attr].from, meta[attr].unit);
          });
          if (elRef.current) {
            elRef.current.addEventListener(
              "animationend",
              () => {
                Object.keys(lastState).forEach((attr) => {
                  if (boomerang) {
                    setStyle(attr, meta[attr].from, meta[attr].unit);
                  } else {
                    setStyle(attr, meta[attr].to, meta[attr].unit);
                  }
                });
                elRef.current.style.animation = "";
                setTimeout(resolve, 10);
              },
              { once: true }
            );
            elRef.current.style.animation = `${animationName} ${duration}ms ${timing} ${delay || 0}ms ${iterations} ${direction}`;
          }
        }
        if (native || web_useRawStyle) {
          const runId = uuid();
          runIdRef.current = runId;
          const timingFn = pick(timing, "ease", {
            linear: easeLinear,
            ease: easeInSine,
            "ease-in": easeInQuad,
            "ease-out": easeOutQuad,
            "ease-in-out": easeInOutQuad
          });
          iterations = iterations === -1 ? "infinite" : iterations;
          iterations = iterations === "infinite" ? Number.POSITIVE_INFINITY : Number(iterations) * (boomerang ? 2 : 1);
          timeoutRef.current = setTimeout(
            async () => {
              for (let index = 0; index < Number(iterations); index++) {
                if (runId !== runIdRef.current) {
                  break;
                }
                const isBackward = boomerang && Number(index) % 2 !== 0;
                await new Promise((resolveStep) => {
                  const startAt = Date.now();
                  const endAt = startAt + duration;
                  const apply = async () => {
                    if (endAt < Date.now() || runId !== runIdRef.current) {
                      return resolveStep(true);
                    }
                    const pos = duration - (endAt - Date.now());
                    Object.keys(to).forEach((attr) => {
                      const [startValue] = isBackward ? resolveValue(to[attr]) : resolveValue(from[attr]);
                      const [endValue] = isBackward ? resolveValue(from[attr]) : resolveValue(to[attr]);
                      const newValue = timingFn(pos, startValue, endValue - startValue, duration);
                      setStyle(attr, newValue, meta[attr].unit);
                    });
                    timeoutRef.current = setTimeout(apply, throttle);
                  };
                  timeoutRef.current = setTimeout(apply, 0);
                });
                if (runId === runIdRef.current) {
                  Object.keys(lastState).forEach((attr) => {
                    if (isBackward) {
                      setStyle(attr, meta[attr].from, meta[attr].unit);
                    } else {
                      setStyle(attr, meta[attr].to, meta[attr].unit);
                    }
                  });
                }
              }
              setTimeout(resolve, 10);
            },
            Math.max(delay || 0, 0)
          );
        }
      });
    },
    [animationName, elRef, native, resolveValue, setStyle, stop, web]
  );
  useEffect(() => {
    return () => {
      stop();
    };
  }, [stop]);
  return useMemo(
    () => ({
      start,
      stop,
      reset,
      props: { ref: elRef, style }
    }),
    [elRef, reset, start, stop, style]
  );
}
function easeLinear(currentIteration, startValue, changeInValue, totalIterations) {
  return changeInValue * currentIteration / totalIterations + startValue;
}
function easeInQuad(currentIteration, startValue, changeInValue, totalIterations) {
  return changeInValue * (currentIteration /= totalIterations) * currentIteration + startValue;
}
function easeOutQuad(currentIteration, startValue, changeInValue, totalIterations) {
  return -changeInValue * (currentIteration /= totalIterations) * (currentIteration - 2) + startValue;
}
function easeInOutQuad(currentIteration, startValue, changeInValue, totalIterations) {
  if ((currentIteration /= totalIterations / 2) < 1) {
    return changeInValue / 2 * currentIteration * currentIteration + startValue;
  }
  return -changeInValue / 2 * (--currentIteration * (currentIteration - 2) - 1) + startValue;
}
function easeInSine(currentIteration, startValue, changeInValue, totalIterations) {
  return changeInValue * (1 - Math.cos(currentIteration / totalIterations * (Math.PI / 2))) + startValue;
}

function registry(theme) {
  theme = theme ?? rbkGlobal.theme ?? {};
  if (theme) {
    if (Platform.web) {
      cometta.polyfill({
        fontSize: theme.rem(1)
      });
    }
    if (Platform.native) {
      cometta.polyfill({
        fontSize: theme.rem(1),
        screenWidth: () => rbkGlobal.mapping.dimensions.window().width,
        screenHeight: () => rbkGlobal.mapping.dimensions.window().height
      });
    }
    cometta.unit("gap", (value) => {
      return value * theme.shape.gap * theme.shape.spacing;
    });
    const vars = Object.assign(
      {},
      ...Object.entries(theme.colors).map(
        ([name, variants]) => Object.entries(variants).map(([variant, color]) => {
          const result = {
            [`${name}-${variant}`]: color
          };
          if (variant === "main") {
            result[`${name}`] = color;
          }
          return result;
        })
      ).flat()
    );
    cometta.variables(vars);
  }
  cometta.parser("web", (value) => Platform.web ? value : {});
  cometta.parser("native", (value) => Platform.native ? value : {});
  cometta.parser("ww", (value) => ({
    width: value,
    minWidth: value,
    maxWidth: value
  }));
  cometta.parser("hh", (value) => ({
    height: value,
    minHeight: value,
    maxHeight: value
  }));
  cometta.parser("corners", (value) => {
    if (!defined(value))
      return;
    return {
      borderRadius: (theme?.shape?.borderRadius ?? 0) * value
    };
  });
  cometta.parser("shadow", (value) => {
    if (!defined(value))
      return;
    const radius = Math.round(value + (value ? 2 : 0));
    const offset = Math.round(value / 2);
    if (Platform.web) {
      return {
        boxShadow: `${offset}px ${offset}px ${radius}px rgba(0, 0, 0, ${value ? 0.15 : 0})`
      };
    }
    if (Platform.native) {
      return {
        elevation: Math.round(value),
        shadowColor: "rgba(0, 0, 0)",
        shadowOpacity: 0.1,
        shadowRadius: radius,
        shadowOffset: {
          width: offset,
          height: offset
        }
      };
    }
  });
  cometta.parser("transform", (value) => {
    if (!defined(value))
      return;
    const parsed = transform(value);
    return {
      transform: Object.entries(parsed).map(([attr, val]) => {
        if (typeof val === "number" && !notPxProps.includes(attr)) {
          val = `${val}px`;
        }
        return `${attr}(${val})`;
      }).join(" ")
    };
  });
  cometta.parser("inset", (value) => {
    if (!defined(value))
      return;
    if (Platform.native) {
      const [v1, v2, v3, v4] = `${value ?? ""}`.trim().split(/\s/g).filter((item) => item.trim());
      return {
        top: parseUnit(v1),
        right: parseUnit(v2 ?? v1),
        bottom: parseUnit(v3 ?? v1),
        left: parseUnit(v4 ?? v2 ?? v1)
      };
    }
    return { inset: value };
  });
  cometta.parser("*", (value, prop) => {
    if (typeof value === "undefined")
      return;
    const prevProp = prop;
    const prevValue = value;
    if (value instanceof Function) {
      value = value(theme);
    }
    if (prop.toLowerCase().includes("color") && theme) {
      value = theme.color(value) ?? value;
    }
    if (boxSizeProps.includes(prop) && value === true) {
      value = "100%";
    }
    if (["g", "gx", "gy"].includes(prop)) {
      prop = prop === "gy" ? "rowGap" : prop === "gx" ? "columnGap" : "gap";
      if (value === true) {
        value = 1;
      }
      if (typeof value === "number" && theme) {
        value = `${value}gap`;
      }
    }
    if (customSpacings.includes(prop)) {
      prop = prop.replace(/^i$/, "inset").replace(/^(.)t$/, "$1Top").replace(/^(.)b$/, "$1Bottom").replace(/^(.)l$/, "$1Left").replace(/^(.)r$/, "$1Right").replace(/^(.)h$/, "$1Horizontal").replace(/^(.)v$/, "$1Vertical").replace(/^(.)x$/, "$1Horizontal").replace(/^(.)y$/, "$1Vertical").replace(/^m/, "margin").replace(/^p/, "padding").replace(/^t$/, "top").replace(/^b$/, "bottom").replace(/^l$/, "left").replace(/^r$/, "right");
      if (value === true) {
        value = 1;
      }
      if (typeof value === "number" && theme) {
        value = theme.spacing(value);
      }
    }
    if (flexAlignProps.includes(prop)) {
      value = parseFlexAlign(value);
    }
    prop = aliases[prop] ?? prop;
    if (Platform.native) {
      const valueSplit = `${value ?? ""}`.trim().split(/\s/g).filter((item) => item.trim());
      ["Content", "Items", "Self"].forEach((suffix) => {
        if (prop === "place" + suffix) {
          const [v1, v2] = valueSplit;
          return {
            [`align${suffix}`]: parseFlexAlign(v1),
            [`justify${suffix}`]: parseFlexAlign(v2 ?? v1)
          };
        }
      });
    }
    if (prop !== prevProp || value !== prevValue) {
      return cometta.jss({ [prop]: value });
    }
  });
}
const parseUnit = (value) => {
  return isNaN(value) ? value : Number(value);
};
const parseFlexAlign = (value) => {
  return value.replace(/^normal$/, "stretch").replace(/^start$/, "flex-start").replace(/^end$/, "flex-end").replace(/^space$/, "space-between").replace(/^between$/, "space-between").replace(/^around$/, "space-around").replace(/^evenly$/, "space-evenly");
};

function sheet(...mixin) {
  return cometta.sheet(...mixin);
}

const isProd = typeof process !== "undefined" && ["prod", "production"].includes(process?.env?.NODE_ENV);
const stdout = {
  log(...args) {
    !isProd && console.log("React Bulk:", ...args);
  },
  error(...args) {
    !isProd && console.error("React Bulk:", ...args);
  },
  warn(...args) {
    !isProd && console.warn("React Bulk:", ...args);
  }
};

function string(value) {
  if ([null, void 0].includes(value)) {
    return "";
  }
  if (!Number.isNaN(Number(value))) {
    return value.toString();
  }
  return `${value || ""}`;
}

const light = {
  mode: "light",
  colors: {
    text: {
      primary: "#232323",
      secondary: "#666666",
      tertiary: "#999999",
      disabled: "#CCCCCC"
    },
    background: {
      primary: "#FFFFFF",
      secondary: "#F2F2F2",
      tertiary: "#FAFAFA",
      disabled: "#616161"
    }
  }
};

const base$1 = {
  custom: {},
  shape: {
    borderRadius: 6,
    spacing: 4,
    gap: 4
    // gap is generally multiplied by spacing
  },
  typography: {
    fontSize: 16,
    lineHeight: 1.15
  },
  colors: {
    primary: "#7E57C2",
    secondary: "#C0CA33",
    info: "#1E88E5",
    success: "#43A047",
    warning: "#F39C12",
    error: "#E53935",
    amber: "#FFAB00",
    blue: "#2962FF",
    blueGray: "#455A64",
    brown: "#5D4037",
    cyan: "#00B8D4",
    deepOrange: "#DD2C00",
    deepPurple: "#6200EA",
    fuchsia: "#D946EF",
    gray: "#6b7280",
    green: "#00C853",
    indigo: "#304FFE",
    lime: "#AEEA00",
    orange: "#FF6D00",
    pink: "#C51162",
    purple: "#AA00FF",
    red: "#D50000",
    teal: "#00BFA5",
    violet: "#8B5CF6",
    yellow: "#FFD600",
    common: {
      transparent: "#00000000",
      trans: "#00000000",
      black: "#000000",
      white: "#FFFFFF"
    },
    text: light.colors.text,
    background: light.colors.background
  },
  breakpoints: {
    xs: 0,
    sm: 320,
    md: 768,
    lg: 992,
    xl: 1200,
    xxl: 1400
  },
  mixins: {
    transitions: {
      slow: {
        transitionProperty: "all",
        transitionDuration: "0.5s",
        transitionTimingFunction: "ease"
      },
      medium: {
        transitionProperty: "all",
        transitionDuration: "0.35s",
        transitionTimingFunction: "ease"
      },
      fast: {
        transitionProperty: "all",
        transitionDuration: "0.2s",
        transitionTimingFunction: "ease"
      }
    },
    scroll: {
      "&::-webkit-scrollbar": { height: "0.375rem", width: "0.375rem" },
      "&::-webkit-scrollbar-track": { background: "#0000000D" },
      "&::-webkit-scrollbar-corner": { background: "#00000000" },
      "&::-webkit-scrollbar-thumb": { bg: "text.disabled", borderRadius: "0.1875rem" }
    },
    // Global CSS (for WEB only)
    globalCSS: "",
    zIndex: {
      backdrop: 900,
      modal: 901,
      dropdown: 902,
      tooltip: 903,
      toaster: 904
    }
  },
  rem(multiplier = 1, base2 = null) {
    return (base2 || this.typography.fontSize) * multiplier;
  },
  spacing(multiplier = 1) {
    return this.shape.spacing * multiplier;
  },
  color(mixin, alpha, luminosity) {
    const [color, variation = "main", opacity] = `${mixin || ""}`.split(".");
    let newColor = this?.colors[color]?.[variation] ?? this?.colors[color]?.primary ?? this?.colors[color] ?? this?.colors?.common?.[color] ?? mixin;
    if (string(newColor).startsWith("rgb")) {
      newColor = "#" + newColor.replace(/^rgba?\(|\s+|\)$/g, "").split(",").map((string2) => parseFloat(string2)).map((number, index) => index === 3 ? Math.round(number * 255) : number).map((number) => number.toString(16)).map((string2) => string2.length === 1 ? "0" + string2 : string2).join("");
    }
    if (string(newColor).startsWith("#")) {
      if (newColor.length < 7) {
        newColor = `#${newColor[1]}${newColor[1]}${newColor[2]}${newColor[2]}${newColor[3]}${newColor[3]}`;
      }
    }
    if (defined(luminosity)) {
      if (!string(newColor).startsWith("#")) {
        stdout.warn(`theme.color() cannot apply luminosity on color "${mixin}".`);
      } else {
        if (typeof luminosity === "string") {
          luminosity = Number(luminosity.replace("%", ""));
        }
        if (typeof luminosity === "number") {
          const digits = newColor.replace(/[^0-9A-Fa-f]/gi, "");
          const decimalColor = parseInt(digits, 16);
          let r = (decimalColor >> 16) + luminosity;
          r > 255 && (r = 255);
          r < 0 && (r = 0);
          let g = (decimalColor & 255) + luminosity;
          g > 255 && (g = 255);
          g < 0 && (g = 0);
          let b = (decimalColor >> 8 & 255) + luminosity;
          b > 255 && (b = 255);
          b < 0 && (b = 0);
          newColor = `#${(g | b << 8 | r << 16).toString(16).padStart(6, "0")}`;
        }
      }
    }
    if (!defined(alpha) && opacity) {
      alpha = Number(opacity) / 100;
    }
    if (defined(alpha)) {
      if (!string(newColor).startsWith("#")) {
        stdout.warn(`theme.color() cannot apply luminosity on color "${mixin}".`);
      } else {
        if (typeof alpha === "string") {
          alpha = Number(alpha.replace("%", "")) / 100;
        }
        if (typeof alpha === "number") {
          newColor = `${newColor}${Math.floor(alpha * 255).toString(16).padStart(2, "0")}`;
        }
      }
    }
    return newColor;
  },
  contrast(mixin, lightColor, darkColor) {
    let color = this.color(mixin);
    lightColor = this.color(lightColor || this.colors.common.white);
    darkColor = this.color(darkColor || this.colors.common.black);
    if (color === "transparent") {
      return darkColor;
    }
    if (!string(color).startsWith("#")) {
      stdout.warn(`theme.contrast() cannot check color "${mixin}".`);
      return darkColor;
    }
    color = color.replace("#", "");
    const r = parseInt(color.substr(0, 2), 16);
    const g = parseInt(color.substr(2, 2), 16);
    const b = parseInt(color.substr(4, 2), 16);
    const yiq = (r * 299 + g * 587 + b * 114) / 1e3;
    return yiq >= 128 ? darkColor : lightColor;
  },
  // Warn user about ReactBulk context
  // @ts-ignore
  setTheme() {
    stdout.error("setTheme() only works when the app is wrapped in the <ReactBulk /> context.");
  },
  get components() {
    return {
      Avatar: {
        name: "rbk-avatar",
        defaultProps: {
          alt: "Avatar",
          color: "gray",
          size: 64
        },
        defaultStyles: {
          root: {
            position: "relative",
            aspectRatio: "1/1",
            borderRadius: 32
          },
          content: {
            position: "absolute",
            inset: 0,
            bg: "gray",
            borderRadius: 32,
            overflow: "hidden"
          }
        },
        variants: {}
      },
      Backdrop: {
        name: "rbk-backdrop",
        defaultProps: {
          platform: {
            web: {
              accessibility: { role: "dialog" }
            }
          }
        },
        defaultStyles: {
          root: {
            position: "absolute",
            inset: 0,
            height: "100%",
            width: "100%",
            backgroundColor: "rgba(0, 0, 0, 0.5)"
          }
        },
        variants: {}
      },
      Badge: {
        name: "rbk-badge",
        defaultProps: {
          color: "secondary",
          size: "medium"
        },
        defaultStyles: {
          root: {
            display: "flex",
            alignItems: "center",
            alignContent: "center",
            justifyContent: "center",
            textAlign: "center",
            alignSelf: "center",
            justifySelf: "center",
            backgroundColor: "secondary",
            borderRadius: "0.5625rem",
            overflow: "hidden",
            minHeight: "1.125rem",
            minWidth: "1.125rem",
            paddingVertical: "0.25rem",
            paddingHorizontal: "0.375rem"
          },
          label: {
            color: "white",
            fontSize: "0.5625rem",
            fontWeight: "bold",
            web: { lineHeight: 1 }
          }
        },
        variants: {
          dot: {
            true: {
              root: {
                minWidth: 0,
                minHeight: 0,
                px: 1
              }
            }
          }
        }
      },
      Box: {
        name: "rbk-box",
        defaultProps: {},
        defaultStyles: {
          root: {
            display: "flex",
            flexDirection: "column",
            flexWrap: "nowrap",
            flexShrink: 0,
            alignContent: "flex-start",
            justifyContent: "flex-start",
            alignItems: "stretch",
            web: { "-webkit-tap-highlight-color": "#00000000" }
          }
        },
        variants: {}
      },
      Button: {
        name: "rbk-button",
        defaultProps: {
          accessibility: { role: "button" },
          color: "primary",
          disabledColor: "gray.lighter",
          size: "medium",
          transform: "uppercase",
          variant: "solid"
        },
        defaultStyles: {
          root: {
            position: "relative",
            display: "flex",
            flexDirection: "row",
            flexWrap: "nowrap",
            alignItems: "center",
            justifyContent: "center",
            bg: "primary",
            border: "1px solid primary",
            borderRadius: (theme) => theme.shape.borderRadius,
            margin: 0,
            minHeight: "2.25rem",
            minWidth: "2.25rem",
            padding: 0,
            paddingHorizontal: "0.75rem",
            web: {
              backgroundImage: "none",
              boxShadow: "none",
              cursor: "pointer",
              "-webkit-user-select": "none",
              "-ms-user-select": "none",
              "user-select": "none",
              outline: "0 !important",
              textDecorationLine: "none !important",
              ...this.mixins.transitions.fast,
              transitionProperty: "background-color, box-shadow",
              "&:hover": { bg: (theme) => theme.color("primary", 0.8) }
            }
          },
          label: {
            color: "white",
            fontSize: "0.75rem",
            fontWeight: "bold",
            letterSpacing: 1,
            textAlign: "center",
            web: {
              lineHeight: "0.75rem"
            }
          }
        },
        variants: {
          block: {
            true: { root: { width: "100%" } }
          },
          circular: {
            true: { root: { borderRadius: "1rem" } }
          },
          disabled: {
            true: {
              root: {
                opacity: 0.75,
                web: {
                  cursor: "not-allowed !important",
                  "& *": { cursor: "not-allowed !important" }
                }
              }
            }
          },
          loading: {
            true: {
              root: {
                web: {
                  cursor: "wait",
                  "& *": { cursor: "wait !important" }
                }
              }
            }
          },
          size: {
            xsmall: {
              root: {
                minHeight: "1.25rem",
                minWidth: "1.25rem",
                paddingHorizontal: "0.25rem"
              }
            },
            small: {
              root: {
                minHeight: "1.75rem",
                minWidth: "1.75rem",
                paddingHorizontal: "0.5rem"
              }
            },
            large: {
              root: {
                minHeight: "2.75rem",
                minWidth: "2.75rem",
                paddingHorizontal: "1rem"
              }
            },
            xlarge: {
              root: {
                minHeight: "3.25rem",
                minWidth: "3.25rem",
                paddingHorizontal: "1.25rem"
              }
            }
          },
          transform: {
            none: { label: { textTransform: "none" } },
            capitalize: { label: { textTransform: "capitalize" } },
            uppercase: { label: { textTransform: "uppercase" } },
            lowercase: { label: { textTransform: "lowercase" } },
            "full-width": { label: { textTransform: "full-width" } }
          },
          variant: {
            outline: {
              root: {
                bg: "trans",
                web: { "&:hover": { bg: (theme) => theme.color("primary", 0.2) } }
              },
              label: {
                color: "primary"
              }
            },
            text: {
              root: {
                bg: "trans",
                borderColor: "trans",
                web: { "&:hover": { bg: (theme) => theme.color("primary", 0.2) } }
              },
              label: {
                color: "primary"
              }
            }
          }
        }
      },
      ButtonGroup: {
        name: "rbk-button-group",
        defaultProps: {
          color: "primary"
        },
        defaultStyles: {
          root: {
            m: -1
          },
          content: {
            display: "flex",
            flexDirection: "row",
            flexWrap: "nowrap",
            alignItems: "stretch",
            p: 1
          }
        },
        variants: {}
      },
      Calendar: {
        name: "rbk-calendar",
        defaultProps: {
          color: "primary"
        },
        defaultStyles: {
          root: {}
        },
        variants: {}
      },
      Card: {
        name: "rbk-card",
        defaultProps: {},
        defaultStyles: {
          root: {
            backgroundColor: "background.primary",
            corners: 2,
            p: "1gap"
          }
        },
        variants: {}
      },
      Carousel: {
        name: "rbk-carousel",
        defaultProps: {
          chevron: true,
          color: "primary",
          gap: 0,
          pagingEnabled: true,
          pointerScroll: true,
          xs: 1,
          // TODO: move to defaultStyles
          chevronStyle: { px: 0 }
        },
        defaultStyles: {
          root: {
            position: "relative"
          },
          content: {},
          chevron: {
            position: "absolute",
            top: 0,
            bottom: 0,
            alignItems: "center",
            justifyContent: "center"
          }
        },
        variants: {}
      },
      Checkbox: {
        name: "rbk-checkbox",
        defaultProps: {
          accessibility: { role: "checkbox" },
          color: "primary",
          size: "medium"
        },
        defaultStyles: {
          root: {
            display: "flex",
            flexDirection: "row",
            flexWrap: "nowrap",
            alignItems: "center"
          },
          button: {
            padding: 0,
            marginLeft: "-0.5rem",
            minHeight: 0,
            minWidth: 0
          },
          error: {
            color: "error",
            mx: 1,
            mt: 1
          }
        },
        variants: {}
      },
      Collapse: {
        name: "rbk-collapse",
        defaultProps: {
          timing: "ease"
        },
        defaultStyles: {
          root: {
            overflow: "hidden"
          }
        },
        variants: {}
      },
      Divider: {
        name: "rbk-divider",
        defaultProps: {
          color: "text.primary",
          size: 1,
          opacity: 0.15
        },
        defaultStyles: {
          root: {
            backgroundColor: "text.primary",
            height: 1,
            opacity: 0.15,
            width: "auto"
          }
        },
        variants: {
          vertical: {
            true: {
              root: {
                alignSelf: "stretch",
                height: "auto",
                width: 1
              }
            }
          }
        }
      },
      Drawer: {
        name: "rbk-drawer",
        defaultProps: {
          placement: "right"
        },
        defaultStyles: {
          root: {
            position: "absolute",
            bg: "background",
            maxh: "100%",
            maxw: "100%"
          },
          backdrop: {}
        },
        variants: {
          placement: {
            left: {
              root: { top: 0, left: 0, height: "100%" }
            },
            right: {
              root: { top: 0, right: 0, height: "100%" }
            },
            top: {
              root: { top: 0, left: 0, width: "100%" }
            },
            bottom: {
              root: { bottom: 0, left: 0, width: "100%" }
            }
          }
        }
      },
      Dropdown: {
        name: "rbk-dropdown",
        defaultProps: {
          accessibility: { role: "menu" },
          placement: "bottom"
        },
        defaultStyles: {
          root: {
            position: "absolute"
          },
          backdrop: {
            justifyContent: "flex-start",
            justifyItems: "flex-start",
            alignContent: "flex-start",
            alignItems: "flex-start",
            bg: "rgba(0, 0, 0, 0.2)",
            zIndex: (theme) => theme.mixins.zIndex.dropdown
          }
        },
        variants: {}
      },
      Form: {
        name: "rbk-form",
        defaultProps: {},
        defaultStyles: {
          root: {
            margin: 0,
            padding: 0
          }
        },
        variants: {}
      },
      Grid: {
        name: "rbk-grid",
        defaultProps: {
          size: 12
        },
        defaultStyles: {
          root: {
            display: "flex",
            flexDirection: "row",
            flexWrap: "wrap",
            alignContent: "stretch"
          },
          item: {
            m: 0,
            p: 0
          }
        },
        variants: {
          direction: {
            horizontal: {
              root: { flexDirection: "row" }
            },
            vertical: {
              root: { flexDirection: "column" }
            }
          }
        }
      },
      GrowBox: {
        name: "rbk-grow-box",
        defaultProps: {},
        defaultStyles: {
          root: {
            overflow: "hidden"
          }
        },
        variants: {}
      },
      Image: {
        name: "rbk-image",
        defaultProps: {
          accessibility: { role: "image" },
          mode: "cover"
        },
        defaultStyles: {
          root: {
            web: { display: "block" }
          }
        },
        variants: {}
      },
      InputBase: {
        name: "rbk-input-base",
        defaultProps: {
          autoCapitalize: "none",
          color: "primary",
          returnKeyType: "default",
          size: "medium",
          type: "text"
        },
        defaultStyles: {
          root: {
            position: "relative"
          },
          content: {
            position: "relative",
            borderWidth: 1,
            borderStyle: "solid",
            borderColor: "gray.light",
            borderRadius: (theme) => theme.shape.borderRadius,
            backgroundColor: "text.primary.03",
            web: {
              ...this.mixins.transitions.fast,
              transitionProperty: "box-shadow"
            }
          },
          label: {
            mx: 1,
            mb: 1
          },
          input: {
            backgroundColor: (theme) => theme.color("trans"),
            borderWidth: 0,
            color: "text.primary",
            flex: 1,
            fontSize: "1rem",
            height: "2.25rem",
            margin: 0,
            paddingVertical: 0,
            paddingHorizontal: "0.625rem",
            textDecorationLine: "none",
            width: "100%",
            web: {
              backgroundImage: "none",
              boxShadow: "none",
              cursor: "inherit",
              lineHeight: "1rem",
              outline: "0 !important",
              paddingVertical: "0.625rem",
              "-moz-appearance": "textfield",
              appearance: "textfield",
              "&::-webkit-outer-spin-button,::-webkit-inner-spin-button": {
                "-webkit-appearance": "none",
                appearance: "none"
              }
            }
          },
          hint: {
            mx: 1,
            mt: 1
          },
          error: {
            color: "error",
            mx: 1,
            mt: 1
          }
        },
        variants: {
          disabled: {
            true: {
              content: {
                backgroundColor: (theme) => theme.color("background.disabled", 0.125),
                borderColor: (theme) => theme.color("background.disabled", 0.25)
              },
              input: {
                web: {
                  cursor: "not-allowed !important",
                  "& *": { cursor: "not-allowed !important" }
                }
              }
            }
          },
          multiline: {
            true: {
              input: {
                textAlignVertical: "top"
              }
            }
          }
        }
      },
      Input: {
        name: "rbk-input",
        defaultProps: {
          color: "primary",
          size: "medium"
        },
        defaultStyles: {
          root: {},
          content: {},
          label: {},
          input: {},
          hint: {},
          error: {}
        },
        variants: {}
      },
      InputDate: {
        name: "rbk-input-date",
        defaultProps: {
          color: "primary",
          size: "medium",
          translate: {
            cancel: "Cancel",
            clear: "Clear",
            today: "Today"
          }
        },
        defaultStyles: {
          root: {},
          content: {},
          label: {},
          input: {},
          hint: {},
          error: {}
        },
        variants: {}
      },
      InputPin: {
        name: "rbk-pin-input",
        defaultProps: {
          length: 4,
          size: "medium",
          type: "alphanumeric"
        },
        defaultStyles: {
          root: {},
          content: {},
          label: {},
          input: {},
          hint: {},
          error: {}
        },
        variants: {}
      },
      Label: {
        name: "rbk-label",
        defaultProps: {},
        defaultStyles: {
          root: {
            web: {
              textRendering: "optimizeLegibility",
              "-webkit-font-smoothing": "antialiased",
              "-moz-osx-font-smoothing": "grayscale",
              "font-smooth": "always"
            }
          }
        },
        variants: {}
      },
      Link: {
        name: "rbk-link",
        defaultProps: {},
        defaultStyles: {
          root: {
            color: "primary",
            web: { cursor: "pointer" }
          }
        },
        variants: {
          underline: {
            true: {
              root: { textDecoration: "underline" }
            }
          }
        }
      },
      List: {
        name: "rbk-list",
        defaultProps: {
          direction: "vertical",
          renderOffset: 200
        },
        defaultStyles: {
          root: {
            position: "relative"
          }
        },
        variants: {}
      },
      ListItem: {
        name: "rbk-list-item",
        defaultProps: {
          gap: 1
        },
        defaultStyles: {
          root: {}
        },
        variants: {}
      },
      Loading: {
        name: "rbk-loading",
        defaultProps: {
          duration: 500,
          accessibility: { role: "progressbar" },
          color: "primary",
          size: "medium"
        },
        defaultStyles: {
          root: {
            flexDirection: "row",
            alignItems: "center",
            justifyContent: "center"
          },
          label: {
            color: "primary",
            fontSize: "1rem",
            ml: "0.5gap"
          }
        },
        variants: {}
      },
      Modal: {
        name: "rbk-modal",
        defaultProps: {
          halign: "center",
          valign: "center"
        },
        defaultStyles: {
          root: {
            maxh: "100%",
            maxw: "100%"
          },
          backdrop: {
            p: "1gap",
            zIndex: (theme) => theme.mixins.zIndex.modal
          }
        },
        variants: {
          halign: {
            center: {
              backdrop: { alignItems: "center" }
            },
            left: {
              backdrop: { alignItems: "flex-start" }
            },
            right: {
              backdrop: { alignItems: "flex-end" }
            }
          },
          valign: {
            center: {
              backdrop: { justifyContent: "center" }
            },
            top: {
              backdrop: { justifyContent: "flex-start" }
            },
            bottom: {
              backdrop: { justifyContent: "flex-end" }
            }
          }
        }
      },
      Outline: {
        name: "rbk-outline",
        defaultProps: {
          color: "primary",
          size: 4,
          visible: "auto"
        },
        defaultStyles: {
          root: {
            web: {
              outline: "0 !important",
              ...this.mixins.transitions.fast,
              transitionProperty: "box-shadow"
            }
          }
        },
        variants: {}
      },
      Progress: {
        name: "rbk-progress",
        defaultProps: {
          accessibility: { role: "progressbar" },
          color: "primary",
          label: true,
          size: "medium"
        },
        defaultStyles: {
          root: {
            position: "relative",
            bg: "text.main.25",
            corners: 2.5,
            overflow: "hidden"
          },
          bar: {
            bg: "primary",
            h: "1.25rem"
          },
          label: {
            textAlign: "center"
          }
        },
        variants: {}
      },
      Resizable: {
        name: "rbk-resizable",
        defaultProps: {},
        defaultStyles: {
          root: {
            position: "relative",
            overflow: "hidden",
            web: {
              "&::-webkit-resizer": { opacity: 0 }
            }
          }
        },
        variants: {}
      },
      Scrollable: {
        name: "rbk-scrollable",
        defaultProps: {
          direction: "vertical"
        },
        defaultStyles: {
          root: {
            flexGrow: 1,
            flexShrink: 1,
            flexDirection: "column",
            web: {
              display: "flex",
              overflowY: "auto",
              scrollBehavior: "smooth",
              ...this.mixins.scroll
            }
          },
          content: {
            display: "flex",
            flexDirection: "column",
            flexWrap: "nowrap",
            flexGrow: 1
          }
        },
        variants: {
          direction: {
            horizontal: {
              root: {
                flexDirection: "row",
                web: { overflowX: "auto" }
              },
              content: {
                flexDirection: "row"
              }
            }
          },
          hideScrollBar: {
            true: {
              root: {
                web: {
                  "-ms-overflow-style": "none",
                  "scrollbar-width": "none",
                  "&::-webkit-scrollbar": { display: "none" }
                }
              }
            }
          }
        }
      },
      Select: {
        name: "rbk-select",
        defaultProps: {
          accessibility: { role: "combobox" },
          color: "primary",
          searchCount: 20,
          size: "medium"
        },
        defaultStyles: {
          root: {},
          label: {
            mx: 1,
            mb: 1
          },
          error: {
            color: "error",
            mx: 1,
            mt: 1
          }
        },
        variants: {}
      },
      Splitter: {
        name: "rbk-splitter",
        defaultProps: {
          direction: "horizontal"
        },
        defaultStyles: {
          root: {
            flexDirection: "row",
            flexWrap: "nowrap"
          },
          item: {
            flexGrow: 1,
            flexShrink: 1,
            flexBasis: "auto"
          }
        },
        variants: {
          direction: {
            vertical: {
              root: {
                flexDirection: "column"
              }
            }
          }
        }
      },
      Switch: {
        name: "rbk-switch",
        defaultProps: {
          accessibility: { role: "switch" },
          size: "medium",
          onColor: "primary",
          offColor: "gray.lighter",
          onThumbColor: "white",
          offThumbColor: "white"
        },
        defaultStyles: {
          root: {
            display: "flex",
            flexDirection: "row",
            flexWrap: "nowrap",
            alignItems: "center"
          },
          button: {
            p: 0,
            h: "1.875rem",
            w: "3.375rem",
            minh: 0,
            minw: 0
          },
          label: {
            ml: "1gap"
          },
          thumb: {
            bg: "white",
            borderRadius: "0.75rem",
            h: "1.5rem",
            w: "1.5rem"
          },
          error: {
            color: "error",
            mx: 1,
            mt: 1
          }
        },
        variants: {}
      },
      Table: {
        name: "rbk-table",
        defaultProps: {},
        defaultStyles: {
          root: {
            corners: 1
          }
        },
        variants: {}
      },
      Tabs: {
        name: "rbk-tabs",
        defaultProps: {
          accessibility: { role: "tablist" },
          alignment: "start",
          variant: "group"
        },
        defaultStyles: {
          root: {
            m: -1
          },
          content: {
            flexDirection: "row",
            flexWrap: "nowrap",
            alignItems: "stretch",
            minw: "100%",
            p: 1
          },
          button: {
            web: { transitionProperty: "all" }
          },
          active: {}
        },
        variants: {
          alignment: {
            start: {
              content: { justifyContent: "start" }
            },
            center: {
              content: { justifyContent: "center" }
            },
            end: {
              content: { justifyContent: "end" }
            }
          },
          variant: {
            group: {
              button: {
                borderBottomWidth: 0,
                borderBottomLeftRadius: 0,
                borderBottomRightRadius: 0,
                marginLeft: -1
              }
            },
            card: {
              root: {
                mx: -2
              },
              button: {
                mx: 1
              }
            }
          }
        }
      },
      Terminal: {
        name: "rbk-terminal",
        defaultProps: {
          commands: [],
          prompt: "$",
          version: "0.0.0",
          welcomeMessage: "Welcome to Terminal!"
        },
        defaultStyles: {
          root: {
            backgroundColor: "background.primary",
            border: "1px solid backgorund.secondary",
            corners: 1,
            cursor: "text"
          }
        },
        variants: {}
      },
      Text: {
        name: "rbk-text",
        defaultProps: {},
        defaultStyles: {
          root: {
            color: "text.primary",
            fontSize: "1rem",
            margin: 0,
            textAlign: "left",
            textDecorationLine: "none",
            web: {
              display: "block",
              "& .rbk-text": { display: "inline" }
            },
            native: {
              flexDirection: "row"
            }
          }
        },
        variants: {
          variant: {
            h1: { root: { fontSize: "2.6rem", fontWeight: 700 } },
            h2: { root: { fontSize: "2.1rem", fontWeight: 700 } },
            h3: { root: { fontSize: "1.8rem", fontWeight: 700 } },
            h4: { root: { fontSize: "1.4rem", fontWeight: 700 } },
            h5: { root: { fontSize: "1.2rem", fontWeight: 700 } },
            h6: { root: { fontSize: "1.1rem", fontWeight: 700 } },
            title: { root: { fontSize: "1.5rem", fontWeight: 600 } },
            subtitle: { root: { fontSize: "1.25rem", fontWeight: 500 } },
            primary: { root: { fontSize: "1rem", fontWeight: 400 } },
            secondary: { root: { fontSize: "0.875rem", fontWeight: 400, color: "text.secondary" } },
            caption: { root: { fontSize: "0.75rem", fontWeight: 400, letterSpacing: 0.5 } }
          },
          bold: {
            true: { root: { fontWeight: "bold" } },
            false: { root: { fontWeight: "normal" } }
          },
          italic: {
            true: { root: { fontStyle: "italic" } },
            false: { root: { fontStyle: "normal" } }
          },
          center: {
            true: { root: { textAlign: "center" } },
            false: { root: { web: { textAlign: "initial" }, native: { textAlign: "auto" } } }
          },
          left: {
            true: { root: { textAlign: "left" } },
            false: { root: { web: { textAlign: "initial" }, native: { textAlign: "auto" } } }
          },
          right: {
            true: { root: { textAlign: "right" } },
            false: { root: { web: { textAlign: "initial" }, native: { textAlign: "auto" } } }
          },
          justify: {
            true: { root: { textAlign: "justify" } },
            false: { root: { web: { textAlign: "initial" }, native: { textAlign: "auto" } } }
          },
          smallCaps: {
            true: { root: { web: { fontVariant: "small-caps" }, native: { fontVariant: ["small-caps"] } } },
            false: { root: { web: { fontVariant: "initial" }, native: { fontVariant: [] } } }
          },
          selectable: {
            false: {
              root: {
                web: {
                  "-webkit-user-select": "none",
                  "-ms-user-select": "none",
                  "user-select": "none"
                }
              }
            }
          }
        }
      },
      Toaster: {
        name: "rbk-toaster",
        defaultProps: {
          color: "gray.dark",
          duration: 4e3,
          halign: "left",
          valign: "bottom",
          offset: {
            x: "1gap",
            y: "1gap"
          }
        },
        defaultStyles: {
          root: {
            position: "relative",
            corners: 2,
            p: "1gap",
            bg: "gray.dark",
            overflow: "hidden"
          }
        },
        variants: {}
      },
      Tooltip: {
        name: "rbk-tooltip",
        defaultProps: {
          position: "top",
          color: "black",
          delay: 200,
          offset: 1
        },
        defaultStyles: {
          root: {
            zIndex: (theme) => theme.mixins.zIndex.tooltip,
            web: {
              position: "fixed",
              opacity: 0,
              pointerEvents: "none",
              visibility: "hidden",
              zIndex: -1,
              ...this.mixins.transitions.fast
            },
            native: {
              position: "absolute",
              display: "none"
            }
          },
          label: {
            backgroundColor: "black",
            borderRadius: (theme) => theme.shape.borderRadius,
            color: "white",
            fontSize: "0.75rem",
            overflow: "hidden",
            py: 1,
            px: 1.5,
            textAlign: "center"
          }
        },
        variants: {
          visible: {
            true: {
              root: {
                web: {
                  opacity: 1,
                  visibility: "visible",
                  zIndex: (theme) => theme.mixins.zIndex.tooltip
                },
                native: {
                  display: "flex"
                }
              }
            }
          }
        }
      }
    };
  }
};

const dark = {
  mode: "dark",
  colors: {
    text: {
      primary: "#EFEFEF",
      secondary: "#A7A7A7",
      tertiary: "#787878",
      disabled: "#545454"
    },
    background: {
      primary: "#292929",
      secondary: "#141414",
      tertiary: "#212121",
      disabled: "#0C0C0C"
    }
  }
};

function deepmerge(...mixed) {
  const result = {};
  for (const item of mixed) {
    if (!item || typeof item !== "object") {
      continue;
    }
    for (const attr in item) {
      let value = item[attr];
      if (value && !Array.isArray(value) && typeof value === "object") {
        value = deepmerge(result[attr], value);
      }
      result[attr] = value;
    }
  }
  return result;
}

function createTheme(options) {
  options = options || {};
  const mode = options?.mode || "light";
  const reference = mode === "dark" ? dark : light;
  const theme = deepmerge(base$1, reference, options, { mode });
  if (theme.colors) {
    for (const prop in theme.colors) {
      const color = theme.colors[prop];
      if (typeof color === "string") {
        theme.colors[prop] = { main: color };
      }
      const main = theme.colors?.[prop]?.main;
      if (!main) {
        continue;
      }
      const light2 = theme.colors?.[prop]?.light;
      const lighter = theme.colors?.[prop]?.lighter;
      const dark2 = theme.colors?.[prop]?.dark;
      const darker = theme.colors?.[prop]?.darker;
      const contrast = theme.colors?.[prop]?.contrast;
      if (!light2) {
        theme.colors[prop].light = theme.color(main, null, "50%");
      }
      if (!lighter) {
        theme.colors[prop].lighter = theme.color(main, null, "100%");
      }
      if (!dark2) {
        theme.colors[prop].dark = theme.color(main, null, "-50%");
      }
      if (!darker) {
        theme.colors[prop].darker = theme.color(main, null, "-100%");
      }
      if (!contrast) {
        theme.colors[prop].contrast = theme.contrast(main);
      }
    }
  }
  registry(theme);
  if (Platform.web) {
    normalize();
    createStyleSheet(
      `
html {
  font-size: ${theme.typography.fontSize}px;
  line-height: ${theme.typography.lineHeight};
  height: 100%;
}

body {
  background-color: ${theme.colors.background.secondary};
  color: ${theme.colors.text.primary};
  margin: 0;
  padding: 0;
  height: 100%;
  width: 100%;
}

a {
  text-decoration: none;
}

${Object.entries(theme.mixins.scroll).map(([selector, styles]) => `${selector.substring(1)} { ${css(styles)} }`).join(`
`)}
`,
      { uniqueId: "rbk-global", prepend: true }
    );
    createStyleSheet(string(theme.mixins.globalCSS), {
      uniqueId: "rbk-global-from-theme",
      prepend: true
    });
  }
  const components = { ...theme.components };
  const ordered = extract(
    ["Box", "Text", "Outline", "Label", "Backdrop", "Scrollable", "Card", "Dropdown", "Button", "ButtonGroup"],
    components
  );
  const list = [...Object.values(ordered), ...Object.values(components)];
  list.forEach((component) => {
    const componentName = component?.name;
    const styles = component?.defaultStyles || {};
    if (!componentName)
      return;
    for (const prop in styles) {
      const style = styles?.[prop];
      const name = componentName + (prop === "root" ? "" : `-${prop}`);
      if (Platform.native) {
        rbkGlobal.styles[name] = jss(style);
      }
      if (Platform.web) {
        rbkGlobal.styles[name] = sheet({
          __className: name,
          ...style
        });
      }
    }
    Object.entries(component?.variants || {}).forEach(([varAttr, varOptions]) => {
      Object.entries(varOptions).map(([optionKey, optionVal]) => {
        Object.entries(optionVal || {}).forEach(([styleId, style]) => {
          const name = `${componentName}-${varAttr}-${optionKey}` + (styleId === "root" ? "" : `-${styleId}`);
          if (Platform.native) {
            rbkGlobal.styles[name] = jss(style);
          }
          if (Platform.web) {
            rbkGlobal.styles[name] = sheet({
              __className: name,
              ...style
            });
          }
        });
      });
    });
  });
  rbkGlobal.theme = theme;
  return theme;
}

function useTheme() {
  const theme = useContext(RbkContext)?.theme;
  if (!theme && !Object.keys(rbkGlobal.theme).length) {
    createTheme();
  }
  return theme || rbkGlobal.theme;
}

function get(prop, ...mixin) {
  let result;
  for (const item of mixin) {
    if (!item)
      continue;
    if (Array.isArray(item)) {
      result = get(prop, ...item) ?? result;
    }
    result = item?.[prop] ?? result;
  }
  return result;
}

function base(event, extra) {
  const { native } = Platform;
  const nativeEvent = event.nativeEvent ?? event;
  const originalTarget = get("target", nativeEvent, event) ?? null;
  const target = native ? { ...originalTarget, value: nativeEvent?.text } : originalTarget;
  return {
    type: get("type", nativeEvent, event) ?? "unknown",
    handler: "RbkEvent",
    // Extra
    ...extra ?? {},
    target,
    currentTarget: get("currentTarget", nativeEvent, event) ?? target,
    timestamp: Date.now(),
    // TODO: check why
    // nativeEvent: Platform.native ? omit('nativeEvent', event) : nativeEvent,
    nativeEvent,
    preventDefault: () => {
      event?.preventDefault?.();
      nativeEvent?.preventDefault?.();
    },
    stopPropagation: () => {
      event?.stopPropagation?.();
      nativeEvent?.stopPropagation?.();
    }
  };
}

function keyboard(event, extra) {
  const nativeEvent = event.nativeEvent ?? event;
  const props = {
    handler: "RbkKeyboardEvent",
    key: get("key", nativeEvent, event),
    code: get("code", nativeEvent, event),
    keyCode: get("keyCode", nativeEvent, event) ?? get("which", nativeEvent, event),
    altKey: get("altKey", nativeEvent, event),
    ctrlKey: get("ctrlKey", nativeEvent, event),
    metaKey: get("metaKey", nativeEvent, event),
    shiftKey: get("shiftKey", nativeEvent, event),
    // Extra
    ...extra ?? {}
  };
  return base(event, props);
}

function pointer(event, extra) {
  const nativeEvent = event.nativeEvent ?? event;
  const targetRect = get("target", nativeEvent, event)?.getBoundingClientRect?.();
  const touches = Array.from(
    get("changedTouches", nativeEvent, event) ?? get("touches", nativeEvent, event) ?? []
  ).map((touch) => ({
    identifier: touch?.identifier ?? 0,
    offsetX: targetRect ? touch.pageX - targetRect.left : get("offsetX", touch) ?? get("locationX", touch),
    offsetY: targetRect ? touch.pageY - targetRect.top : get("offsetY", touch) ?? get("locationY", touch),
    pageX: get("pageX", touch),
    pageY: get("pageY", touch)
  }));
  const props = {
    handler: "RbkPointerEvent",
    button: get("button", nativeEvent, event, touches?.[0]) ?? 0,
    offsetX: get("offsetX", nativeEvent, event, touches?.[0]) ?? get("locationX", nativeEvent, event, touches?.[0]),
    offsetY: get("offsetY", nativeEvent, event, touches?.[0]) ?? get("locationY", nativeEvent, event, touches?.[0]),
    pageX: get("pageX", nativeEvent, event, touches?.[0]),
    pageY: get("pageY", nativeEvent, event, touches?.[0]),
    // Extra
    ...extra ?? {},
    // Touch
    touches
  };
  return base(event, props);
}

function scroll(event, extra) {
  const props = {
    handler: "RbkScrollEvent",
    width: event.target?.clientWidth ?? event.nativeEvent?.layoutMeasurement?.width,
    height: event.target?.clientHeight ?? event.nativeEvent?.layoutMeasurement?.height,
    contentWidth: event.target?.scrollWidth ?? event.nativeEvent?.contentSize?.width,
    contentHeight: event.target?.scrollHeight ?? event.nativeEvent?.contentSize?.height,
    offsetX: event.target?.scrollLeft ?? event.nativeEvent?.contentOffset?.x,
    offsetY: event.target?.scrollTop ?? event.nativeEvent?.contentOffset?.y,
    // Extra
    ...{}
  };
  return base(event, props);
}

function bindings({ ...props }) {
  const { web, native } = Platform;
  const mapping = {
    common: [["onFocus"], ["onBlur"], ["onChange"]],
    keyboard: [["onKeyPress"], ["onKeyDown"], ["onKeyUp"]],
    pointer: [["onClick", "onPress"], ["onMouseDown", "onPressIn"], ["onMouseUp", "onPressOut"], ["onLongPress"]],
    scroll: [["onScroll"]]
  };
  for (const handler in mapping) {
    for (const events of mapping[handler]) {
      const webEventName = events[0];
      const nativeEventName = events[1] ?? events[0];
      for (const eventName of events) {
        const callback = props[eventName];
        delete props[eventName];
        if (!callback) {
          continue;
        }
        const callbackHandled = (event) => handler === "pointer" ? callback(pointer(event)) : handler === "keyboard" ? callback(keyboard(event)) : handler === "scroll" ? callback(scroll(event)) : callback(base(event));
        if (web) {
          props[webEventName] = callbackHandled;
        }
        if (native) {
          props[nativeEventName] = callbackHandled;
        }
      }
    }
  }
  if (props.disabled) {
    [...mapping.keyboard, ...mapping.pointer].flat().forEach((eventName) => {
      delete props[eventName];
    });
  }
  return props;
}

function factory2(props, options) {
  const fallbackProps = {};
  const variants = {};
  Object.entries(options?.defaultProps || {}).forEach(([prop, value]) => {
    fallbackProps[prop] = props?.[prop] ?? value;
  });
  Object.keys(options?.defaultStyles || {}).forEach((styleId) => {
    if (!options?.name)
      return;
    const name = options.name + (styleId === "root" ? "" : `-${styleId}`);
    variants[styleId] = [rbkGlobal.styles[name]];
  });
  Object.entries(options?.variants || {}).forEach(([varAttr, varOptions]) => {
    if (!options?.name)
      return;
    const varValue = props[varAttr] ?? fallbackProps[varAttr];
    if (typeof varValue === "undefined")
      return;
    const varStyles = varOptions[varValue] || {};
    Object.keys(varStyles).forEach((styleId) => {
      const name = `${options?.name}-${varAttr}-${varValue}` + (styleId === "root" ? "" : `-${styleId}`);
      variants[styleId] = variants[styleId] || [];
      variants[styleId].push(rbkGlobal.styles[name]);
    });
  });
  Object.entries(props.variants || {}).forEach(([styleId, style]) => {
    variants[styleId] = variants[styleId] || [];
    variants[styleId].push(style);
  });
  fallbackProps.accessibility = deepmerge(fallbackProps.accessibility, props?.accessibility);
  return { ...props, ...fallbackProps, variants };
}

function merge(...mixin) {
  let result = {};
  for (const item of mixin) {
    if (typeof item !== "object" || !item)
      continue;
    if (Array.isArray(item)) {
      result = merge(result, ...item);
    } else {
      result = { ...result, ...item };
    }
  }
  return result;
}

function clsx(...args) {
  let i = 0, tmp, x, str = "";
  while (i < args.length) {
    if (tmp = args[i++]) {
      if (x = toVal(tmp)) {
        str && (str += " ");
        str += x;
      }
    }
  }
  return str;
}
function toVal(mix) {
  let k, y, str = "";
  if (typeof mix === "string" || typeof mix === "number") {
    str += mix;
  } else if (typeof mix === "object") {
    if (Array.isArray(mix)) {
      for (k = 0; k < mix.length; k++) {
        if (mix[k]) {
          if (y = toVal(mix[k])) {
            str && (str += " ");
            str += y;
          }
        }
      }
    } else {
      for (k in mix) {
        if (mix[k]) {
          str && (str += " ");
          str += k;
        }
      }
    }
  }
  return str;
}

const BoxFactory = React.memo(
  forwardRef(({ platform, className, children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Box;
    const { web, native, Button, Text, View, useDimensions } = rbkGlobal.mapping;
    props = useMemo(() => factory2(props, options), [props, options]);
    if (platform) {
      Object.keys(platform).forEach((item) => {
        if (rbkGlobal.mapping[item] || item === "*") {
          Object.assign(props, merge({}, platform[item]));
        }
      });
    }
    let {
      accessibility,
      animation,
      breakpoints,
      center,
      column,
      component,
      componentProps,
      flex,
      hidden,
      invisible,
      mount,
      noRootStyles,
      noWrap,
      pressable,
      reverse,
      row,
      wrap,
      // Styles
      variants,
      style,
      rawStyle,
      ...rest
    } = props;
    rest = bindings(rest);
    pressable = pressable ?? Boolean(
      props.onPress || props.onClick || props.onPressIn || props.onMouseDown || props.onPointerDown || props.onLongPress
    );
    const curBreakpoints = useMemo(
      () => breakpoints ?? theme.breakpoints,
      [breakpoints, theme.breakpoints]
    );
    if (native && pressable && !component) {
      component = Button;
    }
    const stylesFromProps = [];
    for (const prop of styleProps) {
      if (prop in rest) {
        stylesFromProps.push({ [prop]: rest[prop] });
        delete rest[prop];
      }
    }
    const hiddenStyle = {
      display: "none",
      opacity: 0,
      overflow: "hidden"
    };
    style = [
      typeof invisible === "boolean" && { opacity: invisible ? 0 : 1 },
      // Flex Container
      row && {
        flexDirection: reverse ? "row-reverse" : "row",
        flexWrap: "wrap"
      },
      column && {
        flexDirection: reverse ? "column-reverse" : "column"
      },
      wrap && typeof wrap !== "boolean" && { flexWrap: wrap },
      typeof wrap === "boolean" && { flexWrap: wrap ? "wrap" : "nowrap" },
      noWrap && { flexWrap: "nowrap" },
      center && {
        justifyContent: "center",
        justifyItems: "center",
        alignContent: "center",
        alignItems: "center"
      },
      // Flex Item
      defined(flex) && { flex: Number(flex) },
      web && pressable && { cursor: "pointer" },
      clone(style),
      stylesFromProps,
      hidden === true && hiddenStyle
    ];
    if (Number(get("flex", style) ?? 0)) {
      style.unshift({ minWidth: 0, minHeight: 0 });
    }
    if (noRootStyles && variants.root?.length) {
      variants.root.shift();
    }
    const styles = [variants?.root];
    const responsiveStyle = extract(Object.keys(curBreakpoints), style);
    const breakpointNames = useMemo(
      () => Object.entries(curBreakpoints).sort((a, b) => a[1] - b[1]).map(([bkptName]) => bkptName),
      [curBreakpoints]
    );
    useDimensions(native && Object.keys(responsiveStyle).length > 0);
    for (const bkptIndex in breakpointNames) {
      const bkptName = breakpointNames[bkptIndex];
      const bkptStyle = responsiveStyle?.[bkptName];
      if (bkptStyle) {
        style.push({
          [`@media (min-width: ${curBreakpoints[bkptName]}px)`]: bkptStyle
        });
      }
      if (hidden?.[bkptName]) {
        const bkptNext = curBreakpoints?.[breakpointNames[Number(bkptIndex) + 1]];
        let media = `@media (min-width: ${curBreakpoints[bkptName]}px)`;
        if (bkptNext) {
          media += ` and (max-width: ${bkptNext - 1}px)`;
        }
        style.push({ [media]: hiddenStyle });
      }
    }
    const transition = useAnimation(animation?.from, ref);
    const animStrDiffRef = useRef();
    style.push(transition.props.style);
    if (web) {
      styles.push(sheet(style));
      rest.style = jss(rawStyle);
      rest.className = clsx(styles, className);
    }
    if (native) {
      styles.push(jss(style));
      styles.push(jss(rawStyle));
      rest.style = styles;
    }
    if (pressable && !accessibility?.role) {
      accessibility = { ...Object(accessibility), role: "button" };
    }
    if (accessibility) {
      if (web) {
        rest["aria-placeholder"] = accessibility?.hint;
        rest["aria-label"] = accessibility?.label;
        rest["role"] = accessibility?.role;
        rest["aria-checked"] = accessibility.state?.checked;
        rest["aria-disabled"] = accessibility.state?.disabled;
        rest["aria-expanded"] = accessibility.state?.expanded;
        rest["aria-selected"] = accessibility.state?.selected;
        rest["aria-busy"] = accessibility.state?.busy;
        rest["aria-valuemax"] = accessibility.value?.max;
        rest["aria-valuemin"] = accessibility.value?.min;
        rest["aria-valuenow"] = accessibility.value?.now;
        rest["aria-valuetext"] = accessibility.value?.text;
      }
      if (native) {
        rest.accessible = true;
        rest.accessibilityHint = accessibility?.hint;
        rest.accessibilityLabel = accessibility?.label;
        rest.accessibilityRole = accessibility?.role;
        rest.accessibilityState = accessibility?.state;
        rest.accessibilityValue = accessibility?.value;
      }
    }
    const Component = component || View;
    useEffect(() => {
      const { onStart, onEnd, ...rest2 } = animation || {};
      const animStr = JSON.stringify(rest2);
      if (animStr === animStrDiffRef.current)
        return;
      animStrDiffRef.current = animStr;
      if (!animation)
        return;
      (async () => {
        onStart?.();
        await transition.start(animation);
        onEnd?.();
      })();
    }, [animation, transition]);
    return mount === false ? null : /* @__PURE__ */ jsx(Component, { ...transition.props, ...rest, ...componentProps, children: React.Children.map(children, (child) => {
      const isText = ["string", "number"].includes(typeof child);
      if (!child && !isText) {
        return null;
      }
      if (native && isText) {
        return /* @__PURE__ */ jsx(Text, { children: child });
      }
      return child;
    }) });
  })
);
BoxFactory.displayName = "BoxFactory";

const CardFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Card;
    const {
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    return /* @__PURE__ */ jsx(BoxFactory, { ref, variants: { root: variants.root }, ...rest });
  })
);
CardFactory.displayName = "CardFactory";

function childrenize(children) {
  const items = [];
  Children.forEach(children, (child) => {
    if (child?.type === Fragment) {
      items.push(...childrenize(child.props.children));
    } else {
      items.push(child);
    }
  });
  return items;
}

const TextFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Text;
    const { web, native, Text } = rbkGlobal.mapping;
    let {
      bold,
      center,
      color,
      invisible,
      italic,
      justify,
      left,
      numberOfLines,
      right,
      selectable,
      size,
      smallCaps,
      transform,
      variant,
      weight,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    if (native) {
      rest.includeFontPadding = false;
      if (numberOfLines) {
        rest.numberOfLines = numberOfLines;
      }
      if (defined(selectable)) {
        rest.selectable = selectable;
      }
    }
    style = [
      color && { color },
      size && { fontSize: theme.rem(size) },
      weight && { fontWeight: `${weight}` },
      transform && { textTransform: transform },
      web && defined(numberOfLines) && {
        display: "-webkit-box",
        "-webkit-line-clamp": `${numberOfLines}`,
        "-webkit-box-orient": "vertical",
        overflow: "hidden"
      },
      style
    ];
    bold = get("bold", style) ?? bold;
    color = get("color", style) ?? color;
    weight = get("weight", style) ?? weight;
    const fontSize = get("fontSize", style, rest);
    const fontWeight = get("fontWeight", style, rest);
    return /* @__PURE__ */ jsx(BoxFactory, { ref, component: Text, style, variants: { root: variants.root }, ...rest, noRootStyles: true, children: childrenize(children).map((child, key) => {
      if (child?.type === TextFactory) {
        child = cloneElement(child, { key, variant, bold, color, weight, fontSize, fontWeight, ...child.props });
      }
      return child;
    }) });
  })
);
TextFactory.displayName = "TextFactory";

function Toaster({ theme }, ref) {
  const { web, native } = Platform;
  const options = theme.components.Toaster;
  const idRef = useRef();
  const timeoutRef = useRef();
  const cardRef = useRef();
  const [props, setProps] = useState();
  const {
    content,
    color,
    duration,
    width,
    halign,
    offset,
    valign,
    // Events
    onPress,
    // Styles
    variants
  } = factory2(props || {}, options);
  const translateY = 120 * (valign === "top" ? -1 : 1);
  const textColor = theme.contrast(color);
  const containerStyle = [
    {
      position: native ? "absolute" : "fixed",
      zIndex: theme.mixins.zIndex.toaster,
      h: 0,
      w: "100%",
      l: 0,
      r: 0
    },
    valign === "top" && { t: 0 },
    valign === "bottom" && { b: 0 }
  ];
  const animationStyle = [
    {
      position: "absolute",
      maxw: "100%",
      width
    },
    valign === "top" && { t: 0 },
    valign === "bottom" && { b: 0 },
    halign === "left" && { l: 0 },
    halign === "right" && { r: 0 }
  ];
  useImperativeHandle(ref, () => ({ props, setProps }), [props]);
  useEffect(() => {
    if (props) {
      idRef.current = uuid();
      if (web) {
        setTimeout(() => cardRef?.current?.focus?.(), 10);
      }
      timeoutRef.current = setTimeout(() => {
        setProps(void 0);
      }, duration);
    }
    return () => {
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
    };
  }, [duration, props, web]);
  return /* @__PURE__ */ jsx(BoxFactory, { row: true, center: true, style: containerStyle, children: Boolean(props) && /* @__PURE__ */ jsx(
    BoxFactory,
    {
      px: offset?.x,
      py: offset?.y,
      style: animationStyle,
      animation: {
        throttle: 0,
        duration: 200,
        from: { transform: { translateY } },
        to: { transform: { translateY: 0 } }
      },
      children: /* @__PURE__ */ jsxs(
        CardFactory,
        {
          "data-rbk-toaster": color,
          ref: cardRef,
          variants: { root: variants.root },
          accessibility: { role: "alert" },
          platform: { web: { tabIndex: "-1" } },
          bg: color,
          onPress,
          children: [
            /* @__PURE__ */ jsx(BoxFactory, { row: true, noWrap: true, alignItems: "center", children: typeof content === "string" ? /* @__PURE__ */ jsx(TextFactory, { flex: true, variant: "secondary", color: textColor, children: content }) : content }),
            /* @__PURE__ */ jsx(BoxFactory, { position: "absolute", b: 0, l: 0, r: 0, children: /* @__PURE__ */ jsx(
              BoxFactory,
              {
                animation: {
                  throttle: 0,
                  duration,
                  timing: "linear",
                  iterations: 1,
                  from: { width: "0%" },
                  to: { width: "100%" }
                },
                children: /* @__PURE__ */ jsx(BoxFactory, { h: 2, bg: theme.color(textColor, 0.5) })
              }
            ) })
          ]
        }
      )
    },
    idRef.current
  ) });
}
var Toaster$1 = forwardRef(Toaster);

const toasterRef = createRef();
function ReactBulk({ theme: themeProp, children }) {
  const { web, native } = Platform;
  const mountedRef = useRef(false);
  const [draggable, setDraggable] = useState();
  const [theme, _setTheme] = useState(() => {
    return createTheme(typeof themeProp === "string" ? { mode: themeProp } : themeProp);
  });
  rbkGlobal.theme = theme;
  const setTheme = useCallback((theme2) => {
    _setTheme((current) => {
      return createTheme(
        theme2 instanceof Function ? theme2(current) : typeof theme2 === "string" ? { mode: theme2 } : theme2
      );
    });
  }, []);
  useEffect(() => {
    if (mountedRef.current) {
      setTheme(themeProp);
    }
    mountedRef.current = true;
  }, [mountedRef, themeProp, setTheme]);
  const context = useMemo(
    () => ({
      theme: { ...theme, setTheme },
      setDraggable,
      toasterRef
    }),
    [theme, setTheme]
  );
  return /* @__PURE__ */ jsxs(RbkContext.Provider, { value: context, children: [
    web && /* @__PURE__ */ jsx(BaseWeb$1, { theme, children }),
    native && /* @__PURE__ */ jsx(BoxFactory, { flex: true, minh: "100%", minw: "100%", ...draggable, children: /* @__PURE__ */ jsx(BaseNative$1, { theme, children }) }),
    /* @__PURE__ */ jsx(Toaster$1, { ref: toasterRef, theme }),
    /* @__PURE__ */ jsx(Portal, {})
  ] });
}

function dotObject(data) {
  const parsed = {};
  for (const baseAttr in data) {
    const attr = baseAttr.replace(/\[(.*)?]/g, (_, $1) => `.${$1 ?? 0}`);
    const value = data[baseAttr];
    const splitAttrs = attr.split(".");
    let splitAttr;
    let ref = parsed;
    while (splitAttr = splitAttrs.shift()) {
      if (!ref[splitAttr]) {
        ref[splitAttr] = splitAttrs.length ? {} : value;
      }
      ref = ref[splitAttr];
    }
  }
  return resolve(parsed);
}
function resolve(items) {
  if (typeof items !== "object" || !items) {
    return items;
  }
  if (!Array.isArray(items) && items?.constructor?.name !== "Object") {
    return items;
  }
  let newValue = items;
  if (!Array.isArray(items)) {
    const keys = Object.keys(items);
    if (keys.length) {
      keys.sort();
      if (keys.every((key, index) => Number(key) === index)) {
        newValue = Object.values(items);
      }
    }
  }
  for (const attr in newValue) {
    newValue[attr] = resolve(newValue[attr]);
  }
  return newValue;
}

const Context = createContext(null);
const useForm = () => useContext(Context);
const FormFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Form;
    const { web, Form } = rbkGlobal.mapping;
    const {
      disableObjectParser,
      initialData,
      // Events
      onSubmit,
      onCancel,
      onReset,
      onClear,
      onChange,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const rootRef = useRef(null);
    const fieldsRef = useRef([]);
    const dataRef = useRef({});
    if (web) {
      rest.noValidate = true;
    }
    const getField = useCallback(
      (name) => {
        return fieldsRef.current.find((item) => item?.name === name);
      },
      [fieldsRef]
    );
    const setField = useCallback(
      (options2) => {
        const { name } = options2;
        let field = getField(name);
        if (field) {
          Object.keys(field).forEach((prop) => delete field?.[prop]);
          Object.assign(field, options2);
        } else {
          field = options2;
          fieldsRef.current.push(options2);
        }
        const newValue = field.get();
        if (dataRef.current[name] !== newValue) {
          dataRef.current[name] = newValue;
          dispatchEvent("change", field);
        }
      },
      [dataRef, fieldsRef, getField, dispatchEvent]
    );
    const unsetField = useCallback(
      (name) => {
        fieldsRef.current = fieldsRef.current.filter((item) => item?.name !== name);
      },
      [fieldsRef]
    );
    const getValue = useCallback((name) => getField(name)?.get?.(), [getField]);
    const setValue = useCallback((name, value) => getField(name)?.set?.(value), [getField]);
    const getData = useCallback(() => {
      const data = {};
      fieldsRef.current.forEach(({ name, get }) => data[name] = get());
      return disableObjectParser ? data : dotObject(data);
    }, [fieldsRef, disableObjectParser]);
    const setData = useCallback(
      (data = {}) => {
        Object.keys(Object(data)).forEach((attr) => {
          const field = getField(attr);
          if (field) {
            const value = data[attr];
            if (value !== field.get()) {
              field.set(value);
            }
          }
        });
      },
      [getField]
    );
    const getErrors = useCallback(() => {
      const errors = {};
      fieldsRef.current.forEach((item) => {
        const message = item?.getError?.();
        if (message) {
          errors[item.name] = message;
        }
      });
      return Object.keys(errors).length ? errors : null;
    }, [fieldsRef]);
    const setErrors = useCallback(
      (errors) => {
        fieldsRef.current.forEach((item) => {
          item?.setError?.(errors?.[item.name] ?? false);
        });
      },
      [fieldsRef]
    );
    function dispatchEvent(type, field, nativeEvent) {
      const callback = {
        change: onChange,
        submit: onSubmit,
        cancel: onCancel,
        reset: onReset,
        clear: onClear
      }[type];
      const target = rootRef.current;
      const data = getData();
      const errors = getErrors();
      const event = {
        type,
        name: field?.name,
        form: context,
        target,
        nativeEvent
      };
      if (type === "change") {
        fieldsRef.current.forEach(({ onFormChange }) => onFormChange?.(event, data));
      }
      if (typeof callback === "function") {
        callback(event, data, errors);
      }
    }
    function submit(e) {
      e?.preventDefault?.();
      const nativeEvent = e?.nativeEvent ?? e;
      dispatchEvent("submit", void 0, nativeEvent);
    }
    function cancel(e) {
      e?.preventDefault?.();
      const nativeEvent = e?.nativeEvent ?? e;
      dispatchEvent("cancel", void 0, nativeEvent);
    }
    function reset(e) {
      e?.preventDefault?.();
      const nativeEvent = e?.nativeEvent ?? e;
      fieldsRef.current.forEach(({ set, initialValue }) => set(initialValue ?? null));
      setErrors(null);
      dispatchEvent("reset", void 0, nativeEvent);
    }
    function clear(e) {
      e?.preventDefault?.();
      const nativeEvent = e?.nativeEvent ?? e;
      fieldsRef.current.forEach(({ set }) => set(null));
      setErrors(null);
      dispatchEvent("clear", void 0, nativeEvent);
    }
    const context = useMemo(
      () => ({
        initialData,
        submit,
        cancel,
        reset,
        clear,
        getData,
        setData,
        getErrors,
        setErrors,
        getValue,
        setValue,
        getField,
        setField,
        unsetField,
        target: rootRef.current
      }),
      [
        rootRef.current,
        initialData,
        submit,
        cancel,
        reset,
        clear,
        getData,
        getErrors,
        getField,
        getValue,
        setData,
        setErrors,
        setField,
        setValue,
        unsetField
      ]
    );
    useImperativeHandle(ref, () => context, [context]);
    useEffect(() => {
      Object.assign(ref?.current || {}, { target: rootRef.current });
    }, [ref, rootRef]);
    return /* @__PURE__ */ jsx(Context.Provider, { value: context, children: /* @__PURE__ */ jsx(BoxFactory, { ref: rootRef, component: Form, variants: { root: variants.root }, ...rest, onSubmit: submit }) });
  })
);
FormFactory.displayName = "FormFactory";

function useBreakpoints(breakpoints) {
  const { useDimensions } = rbkGlobal.mapping;
  const theme = useTheme();
  const dimensions = useDimensions();
  const memoizedBreakpoints = useMemo(
    () => ({ ...theme.breakpoints, ...Object(breakpoints) }),
    [theme.breakpoints, breakpoints]
  );
  const getState = useCallback(
    () => Object.fromEntries(
      Object.entries(memoizedBreakpoints).sort((a, b) => a[1] - b[1]).map(([breakpoint, minWidth]) => [breakpoint, dimensions.width >= minWidth])
    ),
    [dimensions, memoizedBreakpoints]
  );
  const [state, setState] = useState(getState());
  useEffect(() => {
    const newState = getState();
    if (JSON.stringify(newState) !== JSON.stringify(state)) {
      setState(newState);
    }
  }, [dimensions, state, memoizedBreakpoints, getState]);
  return state;
}

function useDependentState(factory, deps) {
  const [state, setState] = useState(factory(void 0));
  useMemo(() => {
    setState(factory);
  }, deps);
  return [state, setState];
}

function event($el, event2, callback, options) {
  if (!callback.name) {
    const original = callback;
    const name = "fn_anonym_" + crypt(callback.toString());
    callback = {
      [name](e) {
        return original(e);
      }
    }[name];
  }
  $el.addEventListener(event2, callback, options);
  return () => $el.removeEventListener(event2, callback);
}

function useDraggable({ onDragStart, onDragMove, onDragEnd } = {}) {
  const { setDraggable } = useContext(RbkContext);
  const [isCapturing, setIsCapturing] = useState(false);
  const capture = useCallback(() => {
    setIsCapturing(true);
  }, []);
  const release = useCallback(() => {
    setIsCapturing(false);
    setDraggable(void 0);
  }, [setDraggable]);
  useEffect(() => {
    if (!Platform.web)
      return;
    if (!isCapturing)
      return;
    const removePointerStart = event(
      document,
      "pointermove",
      (e) => {
        e.preventDefault();
        e.stopPropagation();
        onDragStart?.(pointer(e));
      },
      { once: true }
    );
    const removePointerMove = event(document, "pointermove", (e) => {
      e.preventDefault();
      e.stopPropagation();
      onDragMove?.(pointer(e));
    });
    const removePointerUp = event(document, "pointerup", (e) => {
      e.preventDefault();
      e.stopPropagation();
      onDragEnd?.(pointer(e));
    });
    const removePointerCancel = event(document, "pointercancel", (e) => {
      e.preventDefault();
      e.stopPropagation();
      onDragEnd?.(pointer(e));
    });
    return () => {
      removePointerStart();
      removePointerMove();
      removePointerUp();
      removePointerCancel();
    };
  }, [isCapturing]);
  useEffect(() => {
    if (!Platform.native)
      return;
    if (!isCapturing)
      return;
    setDraggable({
      onStartShouldSetResponder: () => true,
      onMoveShouldSetResponder: () => true,
      onResponderGrant: (e) => {
        e.preventDefault();
        e.stopPropagation();
        onDragStart?.(pointer(e));
      },
      onResponderMove: (e) => {
        e.preventDefault();
        e.stopPropagation();
        onDragMove?.(pointer(e));
      },
      onResponderRelease: (e) => {
        e.preventDefault();
        e.stopPropagation();
        onDragEnd?.(pointer(e));
      },
      onResponderTerminate: (e) => {
        e.preventDefault();
        e.stopPropagation();
        onDragEnd?.(pointer(e));
      }
    });
  }, [isCapturing, setDraggable]);
  return useMemo(
    () => ({
      capture,
      release,
      isCapturing
    }),
    [capture, release, isCapturing]
  );
}

function useInput({
  prop = "value",
  name,
  value,
  defaultValue,
  controlled,
  editable = true,
  error: errorProp,
  mask = (value2) => `${value2 ?? ""}`,
  unmask = (value2) => value2,
  onChange,
  onFormChange
}) {
  const resolveValue = useCallback(
    (value2) => {
      const newValue = value2 === "" ? null : value2 ?? null;
      return ["string", "number"].includes(typeof newValue) ? mask(newValue) : newValue;
    },
    [mask]
  );
  const form = useForm();
  const [initialValue] = useState(value ?? defaultValue ?? form?.initialData?.[name]);
  const [_internal, _setInternal] = useState(initialValue);
  const [prevInternal, setPrevInternal] = useState();
  const [error, setError] = useState(errorProp);
  const internal = useMemo(
    () => resolveValue(controlled ? value : _internal),
    [_internal, controlled, resolveValue, value]
  );
  const setInternal = useCallback(
    (value2, event, ...rest) => {
      const newValue = resolveValue(value2);
      if (newValue !== internal) {
        setPrevInternal(internal);
        _setInternal(newValue);
        onChange?.(event, newValue, ...rest);
      }
    },
    [resolveValue, internal, onChange]
  );
  useMemo(() => {
    if (typeof value === "undefined")
      return;
    _setInternal((current) => {
      setPrevInternal(current);
      return resolveValue(value);
    });
  }, [value]);
  useMemo(() => {
    setError(errorProp);
  }, [errorProp]);
  useEffect(() => {
    if (!name || !form)
      return;
    form.setField({
      name,
      get: () => unmask(internal),
      set: setInternal,
      getError: () => error ?? null,
      setError,
      onFormChange,
      initialValue
    });
    return () => {
      form.unsetField(name);
    };
  }, [name, form, error, internal, initialValue, setInternal, onFormChange, unmask]);
  const reset = useCallback(() => {
    _setInternal(initialValue);
  }, [initialValue]);
  const clear = useCallback(() => {
    _setInternal(void 0);
  }, []);
  const handleChange = useCallback(
    (event) => {
      if (!editable)
        return;
      let value2 = event?.target?.value ?? event?.nativeEvent?.text;
      if (prop !== "value") {
        value2 = event?.target?.[prop] ?? event?.nativeEvent?.[prop];
      }
      setInternal(value2, event);
    },
    [prop, editable, setInternal]
  );
  return useMemo(
    () => ({
      state: internal,
      setState: setInternal,
      prevState: prevInternal,
      error: error || null,
      clear,
      reset,
      form,
      props: {
        name,
        value: mask(internal ?? ""),
        onChange: handleChange
      }
    }),
    [internal, setInternal, prevInternal, error, clear, reset, form, name, mask, handleChange]
  );
}

function usePropState(prop, initialState) {
  const [state, setState] = useState(typeof prop !== "undefined" ? prop : initialState);
  useEffect(() => {
    if (typeof prop !== "undefined") {
      setState(prop);
    }
  }, [prop]);
  return [state, setState];
}

function useToaster() {
  const { toasterRef } = useContext(RbkContext);
  const _open = useCallback(
    (options, color) => {
      const content = typeof options === "string" ? options : options?.content;
      const rest = typeof options === "string" ? {} : options;
      toasterRef?.current?.setProps({
        ...rest,
        content,
        color: color ?? rest?.color
      });
    },
    [toasterRef]
  );
  const close = useCallback(() => toasterRef?.current?.setProps(void 0), [toasterRef]);
  const open = useCallback((options) => _open(options), [_open]);
  const info = useCallback((options) => _open(options, "info"), [_open]);
  const success = useCallback((options) => _open(options, "success"), [_open]);
  const warning = useCallback((options) => _open(options, "warning"), [_open]);
  const error = useCallback((options) => _open(options, "error"), [_open]);
  const primary = useCallback((options) => _open(options, "primary"), [_open]);
  const secondary = useCallback(
    (options) => _open(options, "secondary"),
    [_open]
  );
  return { open, close, info, success, warning, error, primary, secondary };
}

function getSize(value) {
  const sizes = {
    xsmall: 1.25,
    small: 1.75,
    medium: 2.25,
    large: 2.75,
    xlarge: 3.25
  };
  if (typeof value === "string") {
    value = pick(value, "medium", sizes);
  }
  return typeof value === "number" ? value : sizes.medium;
}

function only(props, ...mixin) {
  let result = {};
  if (!Array.isArray(props)) {
    props = [props];
  }
  for (const item of mixin) {
    if (!item)
      continue;
    if (Array.isArray(item)) {
      result = { ...result, ...only(props, ...item) };
      continue;
    }
    for (const prop of props) {
      if (prop in item) {
        result[prop] = item[prop];
      }
    }
  }
  return result;
}

function remove(props, ...mixin) {
  if (!mixin || !props) {
    return;
  }
  if (!Array.isArray(props)) {
    props = [props];
  }
  for (const item of mixin) {
    if (!item)
      continue;
    if (Array.isArray(item)) {
      remove(props, ...item);
      return;
    }
    for (const prop of props) {
      delete item[prop];
    }
  }
}

function sleep(ms) {
  return new Promise((res) => setTimeout(res, ms));
}

async function rect($el) {
  const { web, native } = Platform;
  if ($el) {
    if (web) {
      const { offsetLeft: offsetX, offsetTop: offsetY } = $el;
      const { width, height, left: pageOffsetX, top: pageOffsetY } = $el.getBoundingClientRect();
      return { width, height, offsetX, offsetY, pageOffsetX, pageOffsetY };
    }
    if (native) {
      return new Promise(
        (resolve) => $el.measure(
          (offsetX, offsetY, width, height, pageOffsetX, pageOffsetY) => resolve({ width, height, offsetX, offsetY, pageOffsetX, pageOffsetY })
        )
      );
    }
  }
  return {};
}

function setNativeStyle($el, styles) {
  const { web, native } = Platform;
  if ($el) {
    if (web) {
      $el.style.cssText += `; ${css(styles)} ;`;
    }
    if (native) {
      $el.setNativeProps(jss(styles));
    }
  }
}

async function getFullHeight($el) {
  const { native } = Platform;
  const metricsINI = await rect($el);
  setNativeStyle($el, {
    height: "auto",
    opacity: 0,
    position: "absolute",
    width: metricsINI.width
  });
  if (native) {
    await sleep(0);
  }
  const metricsAuto = await rect($el);
  setNativeStyle($el, {
    height: metricsINI.height,
    opacity: 1,
    position: "relative",
    width: "auto"
  });
  if (native) {
    await sleep(0);
  }
  return metricsAuto.height;
}

async function getFullSize($el) {
  const { native } = Platform;
  const metricsINI = await rect($el);
  setNativeStyle($el, {
    height: "auto",
    opacity: 0,
    position: "absolute",
    width: "auto"
  });
  if (native) {
    await sleep(0);
  }
  const metricsAuto = await rect($el);
  setNativeStyle($el, {
    height: metricsINI.height,
    opacity: 1,
    position: "relative",
    width: metricsINI.height
  });
  if (native) {
    await sleep(0);
  }
  return metricsAuto;
}

function reference(ref, mirrorRef) {
  let newRef = ref ?? mirrorRef;
  if (ref && mirrorRef) {
    if (typeof ref === "function") {
      newRef = ($scope) => {
        ref($scope);
        mirrorRef.current = $scope;
      };
    } else {
      mirrorRef.current = ref.current;
    }
  }
  return newRef;
}

function scrollIntoView($scrollableEl, $targetEl, animated = true) {
  const { web, native } = Platform;
  if (web) {
    $targetEl.scrollIntoView({
      behavior: animated ? "smooth" : "instant",
      block: "start",
      inline: "start"
    });
  }
  if (native) {
    $targetEl.measureLayout($scrollableEl, (left, top) => {
      $scrollableEl.scrollTo({ x: left, y: top, animated });
    });
  }
}

function scrollTo($el, x, y, animated = true) {
  const { web, native } = Platform;
  if (web) {
    $el?.scrollTo({
      top: y,
      left: x,
      behavior: animated ? "smooth" : "instant"
    });
  }
  if (native) {
    $el?.scrollTo({ x, y, animated });
  }
}

function scrollToEnd($el, animated = true) {
  const { web, native } = Platform;
  if (web) {
    $el?.scrollTo({
      top: $el.scrollHeight,
      left: $el.scrollWidth,
      behavior: animated ? "smooth" : "instant"
    });
  }
  if (native) {
    $el?.scrollToEnd({ animated });
  }
}

function scrollToStart($el, animated = true) {
  const { web, native } = Platform;
  if (web) {
    $el?.scrollTo({
      top: 0,
      left: 0,
      behavior: animated ? "smooth" : "instant"
    });
  }
  if (native) {
    $el?.scrollTo({ x: 0, y: 0, animated });
  }
}

function PortalItem({ portalKey, children }) {
  const [, setPortals] = usePortals();
  useEffect(() => {
    setPortals((current) => ({ ...current, [portalKey]: children }));
  }, [portalKey, children, setPortals]);
  useEffect(() => {
    return () => {
      setPortals((current) => {
        delete current[portalKey];
        return { ...current };
      });
    };
  }, [portalKey, setPortals]);
  return /* @__PURE__ */ jsx(Fragment, {});
}
function createPortal(key, children) {
  return /* @__PURE__ */ jsx(PortalItem, { portalKey: key, children }, key);
}

const ImageFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Image;
    const { web, native, Image } = rbkGlobal.mapping;
    let {
      alt,
      circular,
      fallback,
      mode,
      source,
      // Sizes
      width,
      height,
      w,
      h,
      // Events
      onLayout,
      onLoad,
      onError,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const imageRef = useDefaultRef(ref);
    const widthFound = width ?? w ?? get("width", style) ?? get("w", style);
    const heightFound = height ?? h ?? get("height", style) ?? get("h", style);
    width = widthFound === true ? "100%" : widthFound;
    height = heightFound === true ? "100%" : heightFound;
    if (typeof width === "string" && width.endsWith("px")) {
      width = Number(width.replace(/px$/g, ""));
    }
    if (typeof height === "string" && height.endsWith("px")) {
      height = Number(height.replace(/px$/g, ""));
    }
    const [status, setStatus] = useState("loading");
    const [containerWidth, setContainerWidth] = useState();
    const [containerHeight, setContainerHeight] = useState();
    const [imgWidth, setImgWidth] = useState(null);
    const [aspectRatio, setAspectRatio] = useState(null);
    const [finalWidth, setFinalWidth] = useState(typeof width === "number" ? width : void 0);
    const [finalHeight, setFinalHeight] = useState(typeof height === "number" ? height : void 0);
    const isProcessed = [width, height].every((value) => typeof value === "number") || [finalWidth, finalHeight].every((value) => typeof value === "number");
    if (alt) {
      rest.accessibility = rest.accessibility || {};
      rest.accessibility.label = rest.accessibility.label ?? alt;
    }
    Object.assign(rest, {
      onLoad: (e) => handleLoad(e),
      onError: (err) => handleError(err)
    });
    if (web) {
      Object.assign(rest, {
        alt: alt ?? "",
        // @ts-expect-error
        src: source?.uri ?? source
      });
    }
    if (native) {
      Object.assign(rest, {
        source: typeof source === "string" ? { uri: source } : source,
        resizeMode: mode === "fill" ? "stretch" : mode,
        width: finalWidth,
        height: finalHeight
      });
    }
    const handleLayout = (e) => {
      setContainerWidth(e.nativeEvent.layout.width);
      setContainerHeight(e.nativeEvent.layout.height);
      onLayout?.(e);
    };
    const handleLoad = (e) => {
      setStatus("ready");
      onLoad?.(e);
    };
    const handleError = useCallback(
      (e) => {
        setStatus("error");
        onError?.(e);
      },
      [onError]
    );
    useEffect(() => {
      setStatus("loading");
    }, [source]);
    useEffect(() => {
      if (isProcessed || !native || !source)
        return;
      const asset = source?.uri ?? source;
      try {
        if (typeof asset === "number") {
          const image = Image.resolveAssetSource(asset);
          setImgWidth(image.width);
          setAspectRatio(image.height / (image.width || 1));
        } else {
          Image.getSize(
            asset,
            (width2, height2) => {
              setImgWidth(width2);
              setAspectRatio(height2 / (width2 || 1));
            },
            (err) => handleError(err)
          );
        }
      } catch (err) {
        handleError(err);
      }
    }, [source, width, height, isProcessed, Image, handleError, native]);
    useEffect(() => {
      if (isProcessed || !native || !defined(aspectRatio) || !defined(containerWidth) || !defined(containerHeight))
        return;
      let widthBase = typeof width === "number" ? width : null;
      let heightBase = typeof height === "number" ? height : null;
      const widthStr = `${width ?? ""}`.toLowerCase().trim();
      const heightStr = `${height ?? ""}`.toLowerCase().trim();
      if (widthBase === null) {
        if (widthStr.endsWith("px")) {
          widthBase = Number(widthStr.replace(/px$/g, ""));
        }
        if (widthStr.endsWith("%")) {
          const multiplier = Number(widthStr.replace(/%$/g, "")) / 100;
          widthBase = (containerWidth ?? 0) * multiplier;
        }
      }
      if (heightBase === null) {
        if (heightStr.endsWith("px")) {
          heightBase = Number(heightStr.replace(/px$/g, ""));
        }
        if (heightStr.endsWith("%")) {
          const multiplier = Number(heightStr.replace(/%$/g, "")) / 100;
          heightBase = (containerHeight ?? 0) * multiplier;
        }
      }
      if (widthBase === null && heightBase === null) {
        widthBase = containerWidth ?? imgWidth;
      }
      if (heightBase === null && widthBase !== null) {
        heightBase = widthBase * (aspectRatio || 0);
      }
      if (widthBase === null && heightBase !== null) {
        widthBase = heightBase / (aspectRatio || 1);
      }
      const newWidth = Number(widthBase ?? 0);
      const newHeight = Number(heightBase ?? 0);
      setFinalWidth(newWidth);
      setFinalHeight(newHeight);
    }, [width, height, containerWidth, containerHeight, imgWidth, aspectRatio, isProcessed, native]);
    const borderRadius = finalWidth ?? finalHeight ?? height ?? width;
    style = [
      { overflow: "hidden" },
      web && {
        objectFit: mode,
        height: height ?? void 0,
        width: width ?? void 0
      },
      native && !isProcessed && { flex: 1 },
      native && {
        height: isProcessed ? finalHeight : void 0,
        width: isProcessed ? finalWidth : void 0
      },
      circular && {
        web: { borderRadius: "50%" },
        native: { borderRadius: (typeof borderRadius === "number" ? borderRadius : 0) / 2 }
      },
      style
    ];
    if (status === "error") {
      if (defined(fallback)) {
        return typeof fallback === "function" ? fallback() : fallback;
      }
      return defined(alt) ? /* @__PURE__ */ jsx(TextFactory, { children: alt }) : null;
    }
    if (native) {
      return /* @__PURE__ */ jsx(BoxFactory, { style, variants: { root: variants.root }, onLayout: !isProcessed ? handleLayout : void 0, children: /* @__PURE__ */ jsx(BoxFactory, { ref: imageRef, component: Image, ...rest, noRootStyles: true }) });
    }
    return /* @__PURE__ */ jsx(
      BoxFactory,
      {
        ref: imageRef,
        component: Image,
        style,
        variants: { root: variants.root },
        ...rest,
        noRootStyles: true
      }
    );
  })
);
ImageFactory.displayName = "ImageFactory";

const AvatarFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Avatar;
    let {
      alt,
      color,
      corners,
      placeholder,
      size,
      source,
      // Styles
      variants,
      style,
      contentStyle,
      ...rest
    } = factory2(props, options);
    const [isLoadingImage, setIsLoadingImage] = useState(Boolean(source));
    const sourceKey = useMemo(() => Date.now(), [source]);
    extract(["width", "height", "minWidth", "minHeight", "w", "h", "ww", "hh", "minw", "minh"], rest, style);
    const borderRadius = size / 2;
    const absoluteFill = {
      position: "absolute",
      inset: 0
    };
    style = [
      style,
      {
        height: size,
        width: size,
        borderRadius
      }
    ];
    contentStyle = [
      {
        backgroundColor: color,
        borderRadius,
        corners
      },
      contentStyle
    ];
    return /* @__PURE__ */ jsxs(BoxFactory, { ref, variants: { root: variants.root }, style, ...rest, children: [
      /* @__PURE__ */ jsx(BoxFactory, { variants: { root: variants.content }, style: contentStyle, children: /* @__PURE__ */ jsxs(BoxFactory, { center: true, style: absoluteFill, children: [
        Boolean(source) && /* @__PURE__ */ jsx(
          ImageFactory,
          {
            w: size,
            h: size,
            alt,
            source,
            onLoad: () => setIsLoadingImage(false),
            animation: {
              throttle: 0,
              timing: "linear",
              iterations: 1,
              from: { opacity: 0 },
              to: { opacity: isLoadingImage ? 0 : 1 }
            }
          },
          sourceKey
        ),
        placeholder
      ] }) }),
      children
    ] });
  })
);
AvatarFactory.displayName = "AvatarFactory";

const BackdropFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Backdrop;
    const { web, native, Dialog, BackHandler } = rbkGlobal.mapping;
    const portalId = useHtmlId();
    const {
      keepMounted,
      visible,
      // Events
      onPress,
      // Styles
      variants,
      containerStyle,
      ...rest
    } = factory2(props, options);
    const rootRef = useDefaultRef(ref);
    const dialogRef = useRef();
    const visibleAfterAnim = useDeferredValue(visible, visible, 250);
    useEffect(() => {
      if (!web)
        return;
      if (visible) {
        setTimeout(() => rootRef.current?.focus?.(), 100);
        const top = document.documentElement.scrollTop;
        const left = document.documentElement.scrollLeft;
        window.onscroll = () => window.scrollTo({ top, left, behavior: "auto" });
      }
      return () => {
        window.onscroll = null;
      };
    }, [rootRef, visible, web]);
    useEffect(() => {
      if (!native)
        return;
      if (!visible)
        return;
      if (typeof onPress !== "function")
        return;
      const backHandler = BackHandler.addEventListener("hardwareBackPress", () => {
        onPress?.();
        return true;
      });
      return () => {
        backHandler.remove();
      };
    }, [native, visible, onPress]);
    if (!keepMounted && !visible && !visibleAfterAnim) {
      return null;
    }
    const $el = /* @__PURE__ */ jsx(
      BoxFactory,
      {
        noRootStyles: true,
        ref: dialogRef,
        animation: {
          to: { opacity: visible ? 1 : 0 },
          onStart: () => {
            if (visible) {
              setNativeStyle(dialogRef.current, { zIndex: theme.mixins.zIndex.backdrop });
            }
          },
          onEnd: () => {
            if (!visible) {
              setNativeStyle(dialogRef.current, { zIndex: -1 });
            }
          }
        },
        style: [
          {
            inset: 0,
            opacity: 0,
            zIndex: -1,
            pointerEvents: visible ? "auto" : "none",
            web: {
              position: "fixed",
              cursor: "auto !important"
            },
            native: {
              position: "absolute"
            }
          }
        ],
        children: /* @__PURE__ */ jsxs(BoxFactory, { ref: rootRef, variants: { root: variants.root }, tabIndex: visible ? "0" : "-1", ...rest, children: [
          /* @__PURE__ */ jsx(
            BoxFactory,
            {
              position: "absolute",
              i: 0,
              zIndex: 0,
              accessibility: {
                label: "close"
              },
              style: {
                cursor: "auto",
                web: { "& ~ *": { zIndex: 1 } }
              },
              onPress
            }
          ),
          children
        ] })
      }
    );
    return keepMounted ? createPortal(portalId, $el) : native ? /* @__PURE__ */ jsx(
      Dialog,
      {
        transparent: true,
        hardwareAccelerated: true,
        statusBarTranslucent: true,
        visible: Boolean(visible),
        animationType: "fade",
        presentationStyle: "overFullScreen",
        onRequestClose: (e) => onPress?.(pointer(e)),
        children: $el
      }
    ) : $el;
  })
);
BackdropFactory.displayName = "BackdropFactory";

const BadgeFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Badge;
    let {
      bottom,
      color,
      dot,
      left,
      right,
      size,
      top,
      value,
      // Styles,
      variants,
      labelStyle,
      style,
      ...rest
    } = factory2(props, options);
    if (typeof size === "string") {
      size = pick(size, "medium", {
        xsmall: 1.25,
        small: 1.75,
        medium: 2.25,
        large: 2.75,
        xlarge: 3.25
      });
    }
    const absolute = top || bottom || left || right;
    const baseSize = theme.rem(size) / 2;
    const halfSize = baseSize / 2;
    labelStyle = [
      {
        fontSize: halfSize
      },
      labelStyle
    ];
    style = [
      {
        backgroundColor: color,
        borderRadius: halfSize
      },
      absolute && {
        position: "absolute"
      },
      !dot && {
        minHeight: baseSize,
        minWidth: baseSize,
        paddingVertical: Math.round(baseSize * 0.222222),
        paddingHorizontal: Math.round(baseSize * 0.333333)
      },
      top && { top: -halfSize },
      bottom && { bottom: -halfSize },
      left && { left: -halfSize },
      right && { right: -halfSize },
      style
    ];
    return /* @__PURE__ */ jsx(BoxFactory, { ref, style, variants: { root: variants.root }, ...rest, children: !dot && /* @__PURE__ */ jsx(TextFactory, { style: labelStyle, variants: { root: variants.label }, children: value ?? children ?? "" }) });
  })
);
BadgeFactory.displayName = "BadgeFactory";

const LoadingFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Loading;
    const { Svg, Circle } = rbkGlobal.mapping.svg;
    let {
      duration,
      color,
      label,
      size,
      // Styles,
      variants,
      labelStyle,
      ...rest
    } = factory2(props, options);
    if (typeof size === "string") {
      size = pick(size, "medium", {
        xsmall: 0.25,
        small: 0.75,
        medium: 1.25,
        large: 1.75,
        xlarge: 1.25
      });
    }
    color = theme.color(color ?? "primary");
    size = size ?? options.defaultProps.size ?? 1;
    const fontSize = theme.rem(size - 0.25);
    const base = theme.rem(size);
    const c = base / 2;
    const w = c / 4;
    const r = c - w / 2;
    labelStyle = [
      {
        color,
        fontSize,
        ml: size * 1.6
      },
      labelStyle
    ];
    return /* @__PURE__ */ jsxs(BoxFactory, { ref, variants: { root: variants.root }, ...rest, children: [
      /* @__PURE__ */ jsx(
        BoxFactory,
        {
          animation: {
            throttle: 0,
            duration,
            timing: "linear",
            iterations: "infinite",
            from: { rotate: "0deg" },
            to: { rotate: "360deg" }
          },
          children: /* @__PURE__ */ jsxs(Svg, { viewBox: `0 0 ${base} ${base}`, height: base, width: base, children: [
            /* @__PURE__ */ jsx(
              Circle,
              {
                cx: c,
                cy: c,
                fill: "none",
                r,
                stroke: color,
                strokeWidth: w,
                strokeOpacity: 0.2
              }
            ),
            /* @__PURE__ */ jsx(
              Circle,
              {
                cx: c,
                cy: c,
                r,
                fill: "none",
                stroke: color,
                strokeWidth: w,
                strokeDasharray: base * 2.5,
                strokeDashoffset: base * 1.875
              }
            )
          ] })
        }
      ),
      Boolean(label) && /* @__PURE__ */ jsx(TextFactory, { style: labelStyle, variants: { root: variants.label }, children: label })
    ] });
  })
);
LoadingFactory.displayName = "LoadingFactory";

const ButtonFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Button;
    const { web, Button } = rbkGlobal.mapping;
    let {
      badge,
      circular,
      color,
      contrastColor,
      disabled,
      disabledColor,
      endAddon,
      label,
      loading,
      transform,
      type = "button",
      size,
      startAddon,
      variant,
      // Styles
      variants,
      labelStyle,
      contentStyle,
      style,
      ...rest
    } = factory2(
      props,
      options
    );
    const form = useForm();
    const isBasic = ["outline", "text"].includes(variant);
    children = children ?? label;
    color = disabled ? disabledColor : color;
    contrastColor = contrastColor ?? theme.color(theme.contrast(color), disabled ? 0.5 : 1);
    badge = typeof badge === "number" ? { value: badge } : badge;
    size = size ?? "medium";
    if (form && !props.onPress && !props.onClick) {
      if (type === "submit") {
        rest.onPress = form.submit;
      }
      if (type === "cancel") {
        type = "button";
        rest.onPress = form.cancel;
      }
      if (type === "reset") {
        type = "button";
        rest.onPress = form.reset;
      }
      if (type === "clear") {
        type = "button";
        rest.onPress = form.clear;
      }
    }
    const isSizeNumber = typeof size === "number";
    if (typeof size === "string") {
      size = pick(size, "medium", {
        xsmall: 1.25,
        small: 1.75,
        medium: 2.25,
        large: 2.75,
        xlarge: 3.25
      });
    }
    const baseSize = theme.rem(size);
    const spacing = (baseSize - theme.rem(0.75)) / 2;
    const textColor = theme.color(isBasic ? color : contrastColor);
    style = [
      isSizeNumber && {
        minHeight: baseSize,
        minWidth: baseSize,
        paddingHorizontal: spacing
      },
      circular && {
        borderRadius: baseSize / 2,
        paddingHorizontal: 0
      },
      color && variant !== "text" && { borderColor: color },
      color && !isBasic && { backgroundColor: color },
      web && color && {
        "&:hover": { bg: theme.color(color, isBasic ? 0.2 : 0.8) },
        "&:focus": { boxShadow: `0 0 0 4px ${theme.color(color, 0.3)}` }
      },
      style
    ];
    contentStyle = [
      { maxw: "100%" },
      startAddon && { ml: 1.5 },
      endAddon && { mr: 1.5 },
      loading && { opacity: 0 },
      contentStyle
    ];
    labelStyle = [{ color: textColor }, labelStyle];
    let childrenArray = childrenize(children);
    if (childrenArray.length && childrenArray.every((child) => ["string", "number"].includes(typeof child))) {
      childrenArray = [
        /* @__PURE__ */ jsx(TextFactory, { style: labelStyle, variants: { root: variants.label }, children: childrenArray.join("") }, 0)
      ];
    }
    return /* @__PURE__ */ jsxs(
      BoxFactory,
      {
        ref,
        component: web && rest.href ? "a" : Button,
        style,
        variants: { root: variants.root },
        ...rest,
        type,
        disabled: disabled || loading,
        children: [
          Boolean(startAddon) && /* @__PURE__ */ jsx(BoxFactory, { style: loading && { opacity: 0 }, children: typeof startAddon === "function" ? startAddon({ color: textColor }) : startAddon }),
          Boolean(childrenArray?.length) && /* @__PURE__ */ jsx(BoxFactory, { style: contentStyle, children: childrenArray }),
          Boolean(endAddon) && /* @__PURE__ */ jsx(BoxFactory, { style: loading && { opacity: 0 }, children: typeof endAddon === "function" ? endAddon({ color: textColor }) : endAddon }),
          loading && /* @__PURE__ */ jsx(BoxFactory, { position: "absolute", i: 0, center: true, children: /* @__PURE__ */ jsx(LoadingFactory, { color: textColor, size: size / 2 }) }),
          Boolean(badge) && /* @__PURE__ */ jsx(BadgeFactory, { top: true, right: true, ...badge })
        ]
      }
    );
  })
);
ButtonFactory.displayName = "ButtonFactory";

const ScrollableFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Scrollable;
    const { web, native, ios, RefreshControl, ScrollView } = rbkGlobal.mapping;
    let {
      contentInset: contentInsetProp,
      direction,
      hideScrollBar,
      pagingEnabled,
      // Refresh Control (Native)
      refreshControl,
      refreshing,
      onRefresh,
      // Styles
      variants,
      contentStyle,
      style,
      ...rest
    } = factory2(props, options);
    const isHorizontal = direction === "horizontal";
    const primaryColor = theme.color("primary");
    const contentInset = useMemo(
      () => ({
        // @ts-expect-error
        top: contentInsetProp?.top ?? contentInsetProp?.vertical ?? contentInsetProp ?? 0,
        // @ts-expect-error
        bottom: contentInsetProp?.bottom ?? contentInsetProp?.vertical ?? contentInsetProp ?? 0,
        // @ts-expect-error
        left: contentInsetProp?.left ?? contentInsetProp?.horizontal ?? contentInsetProp ?? 0,
        // @ts-expect-error
        right: contentInsetProp?.right ?? contentInsetProp?.horizontal ?? contentInsetProp ?? 0
      }),
      [contentInsetProp]
    );
    contentStyle = [
      extract([...flexContainerProps], style),
      {
        pt: contentInset?.top,
        pb: contentInset?.bottom,
        pl: contentInset?.left,
        pr: contentInset?.right
      },
      pagingEnabled && {
        web: {
          "&> *": { scrollSnapAlign: "start" }
        }
      },
      contentStyle
    ];
    style = [
      web && pagingEnabled && {
        scrollSnapType: `${isHorizontal ? "x" : "y"} mandatory`
      },
      style
    ];
    if (native) {
      rest = {
        contentInsetAdjustmentBehavior: "never",
        indicatorStyle: theme.mode === "dark" ? "white" : "black",
        keyboardDismissMode: ios ? "interactive" : "on-drag",
        keyboardShouldPersistTaps: "always",
        nestedScrollEnabled: true,
        pinchGestureEnabled: false,
        scrollEventThrottle: theme.rem(),
        scrollIndicatorInsets: isHorizontal ? { bottom: 1, left: 1 } : { top: 1, right: 1 },
        ...rest,
        pagingEnabled,
        horizontal: isHorizontal,
        contentContainerStyle: jss(variants.content, contentStyle)
      };
      if (hideScrollBar) {
        Object.assign(rest, {
          showsVerticalScrollIndicator: false,
          showsHorizontalScrollIndicator: false
        });
      }
      if (!refreshControl && (onRefresh || refreshing)) {
        refreshControl = /* @__PURE__ */ jsx(
          RefreshControl,
          {
            refreshing,
            onRefresh,
            colors: [primaryColor],
            tintColor: primaryColor,
            progressBackgroundColor: theme.color("background.primary")
          }
        );
      }
      if (refreshControl) {
        rest.refreshControl = refreshControl;
      }
    }
    return /* @__PURE__ */ jsx(
      BoxFactory,
      {
        ref,
        component: ScrollView,
        style,
        variants: { root: variants.root },
        ...rest,
        noRootStyles: true,
        children: native ? children : /* @__PURE__ */ jsx(BoxFactory, { style: contentStyle, variants: { root: variants.content }, noRootStyles: true, children })
      }
    );
  })
);
ScrollableFactory.displayName = "ScrollableFactory";

const ButtonGroupFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.ButtonGroup;
    const {
      color,
      disabled,
      loading,
      size,
      variant,
      contentStyle,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const childrenArray = childrenize(children);
    return /* @__PURE__ */ jsx(ScrollableFactory, { ref, variants: { root: variants.root }, ...rest, direction: "horizontal", children: /* @__PURE__ */ jsx(BoxFactory, { style: contentStyle, variants: { root: variants.content }, children: childrenArray.map((child, key) => {
      const length = Array.isArray(children) ? children.length : children ? 1 : 0;
      const isFirst = key === 0;
      const isLast = key === length - 1;
      const borderLeftWidth = length > 1 && !isFirst ? 0 : 1;
      const button = cloneElement(child, {
        color,
        disabled,
        loading,
        size,
        variant,
        ...child.props,
        mt: 0,
        mb: 0,
        ml: 0,
        mr: 0,
        style: [
          { borderLeftWidth, height: "auto", width: "auto" },
          // @ts-expect-error
          child.style,
          !isFirst && !isLast && { borderRadius: 0 },
          !isFirst && isLast && { borderTopLeftRadius: 0, borderBottomLeftRadius: 0 },
          isFirst && !isLast && { borderTopRightRadius: 0, borderBottomRightRadius: 0 }
        ]
      });
      return /* @__PURE__ */ jsx(React.Fragment, { children: button }, key);
    }) }) });
  })
);
ButtonGroupFactory.displayName = "ButtonGroupFactory";

function ChevronLeft({ svg, color = "#000000", size = 16 }) {
  const { Svg, Polyline } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(
    Polyline,
    {
      points: "160 208 80 128 160 48",
      fill: "none",
      stroke: color,
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeWidth: "16"
    }
  ) });
}

function ChevronRight({ svg, color = "#000000", size = 16 }) {
  const { Svg, Polyline } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(
    Polyline,
    {
      points: "96 48 176 128 96 208",
      fill: "none",
      stroke: color,
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeWidth: "16"
    }
  ) });
}

function dateify(input) {
  if (!input) {
    input = /* @__PURE__ */ new Date();
  }
  if (typeof input === "number") {
    input = new Date(input);
  }
  if (input instanceof Date) {
    input = [
      input.getFullYear(),
      `${input.getMonth() + 1}`.padStart(2, "0"),
      `${input.getDate()}`.padStart(2, "0")
    ].join("-");
  }
  return /* @__PURE__ */ new Date(`${input.substring(0, 10)}T12:00:00`);
}

const GridFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Grid;
    let {
      breakpoints,
      gap,
      size,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    gap = (gap === true ? 1 : gap ?? 0) * theme.shape.gap;
    const spacing = theme.spacing(gap / 2);
    const checkBreakpoints = breakpoints ?? theme.breakpoints;
    return /* @__PURE__ */ jsx(BoxFactory, { ref, style: [{ margin: -spacing }, style], variants: { root: variants.root }, ...rest, children: childrenize(children).map((child, index) => {
      if (!child && !["string", "number"].includes(typeof child)) {
        return null;
      }
      const props2 = { ...Object(child?.props) };
      const bkptStyle = {};
      Object.keys(checkBreakpoints).forEach((bkptName) => {
        if (bkptName in props2) {
          const bkptPropValue = props2[bkptName];
          const isSize = typeof bkptPropValue === "number";
          const isFlex = bkptPropValue === "flex" || bkptPropValue === true;
          const isHidden = bkptPropValue === "hide" || bkptPropValue === "hidden" || bkptPropValue === false;
          bkptStyle[bkptName] = {
            minWidth: 0,
            minHeight: 0,
            flexBasis: isFlex ? 0 : "auto",
            flexGrow: Number(isFlex),
            flexShrink: Number(isFlex),
            display: isHidden ? "none" : "flex",
            width: isSize ? `${bkptPropValue / size * 100}%` : "auto"
          };
        }
        delete props2[bkptName];
      });
      if (child?.type !== BoxFactory && Object.keys(props2).filter((prop) => !["ref", "key", "children"].includes(prop)).length > 0) {
        stdout.warn(
          `It's recommended to wrap the grid child in a "Box" to pass props in addition to breakpoints.`
        );
      }
      return /* @__PURE__ */ jsx(
        BoxFactory,
        {
          component: child?.type,
          ...props2,
          breakpoints,
          style: [props2.style, { padding: spacing }, bkptStyle],
          variants: { root: variants.item }
        },
        index
      );
    }) });
  })
);
GridFactory.displayName = "GridFactory";

function ChevronDown({ svg, color = "#000000", size = 16 }) {
  const { Svg, Polyline } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(
    Polyline,
    {
      points: "208 96 128 176 48 96",
      fill: "none",
      stroke: color,
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeWidth: "16"
    }
  ) });
}

function ChevronUp({ svg, color = "#000000", size = 16 }) {
  const { Svg, Polyline } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(
    Polyline,
    {
      points: "48 160 128 80 208 160",
      fill: "none",
      stroke: color,
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeWidth: "16"
    }
  ) });
}

const LabelFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Label;
    const { web, native, Label } = rbkGlobal.mapping;
    let {
      for: forProp,
      forRef,
      platform,
      // Styles
      variants,
      // Events
      onPress,
      ...rest
    } = factory2(props, options);
    if (web && forProp) {
      Object.assign(rest, {
        // @ts-expect-error
        htmlFor: forProp ?? forRef?.current?.id
      });
    }
    if (native && !onPress) {
      onPress = forRef?.current?.focus;
    }
    return /* @__PURE__ */ jsx(TextFactory, { ref, component: Label, variants: { root: variants.root }, ...rest, onPress });
  })
);
LabelFactory.displayName = "LabelFactory";

const InputBaseFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.InputBase;
    const { web, native, Input, TextArea } = rbkGlobal.mapping;
    let {
      autoCapitalize,
      autoCorrect,
      caretHidden,
      color,
      colorful,
      disabled,
      endAddon,
      error,
      hidden,
      hint,
      id,
      label,
      multiline,
      name,
      placeholderColor,
      readOnly,
      returnKeyType,
      rows,
      secure,
      selectionColor,
      selectTextOnFocus,
      size,
      startAddon,
      textColor,
      type,
      // Events
      onFocus,
      onBlur,
      onSubmit,
      onKeyDown,
      // Styles
      variants,
      contentStyle,
      errorStyle,
      hintStyle,
      inputStyle,
      labelStyle,
      style,
      ...rest
    } = factory2(
      props,
      options
    );
    const inputRef = useRef(null);
    const [focused, setFocused] = useState(false);
    id = useHtmlId(id);
    size = getSize(size);
    color = theme.color(error ? "error" : !focused && !colorful ? "gray.light" : color || "primary");
    selectionColor = theme.color(selectionColor ?? color);
    autoCapitalize = !autoCapitalize ? "none" : autoCapitalize;
    placeholderColor = theme.color(
      // @ts-expect-error
      placeholderColor ?? get("color", inputStyle) ?? options.defaultStyles.input?.color ?? "text.primary",
      0.4
    );
    const shadowColor = theme.color(color, 0.3);
    const baseSize = theme.rem(size);
    const spacing = (baseSize - theme.rem()) / 2;
    const height = multiline ? theme.rem() * (rows ?? 6) + spacing * 2 : baseSize;
    if (web) {
      Object.assign(rest, {
        autoCapitalize,
        autoCorrect: autoCorrect ? "on" : "off",
        enterKeyHint: returnKeyType,
        disabled,
        readOnly,
        type: pick(secure ? "password" : type, "default", {
          default: type,
          password: "password",
          phone: "tel",
          hidden: "text"
        }) || "text"
      });
    }
    if (native) {
      Object.assign(rest, {
        autoCapitalize,
        autoCorrect,
        caretHidden,
        multiline,
        editable: disabled ? false : !readOnly,
        keyboardAppearance: theme.mode,
        placeholderTextColor: placeholderColor,
        returnKeyType: returnKeyType === "default" ? "done" : returnKeyType,
        secureTextEntry: type === "password" || secure,
        selectionColor,
        selectTextOnFocus,
        cursorColor: selectionColor,
        underlineColorAndroid: "transparent",
        keyboardType: pick(type, "text", {
          text: "default",
          number: "number-pad",
          email: "email-address",
          phone: "phone-pad",
          url: "url",
          hidden: "default"
        })
      });
    }
    style = [extract(customStyleProps, rest), style];
    labelStyle = [error && { color: "error" }, labelStyle];
    hintStyle = [error && { color: "error" }, hintStyle];
    contentStyle = [
      !disabled && {
        borderColor: color
      },
      web && focused && {
        boxShadow: `0 0 0 4px ${shadowColor}`
      },
      contentStyle
    ];
    inputStyle = [
      {
        height,
        paddingHorizontal: spacing
      },
      textColor && { color: textColor },
      web && { paddingVertical: spacing },
      web && { caretColor: caretHidden ? theme.colors.common.trans : selectionColor },
      web && placeholderColor && {
        "&::-webkit-input-placeholder": { color: placeholderColor },
        "&::-ms-input-placeholder": { color: placeholderColor },
        "&::placeholder": { color: placeholderColor }
      },
      web && selectionColor && {
        "&::selection": { backgroundColor: selectionColor, color: theme.contrast(selectionColor) }
      },
      inputStyle
    ];
    const handleFocus = useCallback(
      (event) => {
        setFocused(true);
        if (web && selectTextOnFocus) {
          inputRef.current?.select();
        }
        onFocus?.(event);
      },
      [inputRef, selectTextOnFocus, onFocus, web]
    );
    const handleBlur = useCallback(
      (event) => {
        setFocused(false);
        onBlur?.(event);
      },
      [onBlur]
    );
    const handleSubmit = useCallback(
      (event) => {
        event?.preventDefault();
        onSubmit?.(event);
      },
      [onSubmit]
    );
    if (typeof onSubmit === "function" && !multiline) {
      if (web) {
        Object.assign(rest, {
          onKeyDown: (e) => e.key === "Enter" || e.keyCode === 13 ? handleSubmit(e) : onKeyDown?.(e)
        });
      }
      if (native) {
        Object.assign(rest, {
          onSubmitEditing: handleSubmit
        });
      }
    }
    return /* @__PURE__ */ jsxs(
      BoxFactory,
      {
        "data-rbk-input": name,
        hidden: hidden || type === "hidden",
        style,
        variants: { root: variants.root },
        children: [
          Boolean(label) && /* @__PURE__ */ jsx(
            LabelFactory,
            {
              numberOfLines: 1,
              for: id,
              forRef: inputRef,
              style: labelStyle,
              variants: { root: variants.label },
              onPress: native ? () => inputRef?.current?.focus?.() : void 0,
              children: label
            }
          ),
          /* @__PURE__ */ jsxs(BoxFactory, { style: contentStyle, variants: { root: variants.content }, children: [
            native && focused && /* @__PURE__ */ jsx(
              BoxFactory,
              {
                position: "absolute",
                pointerEvents: "none",
                border: `4px solid ${shadowColor}`,
                corners: get("corners", options.defaultStyles.content, contentStyle),
                borderRadius: get("borderRadius", options.defaultStyles.content, contentStyle),
                style: { inset: -4 }
              }
            ),
            /* @__PURE__ */ jsxs(BoxFactory, { row: true, noWrap: true, alignItems: "center", justifyContent: "space-between", style: { marginVertical: -1 }, children: [
              Boolean(startAddon) && /* @__PURE__ */ jsx(BoxFactory, { ml: `${spacing}px`, onPress: () => inputRef?.current?.focus?.(), children: typeof startAddon === "function" ? startAddon({ color }) : startAddon }),
              /* @__PURE__ */ jsx(
                BoxFactory,
                {
                  ref: reference(ref, inputRef),
                  component: multiline ? TextArea : Input,
                  ...rest,
                  noRootStyles: true,
                  id,
                  name,
                  style: inputStyle,
                  variants: { root: variants.input },
                  onFocus: handleFocus,
                  onBlur: handleBlur
                }
              ),
              Boolean(endAddon) && /* @__PURE__ */ jsx(BoxFactory, { mr: `${spacing}px`, onPress: () => inputRef?.current?.focus?.(), children: typeof endAddon === "function" ? endAddon({ color }) : endAddon })
            ] })
          ] }),
          Boolean(hint) && /* @__PURE__ */ jsx(TextFactory, { variant: "caption", style: hintStyle, variants: { root: variants.hint }, children: hint }),
          Boolean(error) && typeof error === "string" && /* @__PURE__ */ jsx(TextFactory, { variant: "caption", style: errorStyle, variants: { root: variants.error }, children: error })
        ]
      }
    );
  })
);
InputBaseFactory.displayName = "InputBaseFactory";

const InputFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Input;
    const { svg } = rbkGlobal.mapping;
    let {
      color,
      controlled,
      defaultValue,
      disabled,
      endAddon,
      error,
      mask,
      max,
      maxLength,
      min,
      name,
      notNull,
      readOnly,
      size,
      type,
      unmask,
      value,
      // Events
      onChange,
      onFocus,
      onBlur,
      onSubmit,
      onFormChange,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const inputRef = useDefaultRef(ref);
    const unmaskValue = useCallback(
      (value2) => {
        let emptyValue = "";
        if (maxLength) {
          value2 = `${value2 ?? ""}`.substring(0, maxLength);
        }
        if (type === "number") {
          emptyValue = 0;
          if (value2) {
            value2 = Number(value2);
          }
          if (defined(value2)) {
            if (defined(min) && value2 < min) {
              value2 = min;
            }
            if (defined(max) && value2 > max) {
              value2 = max;
            }
          }
        }
        if (typeof unmask === "function") {
          value2 = unmask(value2);
        }
        if (!notNull && [void 0, null, NaN, ""].includes(value2)) {
          return null;
        }
        return value2 ?? emptyValue;
      },
      [maxLength, type, unmask, notNull, min, max]
    );
    const input = useInput({
      name,
      value,
      defaultValue,
      error,
      controlled,
      editable: !disabled && !readOnly,
      mask,
      unmask: unmaskValue,
      onChange: (event, value2) => dispatchEvent("change", event, onChange, value2),
      onFormChange
    });
    size = getSize(size);
    color = theme.color(input.error ? "error" : color);
    const focus = useCallback(() => inputRef?.current?.focus?.(), [inputRef]);
    const blur = useCallback(() => inputRef?.current?.blur?.(), [inputRef]);
    const clear = useCallback(() => input.clear(), [input]);
    const reset = useCallback(() => input.reset(), [input]);
    const isFocused = useCallback(
      () => Boolean(inputRef?.current?.isFocused?.()) || inputRef?.current === document?.activeElement,
      [inputRef]
    );
    function dispatchEvent(type2, event, handler, value2) {
      if (typeof handler !== "function")
        return;
      value2 = typeof value2 === "undefined" ? input.state : value2;
      const form = input.form;
      const target = inputRef.current;
      return handler?.(
        {
          ...event,
          handler: "RbkInputEvent",
          type: type2,
          value: value2,
          name,
          form,
          focus,
          blur,
          clear,
          reset,
          isFocused,
          target
        },
        value2
      );
    }
    function handleFocus(event) {
      dispatchEvent("focus", event, onFocus);
    }
    function handleBlur(event) {
      dispatchEvent("blur", event, onBlur);
    }
    function handleSubmit(event) {
      dispatchEvent("submit", event, onSubmit ?? input.form?.submit);
    }
    function handleIncDec(event, signal) {
      if (disabled || readOnly)
        return;
      input.setState(Number(input.state || 0) + (signal || 1), event);
    }
    return /* @__PURE__ */ jsx(
      InputBaseFactory,
      {
        ref: inputRef,
        style,
        variants,
        ...rest,
        onFocus: handleFocus,
        onBlur: handleBlur,
        onSubmit: handleSubmit,
        color,
        disabled,
        maxLength,
        readOnly,
        size,
        type,
        error: input.error,
        ...input.props,
        endAddon: type !== "number" ? endAddon : /* @__PURE__ */ jsxs(BoxFactory, { row: true, noWrap: true, center: true, children: [
          /* @__PURE__ */ jsx(BoxFactory, { h: theme.rem(size), mr: endAddon ? 1 : -1, children: [1, -1].map((item) => {
            const isInc = item > 0;
            const Icon = isInc ? ChevronUp : ChevronDown;
            return /* @__PURE__ */ jsx(
              ButtonFactory,
              {
                variant: "text",
                color,
                size: size / 2,
                disabled,
                h: "50%",
                p: 0,
                accessibility: { label: isInc ? "plus" : "minus" },
                contentStyle: { align: isInc ? "end" : "start" },
                onPress: (e) => handleIncDec(e, item),
                children: /* @__PURE__ */ jsx(Icon, { svg, size: Math.round(theme.rem(size) * 0.45), color })
              },
              item
            );
          }) }),
          endAddon
        ] })
      }
    );
  })
);
InputFactory.displayName = "InputFactory";

const CalendarFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Calendar;
    const { locale, svg } = rbkGlobal.mapping;
    let {
      color,
      date,
      disableds,
      events,
      // Events
      onPressDate,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const today = dateify();
    color = theme.color(color || "primary");
    const [internal, setInternal] = useDependentState(() => dateify(date), [date]);
    const eventsDates = useMemo(() => events?.filter(Boolean)?.map((value) => dateify(value)) || [], [events]);
    const monthName = useMemo(() => internal.toLocaleDateString(locale, { month: "long" }), [internal, locale]);
    const daysInMonth = useMemo(
      () => new Date(internal.getFullYear(), internal.getMonth() + 1, 0).getDate(),
      [internal]
    );
    const monthDates = useMemo(() => {
      const newMonthDates = Array.from({ length: daysInMonth }).map((_, index) => {
        const date2 = dateify(internal);
        date2.setDate(index + 1);
        return date2;
      });
      while (newMonthDates[0].getDay() !== 0) {
        const date2 = dateify(newMonthDates[0]);
        date2.setDate(date2.getDate() - 1);
        newMonthDates.unshift(date2);
      }
      while (newMonthDates.length < 35 || newMonthDates.length % 7 !== 0) {
        const date2 = dateify(newMonthDates.slice(-1)[0]);
        date2.setDate(date2.getDate() + 1);
        newMonthDates.push(date2);
      }
      return newMonthDates;
    }, [daysInMonth, internal]);
    const handleYear = useCallback(
      (year) => {
        if (string(year).length !== 4)
          return;
        setInternal((current) => {
          const newDate = dateify(current.getTime());
          newDate.setFullYear(year);
          return newDate;
        });
      },
      [setInternal]
    );
    const handleMonth = useCallback(
      (sum) => {
        setInternal((current) => {
          const newDate = dateify(current.getTime());
          newDate.setMonth(newDate.getMonth() + sum);
          return newDate;
        });
      },
      [setInternal]
    );
    const handleDate = useCallback(
      (e, date2) => {
        setInternal(date2);
        onPressDate?.(e, date2);
      },
      [onPressDate, setInternal]
    );
    return /* @__PURE__ */ jsxs(CardFactory, { ref, variants: { root: variants.root }, ...rest, children: [
      /* @__PURE__ */ jsxs(GridFactory, { gap: true, noWrap: true, children: [
        /* @__PURE__ */ jsx(BoxFactory, { children: /* @__PURE__ */ jsx(
          ButtonFactory,
          {
            variant: "outline",
            color,
            onPress: () => handleMonth(-1),
            accessibility: { label: "previous month" },
            children: /* @__PURE__ */ jsx(ChevronLeft, { svg, color })
          }
        ) }),
        /* @__PURE__ */ jsx(BoxFactory, { xs: true, center: true, px: 0, children: /* @__PURE__ */ jsxs(TextFactory, { center: true, numberOfLines: 1, maxw: "100%", children: [
          monthName.substring(0, 1).toUpperCase(),
          monthName.substring(1).toLowerCase()
        ] }) }),
        /* @__PURE__ */ jsx(BoxFactory, { children: /* @__PURE__ */ jsx(
          ButtonFactory,
          {
            variant: "outline",
            color,
            onPress: () => handleMonth(1),
            accessibility: { label: "next month" },
            children: /* @__PURE__ */ jsx(ChevronRight, { svg, color })
          }
        ) }),
        /* @__PURE__ */ jsx(BoxFactory, { w: 112, children: /* @__PURE__ */ jsx(
          InputFactory,
          {
            colorful: true,
            controlled: true,
            type: "number",
            min: 1e3,
            color,
            value: internal.getFullYear(),
            inputStyle: { textAlign: "center" },
            onChange: (_, value) => handleYear(value),
            accessibility: { hint: "year" }
          }
        ) })
      ] }),
      /* @__PURE__ */ jsx(BoxFactory, { mt: theme.shape.gap * 2 }),
      /* @__PURE__ */ jsxs(GridFactory, { gap: 0.5, size: 7, children: [
        monthDates.slice(0, 7).map((date2) => /* @__PURE__ */ jsx(BoxFactory, { xs: 1, children: /* @__PURE__ */ jsx(TextFactory, { variant: "secondary", color: "text.disabled", bold: true, center: true, children: date2.toLocaleDateString(locale, { weekday: "narrow" }).split(/[,.]/g)?.[0] }) }, date2.toISOString())),
        monthDates.map((date2) => {
          const dateISO = date2.toISOString().substring(0, 10);
          const isMuted = date2.getMonth() !== internal.getMonth();
          const isToday = today.toISOString().substring(0, 10) === dateISO;
          const hasEvent = eventsDates.some((event) => dateify(event).toISOString().substring(0, 10) === dateISO);
          const isDisabled = Array.isArray(disableds) ? disableds.some((disabledDate) => dateify(disabledDate).toISOString().substring(0, 10) === dateISO) : disableds instanceof Function ? disableds(date2) : false;
          return /* @__PURE__ */ jsx(BoxFactory, { xs: 1, children: /* @__PURE__ */ jsx(
            ButtonFactory,
            {
              disabled: isDisabled,
              variant: !isMuted && hasEvent ? "solid" : "text",
              color: !isMuted && hasEvent ? color : "text.primary",
              style: [{ px: 0, py: 0 }, isToday && !hasEvent && { bg: "background.secondary" }],
              labelStyle: [{ w: 20 }, isMuted && { color: "text.disabled" }],
              onPress: (e) => handleDate(e, date2),
              accessibility: { value: { now: date2.getDate() } },
              "data-rbk-calendar": date2.getDate(),
              children: date2.getDate()
            }
          ) }, date2.toISOString());
        })
      ] })
    ] });
  })
);
CalendarFactory.displayName = "CalendarFactory";

const CarouselFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Carousel;
    const { web, native, svg, useDimensions } = rbkGlobal.mapping;
    const dimensions = useDimensions();
    const breakpoints = useBreakpoints();
    let {
      chevron,
      color,
      gap,
      pagingEnabled,
      pointerScroll,
      // Column Count
      xs,
      sm,
      md,
      lg,
      xl,
      // Styles
      variants,
      itemStyle,
      chevronStyle,
      style,
      ...rest
    } = factory2(
      props,
      options
    );
    const contentRef = useRef(null);
    const itemRef = useRef(null);
    const offsetRef = useRef(0);
    const [contentWidth, setContentWidth] = useState(null);
    const [hasPrev, setHasPrev] = useState(false);
    const [hasNext, setHasNext] = useState(false);
    const showChevron = chevron && chevron !== "hidden";
    gap = gap === true ? 1 : gap ?? 0;
    const base = contentWidth ?? 100;
    const xsValue = typeof xs === "number" || xs === "auto" ? xs : "auto";
    const smValue = typeof sm === "number" || sm === "auto" ? sm : xsValue;
    const mdValue = typeof md === "number" || md === "auto" ? md : smValue;
    const lgValue = typeof lg === "number" || lg === "auto" ? lg : mdValue;
    const xlValue = typeof xl === "number" || xl === "auto" ? xl : lgValue;
    const xsWidth = typeof xsValue === "number" ? base / Number(xsValue) : "auto";
    const smWidth = typeof smValue === "number" ? base / Number(smValue) : "auto";
    const mdWidth = typeof mdValue === "number" ? base / Number(mdValue) : "auto";
    const lgWidth = typeof lgValue === "number" ? base / Number(lgValue) : "auto";
    const xlWidth = typeof xlValue === "number" ? base / Number(xlValue) : "auto";
    itemStyle = [
      itemStyle,
      {
        xs: {
          width: xsWidth,
          ...xs && typeof xs === "object" ? xs : {}
        },
        sm: {
          width: smWidth,
          ...sm && typeof sm === "object" ? sm : {}
        },
        md: {
          width: mdWidth,
          ...md && typeof md === "object" ? md : {}
        },
        lg: {
          width: lgWidth,
          ...lg && typeof lg === "object" ? lg : {}
        },
        xl: {
          width: xlWidth,
          ...xl && typeof xl === "object" ? xl : {}
        }
      }
    ];
    const snapToInterval = breakpoints.xl ? xlWidth : breakpoints.lg ? lgWidth : breakpoints.md ? mdWidth : breakpoints.sm ? smWidth : xsWidth;
    const scrollByCount = useCallback(
      async (count) => {
        const size = (await rect(itemRef.current)).width;
        scrollTo(contentRef.current, count * size + offsetRef.current);
      },
      [contentWidth, web]
    );
    const scrollToPrev = useCallback(() => {
      scrollByCount(-1).catch(() => null);
    }, [scrollByCount]);
    const scrollToNext = useCallback(() => {
      scrollByCount(1).catch(() => null);
    }, [scrollByCount]);
    const handleScroll = useCallback(
      (e) => {
        let scrollLeft = 0;
        let scrollOverflowWidth = 0;
        if (web) {
          const $target = contentRef.current;
          scrollLeft = $target.scrollLeft;
          scrollOverflowWidth = $target.scrollWidth - $target.clientWidth;
        }
        if (native) {
          const { contentOffset, contentSize, layoutMeasurement } = e.nativeEvent;
          scrollLeft = contentOffset.x;
          scrollOverflowWidth = contentSize.width - layoutMeasurement.width;
        }
        offsetRef.current = scrollLeft;
        setHasPrev(scrollLeft - 10 > 0);
        setHasNext(scrollOverflowWidth > scrollLeft + 10);
      },
      [native, web]
    );
    useLayoutEffect(() => {
      if (!web || !contentRef.current)
        return;
      setContentWidth(contentRef.current.clientWidth);
    }, [dimensions.width, web]);
    useEffect(() => {
      if (!contentRef.current || !contentWidth)
        return;
      setTimeout(() => {
        scrollToEnd(contentRef.current, false);
        scrollToStart(contentRef.current, false);
      }, 0);
    }, [contentRef, contentWidth]);
    useEffect(() => {
      if (!web || !pointerScroll || !contentRef.current)
        return;
      return event(contentRef.current, "wheel", (e) => {
        if (e.deltaY > 0 && hasNext) {
          e.preventDefault();
          scrollToNext();
        }
        if (e.deltaY <= 0 && hasPrev) {
          e.preventDefault();
          scrollToPrev();
        }
      });
    }, [contentRef, hasPrev, hasNext, scrollToNext, scrollToPrev, pointerScroll, web]);
    if (native) {
      Object.assign(rest, {
        decelerationRate: "fast",
        onLayout: ({ nativeEvent }) => setContentWidth(nativeEvent.layout.width)
      });
    }
    const chevronProps = {
      color: theme.color(color),
      size: theme.rem()
    };
    return /* @__PURE__ */ jsxs(BoxFactory, { ref, style, variants: { root: variants.root }, ...rest, children: [
      /* @__PURE__ */ jsx(
        ScrollableFactory,
        {
          ref: contentRef,
          variants: { root: variants.content },
          pagingEnabled,
          hideScrollBar: true,
          direction: "horizontal",
          onScroll: handleScroll,
          ...native && typeof snapToInterval === "number" ? { snapToInterval } : {},
          children: contentWidth !== null && childrenize(children).map((child, index) => /* @__PURE__ */ jsx(BoxFactory, { ref: itemRef, style: itemStyle, px: theme.spacing(gap / 2), children: child }, index))
        }
      ),
      showChevron && hasPrev && /* @__PURE__ */ jsx(BoxFactory, { l: 0, variants: { root: variants.chevron }, children: typeof chevron === "function" ? chevron({ prev: true, next: false, color, onPress: scrollToPrev }) : /* @__PURE__ */ jsx(ButtonFactory, { variant: "outline", color, onPress: scrollToPrev, style: chevronStyle, children: /* @__PURE__ */ jsx(ChevronLeft, { svg, ...chevronProps }) }) }),
      showChevron && hasNext && /* @__PURE__ */ jsx(BoxFactory, { r: 0, variants: { root: variants.chevron }, children: typeof chevron === "function" ? chevron({ prev: false, next: true, color, onPress: scrollToNext }) : /* @__PURE__ */ jsx(ButtonFactory, { variant: "outline", color, onPress: scrollToNext, style: chevronStyle, children: /* @__PURE__ */ jsx(ChevronRight, { svg, ...chevronProps }) }) })
    ] });
  })
);
CarouselFactory.displayName = "CarouselFactory";

function Check({ svg, color = "#000000", size = 16 }) {
  const { Svg, Polyline } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(
    Polyline,
    {
      points: "40 144 96 200 224 72",
      fill: "none",
      stroke: color,
      strokeLinecap: "round",
      strokeLinejoin: "round",
      strokeWidth: "16"
    }
  ) });
}

const CheckboxFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Checkbox;
    const { native, svg } = rbkGlobal.mapping;
    let {
      checked,
      color,
      controlled,
      defaultChecked,
      disabled,
      error,
      id,
      label,
      name,
      readOnly,
      size,
      unique,
      value: valueProp,
      // Events
      onFocus,
      onBlur,
      onChange,
      onFormChange,
      // Styles
      variants,
      buttonStyle,
      errorStyle,
      labelStyle,
      style,
      ...rest
    } = factory2(props, options);
    const buttonRef = useDefaultRef(ref);
    const input = useInput({
      name,
      value: checked,
      defaultValue: defaultChecked,
      mask: (value) => value ? valueProp ?? true : false,
      unmask: (value) => value ? valueProp ?? true : false,
      error,
      controlled,
      editable: !disabled && !readOnly,
      onChange: (event, value) => dispatchEvent("change", event, onChange, value),
      onFormChange
    });
    id = useHtmlId(id);
    size = getSize(size);
    color = theme.color(input.error ? "error" : color);
    const baseSize = theme.rem(size);
    const fontSize = baseSize / 2;
    const halfSize = fontSize / 2;
    const focus = useCallback(() => buttonRef?.current?.focus?.(), [buttonRef]);
    const blur = useCallback(() => buttonRef?.current?.blur?.(), [buttonRef]);
    const clear = useCallback(() => input.clear(), [input]);
    const reset = useCallback(() => input.reset(), [input]);
    const isFocused = useCallback(
      () => Boolean(buttonRef?.current?.isFocused?.()) || buttonRef?.current === document?.activeElement,
      [buttonRef]
    );
    function dispatchEvent(type, event, handler, value) {
      if (typeof handler !== "function")
        return;
      value = typeof value === "undefined" ? input.state : value;
      const form = input.form;
      const target = buttonRef.current;
      return handler?.(
        {
          ...event,
          handler: "RbkInputEvent",
          type,
          value,
          name,
          form,
          focus,
          blur,
          clear,
          reset,
          isFocused,
          target
        },
        value
      );
    }
    function handleFocus(event) {
      dispatchEvent("focus", event, onFocus);
    }
    function handleBlur(event) {
      dispatchEvent("blur", event, onBlur);
    }
    function handleChange(event) {
      if (disabled || readOnly)
        return;
      input.setState(!input.state, event);
    }
    style = [style, extract(spacings, rest)];
    buttonStyle = [{ marginLeft: -theme.rem(0.5, fontSize) }, buttonStyle];
    return /* @__PURE__ */ jsxs(Fragment$1, { children: [
      /* @__PURE__ */ jsxs(BoxFactory, { "data-rbk-input": name, style, variants: { root: variants.root }, children: [
        /* @__PURE__ */ jsx(
          ButtonFactory,
          {
            ref: buttonRef,
            ...rest,
            circular: true,
            id,
            onFocus: handleFocus,
            onBlur: handleBlur,
            onPress: handleChange,
            color,
            disabled,
            size,
            variant: "text",
            style: buttonStyle,
            variants: { root: variants.button },
            accessibility: {
              label: typeof label === "string" ? label : void 0,
              role: unique ? "radio" : "checkbox",
              state: { checked: input.state }
            },
            children: /* @__PURE__ */ jsx(
              BoxFactory,
              {
                center: true,
                h: fontSize,
                w: fontSize,
                border: `2px solid ${color}`,
                borderRadius: unique ? halfSize : theme.shape.borderRadius / 2,
                bg: input.state && !unique ? color : void 0,
                children: Boolean(input.state) && /* @__PURE__ */ jsx(Fragment$1, { children: unique ? /* @__PURE__ */ jsx(svg.Svg, { viewBox: "0 0 256 256", height: "100%", width: "100%", children: /* @__PURE__ */ jsx(svg.Circle, { cx: "128", cy: "128", r: "80", fill: color }) }) : /* @__PURE__ */ jsx(Check, { svg, size: fontSize, color: theme.contrast(color) }) })
              }
            )
          }
        ),
        typeof label === "string" ? /* @__PURE__ */ jsx(
          LabelFactory,
          {
            for: id,
            forRef: buttonRef,
            style: [{ ml: 1 }, labelStyle],
            variants: { root: variants.label },
            onPress: native ? handleChange : void 0,
            children: label
          }
        ) : label
      ] }),
      Boolean(input.error) && typeof input.error === "string" && /* @__PURE__ */ jsx(TextFactory, { variant: "caption", style: errorStyle, variants: { root: variants.error }, children: input.error })
    ] });
  })
);
CheckboxFactory.displayName = "CheckboxFactory";

const CollapseFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Collapse;
    const { web, native } = rbkGlobal.mapping;
    const {
      delay,
      duration,
      timing,
      visible,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const defaultRef = useRef();
    const rootRef = ref || defaultRef;
    const transition = useAnimation({}, rootRef);
    const initialStyle = useMemo(
      () => ({
        height: visible ? "auto" : 0,
        overflow: visible ? "visible" : "hidden"
      }),
      // eslint-disable-next-line react-hooks/exhaustive-deps
      []
    );
    useEffect(() => {
      if (!rootRef.current)
        return;
      (async () => {
        const newSize = visible ? await getFullHeight(rootRef.current) : 0;
        const metrics = await rect(rootRef.current);
        if (newSize === metrics.height)
          return;
        setNativeStyle(rootRef.current, {
          overflow: "hidden"
        });
        await transition.start({
          throttle: 0,
          delay,
          timing,
          duration,
          web_useRawStyle: true,
          to: { height: newSize }
        });
        if (newSize > 0) {
          setNativeStyle(rootRef.current, {
            height: "auto",
            overflow: "visible"
          });
        }
      })();
    }, [rootRef, visible, delay, timing, duration, web, native]);
    return /* @__PURE__ */ jsx(
      BoxFactory,
      {
        noRootStyles: true,
        ...transition.props,
        platform: { native: { collapsable: false } },
        style: initialStyle,
        variants: { root: variants.root },
        rawStyle: transition.props.style,
        children: /* @__PURE__ */ jsx(BoxFactory, { ...rest })
      }
    );
  })
);
CollapseFactory.displayName = "CollapseFactory";

function Calendar({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M208,32H184V24a8,8,0,0,0-16,0v8H88V24a8,8,0,0,0-16,0v8H48A16,16,0,0,0,32,48V208a16,16,0,0,0,16,16H208a16,16,0,0,0,16-16V48A16,16,0,0,0,208,32ZM72,48v8a8,8,0,0,0,16,0V48h80v8a8,8,0,0,0,16,0V48h24V80H48V48ZM208,208H48V96H208V208Z" }) });
}

const DividerFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Divider;
    let {
      color,
      opacity,
      size,
      vertical,
      // Styles,
      variants,
      style,
      ...rest
    } = factory2(props, options);
    style = [
      {
        backgroundColor: color,
        opacity
      },
      vertical && { width: size },
      !vertical && { height: size },
      style
    ];
    return /* @__PURE__ */ jsx(BoxFactory, { ref, style, variants: { root: variants.root }, ...rest });
  })
);
DividerFactory.displayName = "DividerFactory";

const DropdownFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Dropdown;
    const { useDimensions } = rbkGlobal.mapping;
    const {
      keepMounted,
      placement,
      visible,
      triggerRef,
      // Events
      onClose,
      // Styles,
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const boxRef = useRef();
    const parentRef = triggerRef ?? boxRef;
    const dimensions = useDimensions();
    const [positions, setPositions] = useState({
      top: 0,
      left: 0,
      width: 0,
      height: 0
    });
    const extracted = extract(["i", "t", "l", "r", "b", "inset", "top", "left", "right", "bottom"], rest, style);
    const inset = extracted.i ?? extracted.inset;
    const top = extracted.t ?? extracted.top ?? inset;
    const left = extracted.l ?? extracted.left ?? inset;
    const right = extracted.r ?? extracted.right ?? inset;
    useEffect(() => {
      if (!visible || !parentRef?.current)
        return;
      (async () => {
        const metrics = await rect(parentRef.current);
        setPositions({
          top: metrics.pageOffsetY,
          left: metrics.pageOffsetX,
          height: metrics.height,
          width: metrics.width
        });
      })();
    }, [parentRef, visible, placement, dimensions]);
    return /* @__PURE__ */ jsx(BoxFactory, { ref: boxRef, children: /* @__PURE__ */ jsx(
      BackdropFactory,
      {
        visible,
        keepMounted,
        variants: { root: variants.backdrop },
        onPress: onClose,
        children: /* @__PURE__ */ jsx(BoxFactory, { ref, position: "absolute", style: positions, children: /* @__PURE__ */ jsx(
          BoxFactory,
          {
            variants: { root: variants.root },
            ...rest,
            style: [
              placement === "top" ? { bottom: "100%" } : { top: "100%" },
              defined(inset) && { inset },
              defined(top) && { top },
              defined(left) && { left },
              defined(right) && { right },
              style
            ],
            children
          }
        ) })
      }
    ) });
  })
);
DropdownFactory.displayName = "DropdownFactory";

const InputDateFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.InputDate;
    const { locale, svg } = rbkGlobal.mapping;
    let {
      color,
      controlled,
      defaultValue,
      disabled,
      error,
      format,
      label,
      locale: localeProp,
      max,
      min,
      name,
      readOnly,
      size,
      translate,
      value,
      variant,
      // Events
      onChange,
      onFocus,
      onBlur,
      onSubmit,
      onFormChange,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    variant = variant || "modal";
    const inputRef = useRef();
    const triggerRef = useRef();
    const Component = variant === "dropdown" ? DropdownFactory : variant === "inline" ? CollapseFactory : BackdropFactory;
    const [calendarVisible, setCalendarVisible] = useState(false);
    const resolveAsDate = useCallback(
      (value2) => {
        if (!value2) {
          return null;
        }
        let dateValue = dateify(value2);
        const dateMin = min ? dateify(min) : null;
        const dateMax = max ? dateify(max) : null;
        if (dateMin && dateValue < dateMin) {
          dateValue = dateMin;
        }
        if (dateMax && dateValue > dateMax) {
          dateValue = dateMax;
        }
        return dateValue ?? null;
      },
      [max, min]
    );
    const resolveAsISO = useCallback(
      (value2) => {
        return resolveAsDate(value2)?.toISOString()?.substring(0, 10) ?? null;
      },
      [resolveAsDate]
    );
    const resolveToFormat = useCallback(
      (value2) => {
        const dateValue = resolveAsDate(value2);
        if (!dateValue) {
          return "";
        }
        if (format || localeProp || locale) {
          return dateValue.toLocaleDateString(localeProp || locale, {
            month: "2-digit",
            day: "2-digit",
            year: "numeric",
            ...format
          });
        }
        return dateValue.toISOString().substring(0, 10);
      },
      [format, locale, localeProp, resolveAsDate]
    );
    const input = useInput({
      name,
      value,
      defaultValue,
      error,
      controlled,
      editable: !disabled && !readOnly,
      mask: (value2) => resolveToFormat(value2),
      unmask: (value2) => resolveAsISO(value2),
      onChange: (event, value2) => dispatchEvent("change", event, onChange, value2),
      onFormChange
    });
    size = getSize(size);
    color = theme.color(input.error ? "error" : color);
    const getDisabledDates = useCallback(
      (date) => {
        const currentDate = dateify(date);
        const minDate = min ? dateify(min) : null;
        const maxDate = max ? dateify(max) : null;
        if (minDate && currentDate < minDate) {
          return true;
        }
        if (maxDate && currentDate > maxDate) {
          return true;
        }
        return false;
      },
      [max, min]
    );
    const focus = useCallback(() => inputRef?.current?.focus?.(), [inputRef]);
    const blur = useCallback(() => inputRef?.current?.blur?.(), [inputRef]);
    const clear = useCallback(() => input.clear(), [input]);
    const reset = useCallback(() => input.reset(), [input]);
    const isFocused = useCallback(
      () => Boolean(inputRef?.current?.isFocused?.()) || inputRef?.current === document?.activeElement,
      [inputRef]
    );
    const dispatchEvent = useCallback(
      (type, event, handler, value2) => {
        if (typeof handler !== "function")
          return;
        value2 = typeof value2 === "undefined" ? input.state : value2;
        const form = input.form;
        const target = inputRef.current;
        return handler?.(
          {
            ...event,
            handler: "RbkInputEvent",
            type,
            value: value2,
            name,
            form,
            focus,
            blur,
            clear,
            reset,
            isFocused,
            target
          },
          value2
        );
      },
      [blur, clear, focus, input.form, input.state, isFocused, name, reset]
    );
    const handleFocus = useCallback(
      (event) => {
        dispatchEvent("focus", event, onFocus);
        if (!disabled && !readOnly) {
          setCalendarVisible(true);
        }
      },
      [disabled, dispatchEvent, onFocus, readOnly]
    );
    const handleBlur = useCallback(
      (event) => {
        dispatchEvent("blur", event, onBlur);
      },
      [dispatchEvent, onBlur]
    );
    const handleSubmit = useCallback(
      (event) => {
        dispatchEvent("submit", event, onSubmit);
      },
      [dispatchEvent, onSubmit]
    );
    const handleChange = useCallback(
      (event, date) => {
        if (disabled || readOnly)
          return;
        input.setState(resolveAsDate(date), event);
        setCalendarVisible(false);
      },
      [disabled, input, readOnly, resolveAsDate]
    );
    return /* @__PURE__ */ jsxs(Fragment$1, { children: [
      /* @__PURE__ */ jsx(
        InputBaseFactory,
        {
          ref: reference(ref, inputRef),
          style,
          variants,
          ...rest,
          readOnly: true,
          onFocus: handleFocus,
          onBlur: handleBlur,
          onSubmit: handleSubmit,
          color,
          disabled,
          size,
          error: input.error,
          label,
          ...input.props,
          endAddon: /* @__PURE__ */ jsx(
            ButtonFactory,
            {
              ref: triggerRef,
              variant: "text",
              color,
              size,
              disabled: disabled || readOnly,
              p: 0,
              mr: -theme.shape.gap / 2,
              accessibility: { label: "calendar" },
              onPress: () => setCalendarVisible((current) => !current),
              children: /* @__PURE__ */ jsx(Calendar, { svg, size: Math.round(theme.rem(size / 2)), color })
            }
          )
        }
      ),
      /* @__PURE__ */ jsx(
        Component,
        {
          visible: calendarVisible,
          ...variant === "dropdown" ? {
            triggerRef,
            r: 0,
            pt: 0.5,
            onClose: () => setCalendarVisible(false)
          } : variant === "inline" ? {
            alignItems: "end",
            p: 1
          } : {
            center: true,
            onPress: () => setCalendarVisible(false)
          },
          children: /* @__PURE__ */ jsxs(CardFactory, { minw: 320, maxw: 360, shadow: 1, p: 0, children: [
            variant === "modal" && Boolean(label) && /* @__PURE__ */ jsx(TextFactory, { bold: true, p: "1gap", pb: 0, children: label }),
            /* @__PURE__ */ jsx(BoxFactory, { h: 380, children: calendarVisible && /* @__PURE__ */ jsx(
              CalendarFactory,
              {
                shadow: 0,
                color,
                date: input.state,
                events: input.state ? [input.state] : [],
                onPressDate: handleChange,
                disableds: getDisabledDates
              }
            ) }),
            /* @__PURE__ */ jsx(DividerFactory, {}),
            /* @__PURE__ */ jsxs(GridFactory, { gap: true, m: 0, children: [
              /* @__PURE__ */ jsx(BoxFactory, { children: /* @__PURE__ */ jsx(
                ButtonFactory,
                {
                  variant: "text",
                  color,
                  size,
                  accessibility: { label: translate?.clear },
                  onPress: (e) => handleChange(e, null),
                  children: translate?.clear
                }
              ) }),
              /* @__PURE__ */ jsx(BoxFactory, { xs: true, p: 0 }),
              /* @__PURE__ */ jsx(BoxFactory, { children: /* @__PURE__ */ jsx(
                ButtonFactory,
                {
                  variant: "text",
                  color,
                  size,
                  accessibility: { label: translate?.cancel },
                  onPress: () => setCalendarVisible(false),
                  children: translate?.cancel
                }
              ) }),
              /* @__PURE__ */ jsx(BoxFactory, { children: /* @__PURE__ */ jsx(
                ButtonFactory,
                {
                  variant: "text",
                  color,
                  size,
                  accessibility: { label: translate?.today },
                  onPress: (e) => handleChange(e, dateify()),
                  children: translate?.today
                }
              ) })
            ] })
          ] })
        }
      )
    ] });
  })
);
InputDateFactory.displayName = "InputDateFactory";

const DrawerFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Drawer;
    const {
      keepMounted,
      placement,
      visible: drawerVisible,
      onClose,
      // Styles
      variants,
      style,
      backdropStyle,
      ...rest
    } = factory2(props, options);
    const hiddenStyle = {
      [placement]: "-100%"
    };
    const visibleStyle = {
      [placement]: "0%"
    };
    return /* @__PURE__ */ jsx(
      BackdropFactory,
      {
        visible: drawerVisible,
        keepMounted,
        onPress: onClose,
        style: backdropStyle,
        variants: { root: variants.backdrop },
        children: /* @__PURE__ */ jsx(
          BoxFactory,
          {
            ref,
            ...rest,
            style,
            variants: { root: variants.root },
            animation: {
              from: drawerVisible ? hiddenStyle : visibleStyle,
              to: drawerVisible ? visibleStyle : hiddenStyle
            },
            children
          },
          placement
        )
      }
    );
  })
);
DrawerFactory.displayName = "DrawerFactory";

const GrowBoxFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.GrowBox;
    const { native } = rbkGlobal.mapping;
    const {
      duration,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const rootRef = useDefaultRef(ref);
    const transition = useAnimation(
      {
        ...jss(style),
        height: 0,
        width: 0
      },
      rootRef
    );
    useEffect(() => {
      if (!rootRef.current)
        return;
      (async () => {
        const { height, width } = await getFullSize(rootRef.current);
        await transition.start({
          duration,
          to: { height, width }
        });
        setNativeStyle(rootRef.current, {
          height: "auto",
          width: "auto"
        });
      })();
    }, [rootRef, children, duration, native, transition]);
    return /* @__PURE__ */ jsx(BoxFactory, { ...transition.props, variants: { root: variants.root }, ...rest, children });
  })
);
GrowBoxFactory.displayName = "GrowBoxFactory";

const InputPinFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.InputPin;
    const { web, ios } = rbkGlobal.mapping;
    let {
      length,
      placeholder,
      size,
      type,
      // Styles
      variants,
      inputStyle,
      style,
      ...rest
    } = factory2(props, options);
    size = getSize(size);
    const resolveValue = useCallback(
      (value) => {
        value = string(value).replace(/[^0-9a-zA-Z]/g, "");
        if (type === "alphabetic") {
          value = value.replace(/[^a-zA-Z]/g, "");
        }
        if (type === "numeric") {
          value = value.replace(/\D/g, "");
        }
        return value.slice(0, length);
      },
      [type, length]
    );
    inputStyle = [
      {
        fontFamily: web ? '"Courier New", monospace' : ios ? "Courier New" : "monospace",
        letterSpacing: theme.spacing(size)
      },
      inputStyle
    ];
    return /* @__PURE__ */ jsx(
      InputFactory,
      {
        ref,
        style,
        variants,
        ...rest,
        size,
        mask: resolveValue,
        unmask: resolveValue,
        placeholder: "".padEnd(length, placeholder ?? "_"),
        inputStyle
      }
    );
  })
);
InputPinFactory.displayName = "InputPinFactory";

const LinkFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Link;
    const { Link } = rbkGlobal.mapping;
    const {
      underline,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    return /* @__PURE__ */ jsx(TextFactory, { ref, component: Link, variants: { root: variants.root }, ...rest });
  })
);
LinkFactory.displayName = "LinkFactory";

const ListFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.List;
    const { web, native, View } = rbkGlobal.mapping;
    const {
      direction,
      renderDelay,
      renderOffset,
      rowHeight,
      rowWidth,
      rowFallbackComponent = View,
      // Events
      onScroll,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const scrollRef = useDefaultRef(ref);
    const offsetTopRef = useRef(0);
    const offsetLeftRef = useRef(0);
    const childrenArray = useMemo(() => childrenize(children), [children]);
    const [sticky, setSticky] = useState([]);
    const [visible, setVisible] = useState([]);
    if (native) {
      rest.stickyHeaderIndices = sticky;
    }
    const render = useCallback(
      ({ clientHeight = 0, clientWidth, offsetTop = 0, offsetLeft = 0 }) => {
        const newVisible = [];
        const newSticky = [];
        let curPosX = 0;
        let curPosY = 0;
        for (const index in childrenArray) {
          const child = childrenArray[index];
          const width = child?.props?.width ?? rowWidth ?? 0;
          const height = child?.props?.height ?? rowHeight ?? 0;
          if (direction === "vertical") {
            curPosY += height;
            const minPosY = offsetTop - height - (renderOffset ?? 0);
            const maxPosY = clientHeight + offsetTop + height + (renderOffset ?? 0);
            if (curPosY >= minPosY && curPosY <= maxPosY) {
              newVisible.push(Number(index));
            }
          }
          if (direction === "horizontal") {
            curPosX += width;
            const minPosX = offsetLeft - width - (renderOffset ?? 0);
            const maxPosX = clientWidth + offsetLeft + width + (renderOffset ?? 0);
            if (curPosX >= minPosX && curPosX <= maxPosX) {
              newVisible.push(Number(index));
            }
          }
          if (child?.props?.sticky || get("position", child?.props, child?.props?.style) === "sticky") {
            newSticky.push(Number(index));
          }
        }
        setVisible(newVisible);
        setSticky(newSticky);
      },
      [childrenArray, rowHeight, rowWidth, renderOffset]
    );
    useEffect(() => {
      if (!scrollRef.current)
        return;
      (async () => {
        await sleep(renderDelay ?? 0);
        const { width, height } = await rect(scrollRef.current);
        render({
          clientHeight: height,
          clientWidth: width,
          offsetTop: offsetTopRef.current,
          offsetLeft: offsetLeftRef.current
        });
      })();
    }, [scrollRef, childrenArray]);
    const handleScroll = useCallback(
      (event) => {
        const { target, nativeEvent } = event;
        const clientWidth = target?.clientWidth ?? nativeEvent?.layoutMeasurement?.width ?? 0;
        const clientHeight = target?.clientHeight ?? nativeEvent?.layoutMeasurement?.height ?? 0;
        const offsetTop = target?.scrollTop ?? nativeEvent?.contentOffset?.y ?? 0;
        const offsetLeft = target?.scrollLeft ?? nativeEvent?.contentOffset?.x ?? 0;
        offsetTopRef.current = offsetTop;
        offsetLeftRef.current = offsetLeft;
        render({ clientHeight, clientWidth, offsetTop, offsetLeft });
        onScroll?.(event);
      },
      [render, onScroll]
    );
    return /* @__PURE__ */ jsx(
      ScrollableFactory,
      {
        ref: scrollRef,
        variants: { root: variants.root },
        direction,
        ...rest,
        onScroll: handleScroll,
        children: childrenArray.map((child, index) => {
          const isVisible = visible.includes(index);
          const isSticky = sticky.includes(index);
          if ((isVisible || isSticky) && isValidElement(child)) {
            return cloneElement(child, {
              key: index,
              // @ts-expect-error
              sticky: void 0,
              position: void 0,
              ...web && isSticky ? { position: "sticky", top: 0 } : {}
            });
          }
          const Component = rowFallbackComponent;
          const ref2 = child?.ref;
          const height = child?.props?.height ?? rowHeight;
          const width = child?.props?.width ?? rowWidth;
          return /* @__PURE__ */ jsx(Component, { ref: ref2, style: { height, width } }, index);
        })
      }
    );
  })
);
ListFactory.displayName = "ListFactory";

const ListItemFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.ListItem;
    const { svg, View } = rbkGlobal.mapping;
    let {
      chevron,
      endAddon,
      gap,
      icon,
      startAddon,
      // Styles
      variants,
      chevronStyle,
      ...rest
    } = factory2(props, options);
    gap = gap === true ? 1 : gap ?? 0;
    const chevronSize = get("size", chevronStyle);
    const chevronColor = get("color", chevronStyle) ?? "primary";
    remove(["size", "color"], chevronStyle);
    chevron = chevron === true ? /* @__PURE__ */ jsx(ChevronRight, { svg, color: theme.color(chevronColor), size: theme.rem(chevronSize || 1) }) : chevron ? /* @__PURE__ */ jsx(TextFactory, { color: chevronColor, size: chevronSize, children: chevron }) : null;
    return /* @__PURE__ */ jsx(CardFactory, { ref, p: theme.spacing(gap), variants: { root: variants.root }, ...rest, children: /* @__PURE__ */ jsxs(GridFactory, { row: true, noWrap: true, alignItems: "center", gap, children: [
      Boolean(startAddon) && /* @__PURE__ */ jsx(BoxFactory, { children: startAddon }),
      /* @__PURE__ */ jsx(View, { xs: true, children: /* @__PURE__ */ jsx(GridFactory, { row: true, noWrap: true, alignItems: "center", gap, children }) }),
      Boolean(endAddon) && /* @__PURE__ */ jsx(BoxFactory, { children: endAddon }),
      Boolean(chevron) && /* @__PURE__ */ jsx(BoxFactory, { style: chevronStyle, children: chevron })
    ] }) });
  })
);
ListItemFactory.displayName = "ListItemFactory";

const ModalFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Modal;
    const {
      keepMounted,
      animation,
      halign,
      valign,
      visible,
      onClose,
      // Styles
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const animStyle = {
      fade: {
        from: { opacity: 0.5 },
        to: { opacity: 1 }
      },
      "zoom-in": {
        from: { transform: { scale: 0.5 } },
        to: { transform: { scale: 1 } }
      },
      "zoom-out": {
        from: { transform: { scale: 1.5 } },
        to: { transform: { scale: 1 } }
      },
      "slide-top": {
        from: { transform: { translateY: 80 } },
        to: { transform: { translateY: 0 } }
      },
      "slide-bottom": {
        from: { transform: { translateY: -80 } },
        to: { transform: { translateY: 0 } }
      },
      "slide-left": {
        from: { transform: { translateX: 80 } },
        to: { transform: { translateX: 0 } }
      },
      "slide-right": {
        from: { transform: { translateX: -80 } },
        to: { transform: { translateX: 0 } }
      }
    }[animation ?? "fade"];
    return /* @__PURE__ */ jsx(
      BackdropFactory,
      {
        visible,
        keepMounted,
        variants: { root: variants.backdrop },
        onPress: onClose,
        children: /* @__PURE__ */ jsx(
          CardFactory,
          {
            ref,
            variants: { root: variants.root },
            ...rest,
            animation: {
              timing: visible ? "ease-out" : "ease-in",
              from: visible ? animStyle.from : animStyle.to,
              to: visible ? animStyle.to : animStyle.from
            },
            children
          }
        )
      }
    );
  })
);
ModalFactory.displayName = "ModalFactory";

const OutlineFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Outline;
    const { web, native } = rbkGlobal.mapping;
    const [focused, setFocused] = useState(false);
    let {
      color,
      size,
      visible,
      // Styles
      variants,
      style,
      // Events
      onFocus,
      onBlur,
      ...rest
    } = factory2(props, options);
    color = theme.color(color, 0.3);
    const boxShadow = `0 0 0 ${size}px ${color}`;
    let handleFocus;
    let handleBlur;
    if (native && visible === "auto") {
      handleFocus = (...args) => {
        setFocused(true);
        onFocus?.(...args);
      };
      handleBlur = (...args) => {
        setFocused(false);
        onBlur?.(...args);
      };
    }
    style = [
      { position: "relative" },
      web && color && visible === true && { boxShadow },
      web && color && visible === "auto" && { "&:focus": { boxShadow } },
      style
    ];
    const borderRadiusStyle = extract(
      [
        "corners",
        "borderRadius",
        "borderTopStartRadius",
        "borderTopEndRadius",
        "borderTopLeftRadius",
        "borderTopRightRadius",
        "borderBottomStartRadius",
        "borderBottomEndRadius",
        "borderBottomLeftRadius",
        "borderBottomRightRadius"
      ],
      clone(rest),
      clone(style)
    );
    return /* @__PURE__ */ jsxs(
      BoxFactory,
      {
        ref,
        style,
        variants: { root: variants.root },
        ...rest,
        onFocus: handleFocus ?? onFocus,
        onBlur: handleBlur ?? onBlur,
        children: [
          native && (visible === true || focused) && /* @__PURE__ */ jsx(
            BoxFactory,
            {
              position: "absolute",
              border: `${size}px solid ${color}`,
              style: [borderRadiusStyle, { inset: -size }]
            }
          ),
          children
        ]
      }
    );
  })
);
OutlineFactory.displayName = "OutlineFactory";

const ProgressFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Progress;
    const { native } = rbkGlobal.mapping;
    let {
      color,
      label,
      size,
      value,
      // Events
      onLayout,
      // Styles
      variants,
      style,
      barStyle,
      labelStyle,
      ...rest
    } = factory2(props, options);
    const [containerWidth, setContainerWidth] = useState(0);
    if (typeof size === "string") {
      size = pick(size, "medium", {
        xsmall: 0.25,
        small: 0.75,
        medium: 1.25,
        large: 1.75,
        xlarge: 1.25
      });
    }
    const fontSize = theme.rem(size * 0.7);
    const isIndeterminate = typeof value !== "number";
    const height = theme.rem(size);
    const translateX = native ? containerWidth * 0.8 : "80%";
    if (!isIndeterminate) {
      rest.accessibility = deepmerge({ label: "percentage" }, rest.accessibility, { value: { now: value } });
    }
    style = [
      {
        corners: size * 2
      },
      style
    ];
    barStyle = [
      {
        bg: color,
        height,
        width: 0
      },
      barStyle
    ];
    if (native) {
      Object.assign(rest, { onLayout: handleLayout });
    }
    function handleLayout(e) {
      setContainerWidth(e.nativeEvent.layout.width);
      onLayout?.(e);
    }
    return /* @__PURE__ */ jsx(BoxFactory, { ref, ...rest, style, variants: { root: variants.root }, children: isIndeterminate ? /* @__PURE__ */ jsx(
      BoxFactory,
      {
        animation: {
          throttle: 0,
          duration: 500,
          timing: "linear",
          boomerang: true,
          iterations: "infinite",
          from: { transform: { translateX: 0 } },
          to: { transform: { translateX } }
        },
        children: /* @__PURE__ */ jsx(BoxFactory, { w: "20%", style: barStyle, variants: { root: variants.bar } })
      }
    ) : /* @__PURE__ */ jsxs(Fragment$1, { children: [
      /* @__PURE__ */ jsx(
        BoxFactory,
        {
          style: barStyle,
          variants: { root: variants.bar },
          animation: {
            throttle: 0,
            duration: 100,
            timing: "linear",
            iterations: 1,
            to: { width: `${value}%` }
          }
        }
      ),
      Boolean(label) && /* @__PURE__ */ jsx(BoxFactory, { position: "absolute", i: 0, center: true, children: typeof label === "function" ? label?.(value) : /* @__PURE__ */ jsxs(TextFactory, { style: [{ fontSize }, labelStyle], variants: { root: variants.label }, children: [
        Math.round(value),
        "%"
      ] }) })
    ] }) });
  })
);
ProgressFactory.displayName = "ProgressFactory";

function Resize({ svg, color = "#000000", size = 16 }) {
  const { Svg, Line, Rect } = svg;
  return /* @__PURE__ */ jsxs(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: [
    /* @__PURE__ */ jsx(Rect, { width: "256", height: "256", fill: "none" }),
    /* @__PURE__ */ jsx(
      Line,
      {
        x1: "208",
        y1: "128",
        x2: "128",
        y2: "208",
        fill: "none",
        stroke: color,
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "16"
      }
    ),
    /* @__PURE__ */ jsx(
      Line,
      {
        x1: "192",
        y1: "40",
        x2: "40",
        y2: "192",
        fill: "none",
        stroke: color,
        strokeLinecap: "round",
        strokeLinejoin: "round",
        strokeWidth: "16"
      }
    )
  ] });
}

const ResizableFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Resizable;
    const { web, svg } = rbkGlobal.mapping;
    let {
      horizontal,
      vertical,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const both = horizontal && vertical || typeof horizontal === "undefined" && typeof vertical === "undefined";
    horizontal = horizontal ?? both;
    vertical = vertical ?? both;
    const rootRef = useDefaultRef(ref);
    const metricsRef = useRef();
    const initEventRef = useRef();
    const { capture, release, isCapturing } = useDraggable({
      onDragMove: (event) => {
        if (horizontal) {
          setNativeStyle(rootRef.current, {
            width: metricsRef.current.width + (event.pageX - initEventRef.current.pageX)
          });
        }
        if (vertical) {
          setNativeStyle(rootRef.current, {
            height: metricsRef.current.height + (event.pageY - initEventRef.current.pageY)
          });
        }
      },
      onDragEnd: () => {
        release();
      }
    });
    useEffect(() => {
      if (!isCapturing)
        return;
      return () => {
        release();
      };
    }, [isCapturing, release]);
    const handleCapture = useCallback(
      async (event) => {
        initEventRef.current = event;
        metricsRef.current = await rect(rootRef.current);
        capture();
      },
      [rootRef, capture]
    );
    return /* @__PURE__ */ jsxs(BoxFactory, { ref: rootRef, variants: { root: variants.root }, ...rest, children: [
      children,
      /* @__PURE__ */ jsx(
        BoxFactory,
        {
          position: "absolute",
          alignItems: "end",
          justifyContent: "end",
          r: 0,
          b: 0,
          h: 24,
          w: 24,
          onPressIn: handleCapture,
          style: [
            web && horizontal && { cursor: "e-resize" },
            web && vertical && { cursor: "s-resize" },
            web && both && { cursor: "se-resize" }
          ],
          children: /* @__PURE__ */ jsx(Resize, { svg, color: theme.color(isCapturing ? "info" : "text") })
        }
      )
    ] });
  })
);
ResizableFactory.displayName = "ResizableFactory";

function MagnifyingGlass({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M229.66,218.34l-50.07-50.06a88.11,88.11,0,1,0-11.31,11.31l50.06,50.07a8,8,0,0,0,11.32-11.32ZM40,112a72,72,0,1,1,72,72A72.08,72.08,0,0,1,40,112Z" }) });
}

const SelectFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Select;
    const { web, native, svg, useDimensions } = rbkGlobal.mapping;
    const dimensions = useDimensions();
    let {
      accessibility,
      defaultValue,
      disabled,
      id,
      color,
      colorful,
      controlled,
      error,
      label,
      loading,
      name,
      options: arrOptions = [],
      placeholder,
      readOnly,
      searchCount,
      size,
      value,
      // Events
      onFocus,
      onBlur,
      onChange,
      onFormChange,
      // Styles
      variants,
      contentStyle,
      buttonStyle,
      errorStyle,
      labelStyle,
      style,
      ...rest
    } = factory2(props, options);
    const buttonRef = useDefaultRef(ref);
    const scrollRef = useRef();
    const selectedRef = useRef();
    const optionsRef = useRef([]);
    const input = useInput({
      name,
      value,
      defaultValue,
      error,
      controlled,
      editable: !disabled && !readOnly,
      onChange: (event, value2) => dispatchEvent("change", event, onChange, value2),
      onFormChange
    });
    const [focused, setFocused] = useState(false);
    const [metrics, setMetrics] = useState({});
    const [visible, _setVisible] = useState(false);
    const [activeIndex, setActiveIndex] = useState(-1);
    const [search, setSearch] = useState("");
    const hasSearch = Number(arrOptions?.length ?? 0) >= searchCount;
    const setVisible = useCallback((value2) => {
      _setVisible(value2);
      setFocused(value2);
      setSearch("");
    }, []);
    const selected = useMemo(() => arrOptions?.find((item) => item.value == input.state), [arrOptions, input.state]);
    id = useHtmlId(id);
    size = getSize(size);
    color = theme.color(input.error ? "error" : !focused && !colorful ? "gray.light" : color || "primary");
    accessibility = deepmerge({ label: label ?? placeholder }, accessibility, { state: { expanded: visible } });
    const baseSize = theme.rem(size);
    const fontSize = baseSize / 2;
    const spacing = (baseSize - theme.rem(0.75)) / 2;
    const displayLabel = selected?.label ?? selected?.value ?? placeholder ?? "";
    const focus = useCallback(() => buttonRef?.current?.focus?.(), [buttonRef]);
    const blur = useCallback(() => buttonRef?.current?.blur?.(), [buttonRef]);
    const clear = useCallback(() => input.clear(), [input]);
    const reset = useCallback(() => input.reset(), [input]);
    const isFocused = useCallback(
      () => Boolean(buttonRef?.current?.isFocused?.()) || buttonRef?.current === document?.activeElement,
      [buttonRef]
    );
    function dispatchEvent(type, event, handler, value2) {
      if (typeof handler !== "function")
        return;
      value2 = typeof value2 === "undefined" ? input.state : value2;
      const form = input.form;
      const target = buttonRef.current;
      const option = arrOptions?.find((item) => item.value == value2) ?? null;
      return handler?.(
        {
          ...event,
          handler: "RbkInputEvent",
          type,
          value: value2,
          name,
          form,
          focus,
          blur,
          clear,
          reset,
          isFocused,
          target
        },
        value2,
        option
      );
    }
    const filteredOptions = (arrOptions || []).filter((option) => {
      if (!hasSearch) {
        return true;
      }
      const searchValue = `${search ?? ""}`.trim();
      if (!searchValue) {
        return true;
      }
      return string(option.searchLabel ?? option.label).toLowerCase().includes(searchValue.toLowerCase());
    });
    useEffect(() => {
      if (!visible || !selectedRef.current)
        return;
      setTimeout(() => {
        scrollIntoView(scrollRef.current, selectedRef.current, false);
      }, 100);
    }, [scrollRef, selectedRef, metrics?.maxHeight, visible]);
    function optionFocus(index) {
      if (index < 0 || index > filteredOptions.length - 1) {
        index = 0;
      }
      if (web) {
        optionsRef?.current?.[index]?.focus?.();
      }
      setActiveIndex(index);
    }
    async function handleOpen() {
      if (!buttonRef.current)
        return;
      optionFocus(arrOptions?.findIndex((item) => item.value == input.state));
      const { pageOffsetX, pageOffsetY, width, height } = await rect(buttonRef.current);
      const gutter = theme.spacing(3);
      const newMetrics = { left: pageOffsetX, width };
      if (pageOffsetY <= dimensions.height / 2) {
        newMetrics.top = Math.max(gutter, pageOffsetY + height);
      } else {
        newMetrics.bottom = Math.max(gutter, dimensions.height - pageOffsetY);
      }
      const sub = (newMetrics.top || 0) + (newMetrics.bottom || 0);
      newMetrics.maxHeight = Math.min(theme.rem(20), dimensions.height - gutter - sub);
      newMetrics.maxWidth = dimensions.width - gutter * 2;
      setMetrics(newMetrics);
      setVisible((current) => readOnly ? false : !current);
    }
    function handleFocus(event) {
      setFocused(true);
      dispatchEvent("focus", event, onFocus);
    }
    function handleBlur(event) {
      setFocused(visible || false);
      dispatchEvent("blur", event, onBlur);
    }
    function handleSelect(event, value2) {
      if (disabled || readOnly)
        return;
      input.setState(value2, event);
      setVisible(false);
      if (web) {
        focus();
      }
    }
    function handleSearch(event, value2) {
      optionsRef.current = [];
      setSearch(value2);
      setActiveIndex(0);
    }
    function handleKeyDown(event) {
      const { key } = event;
      const hasHandler = ["Escape", "ArrowUp", "ArrowDown", "Home", "End", "PageUp", "PageDown"].includes(key);
      let newIndex = activeIndex;
      if (key === "Escape") {
        return setVisible(false);
      }
      if (key === "ArrowUp") {
        newIndex -= 1;
      }
      if (key === "ArrowDown") {
        newIndex += 1;
      }
      if (key === "Home") {
        newIndex = 0;
      }
      if (key === "End") {
        newIndex = filteredOptions.length - 1;
      }
      if (key === "PageUp") {
        newIndex = Math.max(newIndex - 3, 0);
      }
      if (key === "PageDown") {
        newIndex = Math.min(newIndex + 3, filteredOptions.length - 1);
      }
      if (newIndex >= filteredOptions.length) {
        newIndex = 0;
      }
      if (newIndex < 0) {
        newIndex = filteredOptions.length - 1;
      }
      if (hasHandler) {
        event?.preventDefault?.();
        optionFocus(newIndex);
      }
    }
    style = [extract(spacings, rest), style];
    labelStyle = [input.error && { color: "error" }, labelStyle];
    contentStyle = [{ flex: 1 }, contentStyle];
    return /* @__PURE__ */ jsxs(
      BoxFactory,
      {
        "data-rbk-input": name,
        style,
        variants: { root: variants.root },
        onKeyDown: visible ? handleKeyDown : void 0,
        children: [
          Boolean(label) && /* @__PURE__ */ jsx(
            LabelFactory,
            {
              numberOfLines: 1,
              for: id,
              forRef: buttonRef,
              style: labelStyle,
              variants: { root: variants.label },
              onPress: native ? handleOpen : void 0,
              children: label
            }
          ),
          /* @__PURE__ */ jsx(
            ButtonFactory,
            {
              ref: buttonRef,
              color,
              endAddon: loading ? /* @__PURE__ */ jsx(LoadingFactory, { size: fontSize / theme.rem(), color }) : visible ? /* @__PURE__ */ jsx(ChevronUp, { svg, size: fontSize, color }) : /* @__PURE__ */ jsx(ChevronDown, { svg, size: fontSize, color }),
              ...rest,
              accessibility,
              id,
              size,
              disabled,
              variant: "outline",
              style: buttonStyle,
              contentStyle,
              onPress: handleOpen,
              onFocus: handleFocus,
              onBlur: handleBlur,
              children: typeof displayLabel === "string" ? /* @__PURE__ */ jsx(TextFactory, { numberOfLines: 1, w: "100%", children: displayLabel }) : displayLabel
            }
          ),
          Boolean(input.error) && typeof input.error === "string" && /* @__PURE__ */ jsx(TextFactory, { variant: "caption", style: errorStyle, variants: { root: variants.error }, children: input.error }),
          /* @__PURE__ */ jsx(BackdropFactory, { visible, style: { bg: "rgba(0, 0, 0, 0.2)" }, onPress: () => setVisible(false), children: /* @__PURE__ */ jsxs(CardFactory, { position: "absolute", p: 0, style: [{ overflow: "hidden" }, metrics], children: [
            hasSearch && /* @__PURE__ */ jsx(BoxFactory, { p: 1, children: /* @__PURE__ */ jsx(
              InputFactory,
              {
                color,
                size,
                value: search,
                onChange: handleSearch,
                startAddon: /* @__PURE__ */ jsx(MagnifyingGlass, { svg, size: fontSize, color })
              }
            ) }),
            visible && /* @__PURE__ */ jsx(
              ListFactory,
              {
                ref: scrollRef,
                rowHeight: baseSize + theme.spacing(1),
                renderDelay: 10,
                contentInset: 1,
                maxh: metrics?.maxHeight ?? 0,
                maxw: metrics?.maxWidth ?? 0,
                children: filteredOptions?.map((option, index) => {
                  const isSelected = option.value == selected?.value;
                  return /* @__PURE__ */ jsx(
                    ButtonFactory,
                    {
                      my: 0.5,
                      size,
                      variant: "text",
                      disabled: option.disabled,
                      color,
                      bg: isSelected ? theme.color(color, 0.1) : void 0,
                      style: { paddingHorizontal: spacing },
                      contentStyle,
                      onPress: (e) => handleSelect(e, option.value),
                      endAddon: /* @__PURE__ */ jsx(BoxFactory, { w: fontSize, pl: 1, children: isSelected && /* @__PURE__ */ jsx(Check, { svg, size: fontSize, color }) }),
                      ref: (el) => {
                        optionsRef.current[index] = el;
                        if (isSelected) {
                          selectedRef.current = el;
                        }
                      },
                      accessibility: {
                        // @ts-expect-error
                        role: native ? "menuitem" : "option",
                        value: { now: option.value },
                        state: { selected: isSelected }
                      },
                      children: typeof option.label === "string" ? /* @__PURE__ */ jsx(TextFactory, { children: option.label }) : option.label
                    },
                    option.value
                  );
                })
              }
            )
          ] }) })
        ]
      }
    );
  })
);
SelectFactory.displayName = "SelectFactory";

const SplitterFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Splitter;
    const { web } = rbkGlobal.mapping;
    const {
      direction,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const rootRef = useDefaultRef(ref);
    const childrenRefs = useRef([]);
    const currentChildRef = useRef({
      mult: 1,
      containerSize: 0,
      childSize: 0,
      x: 0,
      y: 0,
      $child: null
    });
    const isVertical = useMemo(() => direction === "vertical", [direction]);
    const { childArr, sizes } = useMemo(() => {
      const childArr2 = childrenize(children);
      const sizes2 = childArr2.map(
        (child) => (isVertical ? get("height", child?.props) ?? get("h", child?.props) ?? get("height", child?.props?.style) ?? get("h", child?.props?.style) : get("width", child?.props) ?? get("w", child?.props) ?? get("width", child?.props?.style) ?? get("w", child?.props?.style)) ?? "auto"
      );
      return { childArr: childArr2, sizes: sizes2 };
    }, [children, isVertical]);
    const { release, capture, isCapturing } = useDraggable({
      onDragMove: (event) => {
        event.preventDefault();
        const { $child, childSize, mult, x, y } = currentChildRef.current;
        const diff = isVertical ? (event.pageY - y) * mult : (event.pageX - x) * mult;
        setNativeStyle($child, {
          flexGrow: 0,
          flexBasis: Math.max(0, childSize + diff)
        });
      },
      onDragEnd: () => {
        release();
      }
    });
    useEffect(() => {
      if (!isCapturing)
        return;
      return () => {
        release();
      };
    }, [isCapturing, release]);
    const handleInit = useCallback(
      async (event, index) => {
        let mult = 1;
        let $child = childrenRefs.current[index];
        if (sizes[index] === "auto" && sizes[index + 1] !== "auto") {
          mult = -1;
          $child = childrenRefs.current[index + 1];
        }
        const containerMetrics = await rect(rootRef.current);
        const childMetrics = await rect($child);
        currentChildRef.current = {
          mult,
          containerSize: isVertical ? containerMetrics.height : containerMetrics.width,
          childSize: isVertical ? childMetrics.height : childMetrics.width,
          x: event.pageX,
          y: event.pageY,
          $child
        };
        capture();
      },
      [rootRef, isVertical, capture, sizes]
    );
    return /* @__PURE__ */ jsx(BoxFactory, { ref: rootRef, variants: { root: variants.root }, ...rest, children: childArr.map((child, index) => {
      const childProps = { ...child?.props };
      const childStyle = childProps?.style;
      const extracted = extract(isVertical ? ["height", "h"] : ["width", "w"], childProps, childStyle);
      const size = (isVertical ? extracted.height ?? extracted.h : extracted.width ?? extracted.w) ?? "auto";
      return /* @__PURE__ */ jsxs(Fragment, { children: [
        index > 0 && /* @__PURE__ */ jsx(
          BoxFactory,
          {
            direction: isVertical ? "column" : "row",
            onPressIn: (e) => handleInit(e, index - 1),
            platform: {
              native: {
                hitSlop: { top: 16, bottom: 16, left: 16, right: 16 }
              }
            },
            style: web && {
              cursor: isVertical ? "row-resize" : "col-resize",
              // #hack for hitSlop-like
              margin: -4,
              padding: 4
            },
            children: /* @__PURE__ */ jsx(
              DividerFactory,
              {
                color: isCapturing ? "info" : void 0,
                opacity: isCapturing ? 1 : void 0,
                vertical: !isVertical
              }
            )
          }
        ),
        /* @__PURE__ */ jsx(
          BoxFactory,
          {
            ref: ($el) => {
              childrenRefs.current[index] = $el;
            },
            component: child?.type,
            ...childProps,
            style: [
              childStyle,
              {
                flexGrow: size === "auto" ? 1 : 0,
                flexBasis: size === "auto" ? "auto" : size
              }
            ],
            variants: { root: variants.item }
          },
          index
        )
      ] }, child.key ?? index);
    }) });
  })
);
SplitterFactory.displayName = "SplitterFactory";

const SwitchFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Switch;
    const { native } = rbkGlobal.mapping;
    let {
      accessibility,
      checked,
      controlled,
      defaultChecked,
      disabled,
      error,
      id,
      label,
      name,
      readOnly,
      size,
      // Colors
      onColor,
      offColor,
      onThumbColor,
      offThumbColor,
      // Events
      onFocus,
      onBlur,
      onChange,
      onFormChange,
      // Styles
      variants,
      buttonStyle,
      thumbStyle,
      errorStyle,
      labelStyle,
      style,
      ...rest
    } = factory2(
      props,
      options
    );
    const buttonRef = useDefaultRef(ref);
    const input = useInput({
      name,
      value: checked,
      defaultValue: defaultChecked,
      error,
      controlled,
      editable: !disabled && !readOnly,
      unmask: (state) => Boolean(state),
      onChange: (event, value) => dispatchEvent("change", event, onChange, value),
      onFormChange
    });
    id = useHtmlId(id);
    size = getSize(size) - 0.375;
    const baseSize = theme.rem(size);
    const spacing = baseSize * 0.1;
    const thumbSize = baseSize - spacing * 2;
    const focus = useCallback(() => buttonRef?.current?.focus?.(), [buttonRef]);
    const blur = useCallback(() => buttonRef?.current?.blur?.(), [buttonRef]);
    const clear = useCallback(() => input.clear(), [input]);
    const reset = useCallback(() => input.reset(), [input]);
    const isFocused = useCallback(
      () => Boolean(buttonRef?.current?.isFocused?.()) || buttonRef?.current === document?.activeElement,
      [buttonRef]
    );
    function dispatchEvent(type, event, handler, value) {
      if (typeof handler !== "function")
        return;
      value = typeof value === "undefined" ? input.state : value;
      const form = input.form;
      const target = buttonRef.current;
      return handler?.(
        {
          ...event,
          handler: "RbkInputEvent",
          type,
          value,
          name,
          form,
          focus,
          blur,
          clear,
          reset,
          isFocused,
          target
        },
        value
      );
    }
    function handleFocus(event) {
      dispatchEvent("focus", event, onFocus);
    }
    function handleBlur(event) {
      dispatchEvent("blur", event, onBlur);
    }
    function handleChange(event) {
      if (disabled || readOnly)
        return;
      input.setState(!input.state, event);
    }
    buttonStyle = [
      {
        h: baseSize,
        w: (thumbSize + spacing) * 2
      },
      buttonStyle
    ];
    thumbStyle = [
      {
        h: thumbSize,
        w: thumbSize,
        borderRadius: thumbSize / 2,
        bg: input.state ? onThumbColor : offThumbColor
      },
      thumbStyle
    ];
    return /* @__PURE__ */ jsxs(Fragment$1, { children: [
      /* @__PURE__ */ jsxs(BoxFactory, { "data-rbk-input": name, style, variants: { root: variants.root }, children: [
        /* @__PURE__ */ jsx(
          ButtonFactory,
          {
            ref: buttonRef,
            ...rest,
            circular: true,
            id,
            onFocus: handleFocus,
            onBlur: handleBlur,
            onPress: handleChange,
            color: input.state ? onColor : offColor,
            disabled,
            size,
            variant: "solid",
            style: buttonStyle,
            variants: { root: variants.button },
            contentStyle: {
              justifyContent: "center",
              padding: spacing,
              w: "100%",
              h: "100%"
            },
            accessibility: deepmerge(accessibility, {
              state: { checked: input.state }
            }),
            children: /* @__PURE__ */ jsx(
              BoxFactory,
              {
                style: thumbStyle,
                variants: { root: variants.thumb },
                animation: {
                  duration: 100,
                  timing: "ease-out",
                  from: { marginLeft: input.state ? 0 : thumbSize },
                  to: { marginLeft: input.state ? thumbSize : 0 }
                }
              }
            )
          }
        ),
        typeof label === "string" ? /* @__PURE__ */ jsx(
          LabelFactory,
          {
            for: id,
            forRef: buttonRef,
            style: labelStyle,
            variants: { root: variants.label },
            onPress: native ? handleChange : void 0,
            children: label
          }
        ) : label
      ] }),
      Boolean(input.error) && typeof input.error === "string" && /* @__PURE__ */ jsx(TextFactory, { variant: "caption", style: errorStyle, variants: { root: variants.error }, children: input.error })
    ] });
  })
);
SwitchFactory.displayName = "SwitchFactory";

const TableFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Table;
    let {
      border,
      columns,
      rows,
      // Styles,
      variants,
      style,
      ...rest
    } = factory2(props, options);
    const width = `${100 / columns?.length}%`;
    const renderContent = (child, data = void 0, bold = false) => {
      if (child) {
        if (isValidElement(child)) {
          return child;
        }
        if (typeof child === "function") {
          child = child(data);
        }
        if (["string", "number"].includes(typeof child)) {
          child = /* @__PURE__ */ jsx(TextFactory, { bold, children: child });
        }
      }
      return child ?? null;
    };
    const buildStyle = (column, borderTop = false, borderLeft = false) => {
      return [
        {
          border,
          p: theme.shape.gap / 2,
          borderBottomWidth: 0,
          borderRightWidth: 0,
          width,
          flexGrow: 0,
          flexShrink: 0
        },
        !borderTop && { borderTopWidth: 0 },
        !borderLeft && { borderLeftWidth: 0 },
        column.style
      ];
    };
    style = [{ border }, style];
    return /* @__PURE__ */ jsxs(BoxFactory, { ref, style, variants: { root: variants.root }, ...rest, children: [
      /* @__PURE__ */ jsx(BoxFactory, { row: true, noWrap: true, children: columns?.map((column, index) => /* @__PURE__ */ jsx(BoxFactory, { style: buildStyle(column, false, index > 0), children: renderContent(column.header, column, true) }, index)) }),
      rows?.map((row, rowIndex) => /* @__PURE__ */ jsx(BoxFactory, { row: true, noWrap: true, children: columns.map((column, index) => /* @__PURE__ */ jsx(BoxFactory, { style: buildStyle(column, true, index > 0), children: renderContent(column.content, row) }, index)) }, rowIndex))
    ] });
  })
);
TableFactory.displayName = "TableFactory";

const TabsFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Tabs;
    const {
      alignment,
      circular,
      color,
      size,
      tabs,
      value = 0,
      variant = "group",
      // Events
      onChange,
      // Styles
      variants,
      contentStyle,
      buttonStyle,
      activeStyle,
      ...rest
    } = factory2(props, options);
    const $tabs = tabs?.map((tab, index) => {
      const { value: tabValue = index, style: tabStyle, ...rest2 } = tab;
      const isActive = value === tabValue;
      rest2.accessibility = deepmerge({ role: "tab", state: { selected: isActive } }, rest2.accessibility);
      return /* @__PURE__ */ jsx(
        ButtonFactory,
        {
          variant: isActive ? "solid" : variant === "card" ? "text" : "outline",
          color,
          size,
          circular,
          labelStyle: { px: 1.5 },
          ...rest2,
          style: [buttonStyle, tabStyle, isActive && activeStyle],
          variants: { root: [variants.button, isActive && variants.active] },
          onPress: (e) => onChange?.(e, tabValue)
        },
        tabValue
      );
    });
    return /* @__PURE__ */ jsx(BoxFactory, { ref, ...rest, variants: { root: variants.root }, children: variant === "nav" ? /* @__PURE__ */ jsx(ButtonGroupFactory, { contentStyle, variants: { content: variants.content }, children: $tabs }) : /* @__PURE__ */ jsx(ScrollableFactory, { direction: "horizontal", children: /* @__PURE__ */ jsx(BoxFactory, { style: contentStyle, variants: { root: variants.content }, children: $tabs }) }) });
  })
);
TabsFactory.displayName = "TabsFactory";

function postScroll($el) {
  scrollToEnd($el, false);
  setTimeout(() => scrollToEnd($el, false), 100);
}
function postFocus($el) {
  $el?.focus();
  setTimeout(() => $el?.focus(), 100);
}
const TerminalFactory = React.memo(
  forwardRef(({ ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Terminal;
    const { web, native, ios, Input } = rbkGlobal.mapping;
    const {
      prompt,
      commands: commandsProp,
      version,
      welcomeMessage,
      // Styles
      variants,
      ...rest
    } = factory2(props, options);
    const scrollRef = useDefaultRef(ref);
    const inputRef = useRef();
    const [isLocked, setIsLocked] = useState(false);
    const [arrowIndex, setArrowIndex] = useState(0);
    const [text, setText] = useState("");
    const [messages, setMessages] = useState(
      welcomeMessage ? [{ body: welcomeMessage }] : []
    );
    if (native) {
      rest.contentInsetAdjustmentBehavior = "never";
      rest.onContentSizeChange = () => postScroll(scrollRef.current);
    }
    useEffect(() => {
      postScroll(scrollRef.current);
    }, [scrollRef, messages, isLocked]);
    const addMessage = useCallback((message, options2 = {}) => {
      setMessages((current) => [...current, { ...options2, body: `${message ?? ""}` }]);
    }, []);
    const terminal = useMemo(
      () => ({
        // exec: (command) => execCommand(command),
        lock: () => setIsLocked(true),
        unlock: () => setIsLocked(false),
        focus: () => inputRef.current?.focus(),
        // Messages
        clear: () => setMessages([]),
        log: (message) => addMessage(message),
        info: (message) => addMessage(message, { color: "info" }),
        warning: (message) => addMessage(message, { color: "warning" }),
        success: (message) => addMessage(message, { color: "success" }),
        error: (message) => addMessage(message, { color: "error" })
      }),
      [inputRef, addMessage]
    );
    const commands = useMemo(() => {
      const allCommands = {
        cls: "clear",
        clear: () => setMessages([]),
        date: () => (/* @__PURE__ */ new Date()).toString(),
        ping: () => "pong",
        echo: ({ args }) => args ?? "",
        version: () => version,
        sleep: ({ args }) => new Promise((resolve) => setTimeout(resolve, args)),
        history: ({ args }) => {
          addMessage("");
          addMessage("  Id CommandLine", { color: "success" });
          addMessage("---- -----------", { color: "success" });
          messages.slice(0, -1).filter(({ prompt: prompt2, body }) => prompt2 && body?.trim()).filter((_, index) => !args || Number(args) === index + 1).forEach(({ body }, index) => {
            addMessage(`${args || index + 1}`.padStart(4, " ") + " " + body);
          });
          addMessage("");
        },
        ...commandsProp
      };
      allCommands.help = () => {
        addMessage(``);
        addMessage(`Running version ${version ?? "?"}`);
        addMessage(``);
        addMessage(`Available commands:`);
        addMessage(``);
        Object.keys(allCommands).sort().forEach((cmd) => addMessage(`  ${cmd}`));
        addMessage(``);
      };
      return allCommands;
    }, [commandsProp, version, addMessage, messages]);
    const execCommand = useCallback(
      async (command) => {
        const split = command.split(" ");
        command = split.shift() ?? "";
        const args = split.join(" ");
        let callback = commands?.[command];
        if (typeof callback === "string") {
          callback = commands?.[callback];
        }
        if (typeof callback === "function") {
          try {
            const response = await callback({ command, args, ...terminal });
            if (["number", "string"].includes(typeof response)) {
              addMessage(response);
            }
          } catch (err) {
            addMessage(`Error: ${err.message}`, { color: "error" });
          }
        } else {
          addMessage(`Unknown command: ${command}`, { color: "error" });
        }
      },
      [commands, terminal, addMessage]
    );
    const handleChange = useCallback(
      (event) => {
        setText(`${event?.value ?? event?.target?.value ?? event?.nativeEvent?.text ?? ""}`);
        postScroll(scrollRef.current);
      },
      [scrollRef]
    );
    const handleKeyDown = useCallback(
      async (event) => {
        postFocus(inputRef.current);
        const value = `${event?.target?.value ?? event?.nativeEvent?.text ?? event?.value ?? ""}`.trim();
        if (event.key === "ArrowUp") {
          const prevIndex = arrowIndex - 1;
          const prevMessage = messages.filter(({ prompt: prompt2, body }) => prompt2 && body).at(prevIndex);
          if (typeof prevMessage !== "undefined") {
            setText(prevMessage.body);
            setArrowIndex(prevIndex);
          }
        }
        if (event.key === "ArrowDown") {
          const nextIndex = arrowIndex + 1;
          const nextMessage = messages.filter(({ prompt: prompt2, body }) => prompt2 && body).at(nextIndex);
          if (typeof nextMessage !== "undefined") {
            setText(nextMessage.body);
            setArrowIndex(nextIndex);
          }
        }
        if (event.ctrlKey) {
          if (event.key.toLowerCase() === "l") {
            event.preventDefault();
            setMessages([]);
            setIsLocked(false);
          }
          if (event.key.toLowerCase() === "c" && isLocked) {
            addMessage("^C", { prompt });
            setIsLocked(false);
          }
        }
        if ((event.key === "Enter" || event.keyCode === 13) && !isLocked) {
          addMessage(value, { prompt });
          setArrowIndex(0);
          setText("");
          if (value) {
            setIsLocked(true);
            await execCommand(value);
            setIsLocked(false);
          }
        }
        postFocus(inputRef.current);
      },
      [isLocked, arrowIndex, messages, addMessage, prompt, execCommand]
    );
    const handleSubmit = useCallback(
      async (event) => handleKeyDown({ ...event, key: "Enter", value: text }),
      [text, handleKeyDown]
    );
    const textStyle = {
      fontFamily: web ? '"Courier New", monospace' : ios ? "Courier New" : "monospace",
      fontSize: "1rem",
      lineHeight: "1rem"
    };
    return /* @__PURE__ */ jsx(ScrollableFactory, { ref: scrollRef, variants: { root: variants.root }, ...rest, children: /* @__PURE__ */ jsxs(BoxFactory, { flex: true, p: 2, onPress: () => postFocus(inputRef.current), children: [
      messages.map((message, index) => /* @__PURE__ */ jsx(BoxFactory, { mb: 1, children: /* @__PURE__ */ jsxs(TextFactory, { color: message.color ?? "text", style: textStyle, children: [
        message.prompt ? `${message.prompt.replace(/ /g, "\xA0")} ` : "",
        message.body?.replace(/ /g, "\xA0"),
        "\xA0"
      ] }) }, index)),
      /* @__PURE__ */ jsxs(BoxFactory, { row: true, noWrap: true, invisible: isLocked, children: [
        /* @__PURE__ */ jsxs(TextFactory, { style: textStyle, children: [
          prompt ? prompt.replace(/ /g, "\xA0") : "",
          "\xA0"
        ] }),
        /* @__PURE__ */ jsx(BoxFactory, { flex: true, children: /* @__PURE__ */ jsx(
          BoxFactory,
          {
            noRootStyles: true,
            ref: inputRef,
            component: Input,
            readOnly: isLocked,
            value: text,
            onChange: handleChange,
            onKeyDown: handleKeyDown,
            autoComplete: "off",
            autoCapitalize: "none",
            autoCorrect: web ? "off" : false,
            platform: {
              web: {
                enterKeyHint: "send"
              },
              native: {
                returnKeyType: "send",
                blurOnSubmit: false,
                onSubmitEditing: handleSubmit
              }
            },
            style: {
              ...textStyle,
              backgroundColor: "trans",
              borderWidth: 0,
              color: "text.primary",
              height: "1rem",
              margin: 0,
              padding: 0,
              textDecorationLine: "none",
              width: "100%",
              web: {
                backgroundImage: "none",
                boxShadow: "none",
                outline: "0 !important",
                "-moz-appearance": "textfield",
                appearance: "none",
                "&::-webkit-outer-spin-button,::-webkit-inner-spin-button": {
                  "-webkit-appearance": "none",
                  appearance: "none"
                }
              }
            }
          }
        ) })
      ] })
    ] }) });
  })
);
TerminalFactory.displayName = "TerminalFactory";

function CaretDown({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M213.66,101.66l-80,80a8,8,0,0,1-11.32,0l-80-80A8,8,0,0,1,48,88H208a8,8,0,0,1,5.66,13.66Z" }) });
}

function CaretLeft({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M168,48V208a8,8,0,0,1-13.66,5.66l-80-80a8,8,0,0,1,0-11.32l80-80A8,8,0,0,1,168,48Z" }) });
}

function CaretRight({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M181.66,133.66l-80,80A8,8,0,0,1,88,208V48a8,8,0,0,1,13.66-5.66l80,80A8,8,0,0,1,181.66,133.66Z" }) });
}

function CaretUp({ svg, color = "#000000", size = 16 }) {
  const { Svg, Path } = svg;
  return /* @__PURE__ */ jsx(Svg, { width: size, height: size, fill: color, viewBox: "0 0 256 256", children: /* @__PURE__ */ jsx(Path, { d: "M215.39,163.06A8,8,0,0,1,208,168H48a8,8,0,0,1-5.66-13.66l80-80a8,8,0,0,1,11.32,0l80,80A8,8,0,0,1,215.39,163.06Z" }) });
}

const TooltipFactory = React.memo(
  forwardRef(({ children, ...props }, ref) => {
    const theme = useTheme();
    const options = theme.components.Tooltip;
    const { native, svg, Button } = rbkGlobal.mapping;
    const portalId = useHtmlId();
    let {
      accessibility,
      color,
      delay,
      offset,
      position,
      title,
      visible,
      // Styles
      variants,
      labelStyle,
      style,
      ...rest
    } = factory2(props, options);
    color = theme.color(color);
    const rootRef = useRef();
    const timeoutRef = useRef(null);
    const [shown, setShown] = usePropState(visible, false);
    const [rootRect, setRootRect] = useState({});
    const isControlled = typeof visible === "boolean";
    style = [
      shown && options?.variants?.visible?.true?.root,
      style,
      {
        top: rootRect.pageOffsetY ?? 0,
        left: rootRect.pageOffsetX ?? 0,
        width: rootRect.width ?? 0,
        height: rootRect.height ?? 0
      }
    ];
    labelStyle = [
      {
        backgroundColor: color,
        color: theme.contrast(color),
        shadow: 1
      },
      labelStyle
    ];
    const handleTooltipShow = useCallback(async () => {
      const rootRect2 = await rect(rootRef.current);
      setRootRect(rootRect2);
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
      if (!isControlled) {
        timeoutRef.current = setTimeout(() => {
          setShown(true);
        }, delay);
      }
    }, [timeoutRef, delay, isControlled, setShown]);
    const handleTooltipHide = useCallback(() => {
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
      if (!isControlled) {
        setShown(false);
      }
    }, [isControlled, setShown]);
    return /* @__PURE__ */ jsxs(
      BoxFactory,
      {
        ref: rootRef,
        component: native ? Button : null,
        accessibility: { label: title, ...Object(accessibility) },
        style: { position: "relative" },
        platform: {
          web: {
            onPointerEnter: handleTooltipShow,
            onPointerLeave: handleTooltipHide
          },
          native: {
            onPressIn: handleTooltipShow,
            onPressOut: handleTooltipHide
            // TODO: tooltip on nested pressable child
            // onStartShouldSetResponderCapture
          }
        },
        children: [
          children,
          createPortal(
            portalId,
            /* @__PURE__ */ jsx(BoxFactory, { ref, style, variants: { root: variants.root }, ...rest, children: /* @__PURE__ */ jsx(
              BoxFactory,
              {
                style: [
                  {
                    position: "absolute",
                    h: 80,
                    w: 240,
                    p: offset
                  },
                  position === "top" && {
                    bottom: "100%",
                    l: "50%",
                    ml: "-120px",
                    alignItems: "center",
                    justifyContent: "end"
                  },
                  position === "bottom" && {
                    top: "100%",
                    l: "50%",
                    ml: "-120px",
                    alignItems: "center",
                    justifyContent: "start"
                  },
                  position === "left" && {
                    right: "100%",
                    t: "50%",
                    mt: "-40px",
                    alignItems: "end",
                    justifyContent: "center"
                  },
                  position === "right" && {
                    left: "100%",
                    t: "50%",
                    mt: "-40px",
                    alignItems: "start",
                    justifyContent: "center"
                  }
                ],
                children: /* @__PURE__ */ jsxs(
                  BoxFactory,
                  {
                    noWrap: true,
                    position: "relative",
                    direction: ["left", "right"].includes(position) ? "row" : "column",
                    children: [
                      position === "bottom" && /* @__PURE__ */ jsx(BoxFactory, { center: true, mb: "-6px", children: /* @__PURE__ */ jsx(CaretUp, { svg, color, size: 16 }) }),
                      position === "right" && /* @__PURE__ */ jsx(BoxFactory, { center: true, order: 0, mr: "-6px", children: /* @__PURE__ */ jsx(CaretLeft, { svg, color, size: 16 }) }),
                      /* @__PURE__ */ jsx(TextFactory, { style: labelStyle, variants: { root: variants.label }, children: title }),
                      position === "top" && /* @__PURE__ */ jsx(BoxFactory, { center: true, mt: "-6px", children: /* @__PURE__ */ jsx(CaretDown, { svg, color, size: 16 }) }),
                      position === "left" && /* @__PURE__ */ jsx(BoxFactory, { center: true, ml: "-6px", children: /* @__PURE__ */ jsx(CaretRight, { svg, color, size: 16 }) })
                    ]
                  }
                )
              }
            ) })
          )
        ]
      }
    );
  })
);
TooltipFactory.displayName = "TooltipFactory";

export { AvatarFactory, BackdropFactory, BadgeFactory, BoxFactory, ButtonFactory, ButtonGroupFactory, CalendarFactory, CardFactory, CarouselFactory, CheckboxFactory, CollapseFactory, DividerFactory, DrawerFactory, DropdownFactory, FormFactory, GridFactory, GrowBoxFactory, ImageFactory, InputBaseFactory, InputDateFactory, InputFactory, InputPinFactory, LabelFactory, LinkFactory, ListFactory, ListItemFactory, LoadingFactory, ModalFactory, OutlineFactory, Platform, ProgressFactory, ReactBulk, ResizableFactory, ScrollableFactory, SelectFactory, SplitterFactory, SwitchFactory, TableFactory, TabsFactory, TerminalFactory, TextFactory, TooltipFactory, aliases, base as baseEvent, bindings, boxSizeProps, childrenize, clone, clsx, createMeta, createPortal, createTheme, crypt, css, customSpacings, customStyleProps, deepmerge, defined, event, extract, flexAlignProps, flexChildAlignProps, flexChildProps, flexContainerAlignProps, flexContainerProps, flexProps, gapSpacings, get, getFullHeight, getFullSize, getSize, rbkGlobal as global, jss, keyboard as keyboardEvent, merge, notPxProps, omit, only, pick, pointer as pointerEvent, rect, reference, remove, scrollIntoView, scrollTo, scrollToEnd, scrollToStart, setNativeStyle, sheet, sleep, spacings, stdout, string, styleProps, transformProps, useAnimation, useBreakpoints, useDefaultRef, useDeferredValue, useDependentState, useDraggable, useForm, useHtmlId, useInput, usePropState, useTheme, useToaster, uuid };
